#include "authwindow.h"


authorizeWindow::authorizeWindow(userEventType _mode):
requestDialog::requestDialog(FALSE, NULL, NULL, NULL, NULL, "auth.png", (requestFunction)this->cb_notifyFunction, this)
{		
	mode = _mode;
	switch(mode)
	{
		case UE_AUTHORIZATION:
			topicText = g_strdup_printf("<b>%s</b>", tr("Contact list request from %s"));
			infoText = g_strdup_printf("<small>%s</small>", tr("The user would like to add you to her/his contact list."));
			commentText = g_strdup(tr("implement comment!"));
			preTitle = g_strdup(tr("Contact list request from %s"));
			wType = UE_AUTHORIZATION;
			isServer = FALSE;
			break;
		
		
		case UE_AUTHORIZATIONREQUEST:
			topicText = g_strdup_printf("<b>%s</b>", tr("Ask %s for authorization"));
			infoText = g_strdup_printf("<small>%s</small>", tr("You can ask the user for his admission to add him to your contact list with this form."));
			preTitle = g_strdup(tr_NOOP("Ask %s for authorization"));
			wType = UE_AUTHORIZATIONREQUEST;
			isServer = TRUE;		
			break;
		
		
		case UE_AUTHORIZATIONGRANT:
			topicText = g_strdup_printf("<b>%s</b>", tr("Authorize %s"));
			infoText = g_strdup_printf("<small>%s</small>", tr("Allow the user to add you to his contact list."));
			preTitle = g_strdup(tr_NOOP("Authorize %s"));
			setRequestName(tr_NOOP("Admission"));
			wType = UE_AUTHORIZATIONGRANT;
			isServer = TRUE;
			break;
		
		
		default:
			fprintf(stderr, "authorizeWindow::authorizeWindow(): unknown authorize mode\n");
			break;
	}
	
	
}



authorizeWindow::~authorizeWindow()
{
	delete manager;
}



GtkWidget* authorizeWindow::createWindowContent()
{
	
	return requestDialog::createWindowContent();
}



void authorizeWindow::destroyWindowContent()
{
	requestDialog::destroyWindowContent();
}


void authorizeWindow::focusWindow()
{
	requestDialog::focusWindow();
}



gboolean authorizeWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	if (isServer)
		switch(command)
		{
			case UC_EVENT_RECEIVED:
				if (!isServer && !commentText)
					gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(comment)), 
											 ((messageEventInfo*)info)->text, 
											 strlen(((messageEventInfo*)info)->text));
				break;
				
			// hopefully licq will soon support a feedback on sending authorization requests
			case UC_EVENT_FINISHED:
				destroyWindow();
				delete this;
				break;
	}
	
	return requestDialog::eventCallback(command, lastResult, info);
}



void authorizeWindow::cb_notifyFunction(gint result, gpointer info, authorizeWindow* self)
{
	gchar* charData;
	
	switch(self->mode)
	{
		case UE_AUTHORIZATIONREQUEST:
			switch(result)
			{
				case REQUEST_CANCELLED:
					self->stopWaitForRequestAnswer();
					self->manager->cancelInitatedEvent();
					break;
				
				case REQUEST_CLOSED:
					((IMAuthorizationManager*)self->manager)->cancelInitatedEvent();
					break;
			
				case REQUEST_ACCEPTED:
					((IMAuthorizationManager*)self->manager)->sendRequest((gchar*)info, NULL);
					self->destroyWindow();
					delete self;
				
					// self->waitForRequestAnswer();
					break;
			}
			break;
			
		case UE_AUTHORIZATION:
			// we have a remote request
			switch (result)
			{
				case REQUEST_CLOSED:
					break;
		
				case REQUEST_ACCEPTED:
					charData = self->askForReason(tr("Authorize"), tr("Please enter a short comment for the authorization."));
					((IMAuthorizationManager*)self->manager)->answerRemoteRequest(TRUE, charData);
					g_free(charData);
					self->destroyWindow(); 
					delete self;
					break;
		
				case REQUEST_REFUSED:
					((IMAuthorizationManager*)self->manager)->answerRemoteRequest(FALSE, (gchar*)info);
					self->destroyWindow();
					delete self;
					break;
			}
			break;

		case UE_AUTHORIZATIONGRANT:
			if (result == REQUEST_ACCEPTED)
			{
				((IMAuthorizationManager*)self->manager)->answerRemoteRequest(TRUE, (gchar*)info);				
				self->destroyWindow();
				delete self;				
			}
			
		default: break;
	}
}
