/***************************************************************************
 *            chatwindow.cpp
 *
 *  Tue Feb 15 04:13:32 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "chatwindow.h"



static const guchar chatColors[][3] =
{
  {0x00, 0x00, 0x00},
  {0x80, 0x00, 0x00},
  {0x00, 0x80, 0x00},
  {0x80, 0x80, 0x00},
  {0x00, 0x00, 0x80},
  {0x80, 0x00, 0x80},
  {0x00, 0x80, 0x80},
  {0x80, 0x80, 0x80},
  {0xC0, 0xC0, 0xC0},
  {0xFF, 0x00, 0x00},
  {0x00, 0xFF, 0x00},
  {0xFF, 0xFF, 0x00},
  {0x00, 0x00, 0xFF},
  {0xFF, 0x00, 0xFF},
  {0x00, 0xFF, 0xFF},
  {0xFF, 0xFF, 0xFF}
};

#define NUM_COLORS sizeof(chatColors)/3




chatWindow::chatWindow(gboolean _isServer)
{
	isIRCStyle = FALSE;
	isServer = _isServer;
	ircView = NULL;
	ownerView = NULL;
	
	request = NULL;
	chatViews = NULL;
	
	multipartyWindow = NULL;
	
	if (isServer)
		wType = UE_CHATREQUEST;
	else
		wType = UE_CHAT;
}



chatWindow::~chatWindow()
{
	
	if (request)
	{
		request->destroyWindow();
		delete request;
		request = NULL;
	} 
	
	delete manager;
}




void chatWindow::focusWindow()
{
	if (request)
		request->focusWindow();
	else
		basicWindow::focusWindow();
}



void chatWindow::createRequestDialog(chatEventInfo* event)
{
	if (isServer)
		request = new requestDialog(TRUE, createSpecialWidget(TRUE), 
									tr("Start a chat with %s"),
									tr("You can open a new chat room or invite the user into an existing one."),
									NULL,
									"chat.png",
									(requestFunction)this->cb_requestCallback,
									this);		
	else
		request = new requestDialog(FALSE, createSpecialWidget(event->port!=0),
									tr("Start a chat with %s"),
									tr("The user would like to invite you to a chat session."),
									event->text,
									"chat.png",
									(requestFunction)this->cb_requestCallback,
									this);
						
	
	request->preTitle = g_strdup(tr("Start a chat with %s"));

	request->manager = manager;
	request->createWindow();	
}




	
void chatWindow::rebuildViewMode()
{
	GList	*search,
			*children;
	chatWindowView	*aView;
	
	
	if (isIRCStyle)
	{
		// destroy all children
		search = chatViews;
		
		
		while(search)
		{
			aView = (chatWindowView*)search->data;
			aView->destroyChatView();
			search = search->next;
		} 

		children = NULL;
		children = gtk_container_get_children(GTK_CONTAINER(mainChatBox));
			
		if (children)
		{
			gtk_container_remove(GTK_CONTAINER(mainChatBox), (GtkWidget*)children->data);
			g_list_free(children);
		}

				
		gtk_container_add(GTK_CONTAINER(mainChatBox), ircView->createChatView());
		gtk_widget_show_all(mainChatBox);
		gtk_widget_show_all(userListFrame);
		
		// delete the content of the local input except the started line
		ownerView->clearTextBuffer();
		
		
	} else
	{ 
		ircView->destroyChatView();
		
		children = gtk_container_get_children(GTK_CONTAINER(mainChatBox));		
		if (children)
		{
			gtk_container_remove(GTK_CONTAINER(mainChatBox), (GtkWidget*)children->data);
			g_list_free(children);
		}

		
		// rebuild the chat windows
		search = chatViews;
		while(search)
		{
			addChatView((chatWindowView*)search->data);
			search = search->next;
		}
		
		gtk_widget_show_all(mainChatBox);
		gtk_widget_hide(userListFrame);	

		// show the whole history again
		ownerView->clearTextBuffer();		
	}	
}




		
void chatWindow::startChat()
{
	
}




chatWindowView* chatWindow::findViewByID(gchar* id)
{
	GList *search;

	if (!id)
		return NULL;		
	
	search = chatViews;
	
	while(search)
	{
		if (!strcmp(id, ((chatWindowView*)search->data)->userID))
			return (chatWindowView*)search->data;
		
		search = search->next;
	}
	
	return NULL;
}

	
	
GtkWidget* chatWindow::createWindowContent()
{
	GtkWidget	*mainVBox,
				*buttonBar,
				*panedBox,
				*paned2;
	gchar		*titleText;
	

	// create the two static views
	ownerView = new chatWindowLocalView(this);
	
	ircView = new chatWindowIRCView(this);

	// create users list and add the owner
	createUserList();		
	addViewToList((chatWindowView*)ownerView);
	
	// create the button bar
	buttonBar = createButtonbar();
	
	mainChatBox = gtk_event_box_new();
	
	paned2 = gtk_vpaned_new();
	gtk_paned_pack1(GTK_PANED(paned2), mainChatBox, TRUE, TRUE);
	gtk_paned_pack2(GTK_PANED(paned2), ownerView->createChatView(), FALSE, TRUE);
		
	// create the paned box
	panedBox = gtk_hpaned_new();
	gtk_paned_pack1(GTK_PANED(panedBox), paned2, TRUE, TRUE);
	gtk_paned_pack2(GTK_PANED(panedBox), userListFrame, FALSE, TRUE); 
	

	mainVBox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(mainVBox), 6);
	gtk_box_pack_start(GTK_BOX(mainVBox), panedBox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), gtk_hseparator_new(), FALSE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(mainVBox), buttonBar, FALSE, TRUE, 2);
		
	
	chatVBox = NULL;
	lastFreeCVPlace = NULL;
	chatViews = NULL;
	
		
	setWindowSize(450, 580);
	titleText = g_strdup_printf("Chat session ...");
	setWindowTitle(titleText);
	g_free(titleText);

	// send font information
	ownerView->updateFontInformation(CF_ALL);

	
	// addChatView((chatWindowView*)new chatWindowRemoteView("Hallomann", "123", this));

	return mainVBox;	
}



void chatWindow::destroyWindowContent()
{
	GList	*search;
	chatWindowView* aView;
	
	delete ircView;
	delete ownerView;
	
	search = chatViews;
		
	while(search)
	{
		aView = (chatWindowView*)search->data;
		aView->destroyChatView();
		delete aView;

		search = search->next;
	}	
	
	g_list_free(chatViews);	
}



void chatWindow::windowActivated()
{
	if (!isIRCStyle && GTK_WIDGET_VISIBLE(userListFrame))
		gtk_widget_hide(userListFrame);
}



void chatWindow::windowDeactivated()
{
	
}




gboolean chatWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	chatWindowRemoteView* view;
	chatEventInfo* event;
	charsetType* ch;
	gchar		*line, *toKick;
	GList		*search;
	
	
	switch(command)
	{
		case UC_EVENT_RECEIVED:
			// createWindow();
			if (!isServer && !request)
				createRequestDialog((chatEventInfo*)info);
			break;
			
		case EVENT_REQUEST_ACCEPTED:
			request->destroyWindow();
			delete request;
			request = NULL;
			// there is no other window that should overtake this chat as a multiparty session
			if (!multipartyWindow)
				createWindow();
			else
			{
				// there is one. Let this window do the work and destroy our instance
				multipartyWindow->focusWindow();
				delete this;
			}
			break;
		
		case EVENT_REQUEST_REFUSED:
			u_showAlertMessage(tr("Refused"), tr("Your request was refused!"), GTK_STOCK_DIALOG_INFO);
			request->destroyWindow();
			delete request;
			request = NULL;
			delete this;
			break;
		
		case UC_EVENT_FINISHED:
			if (lastResult != EVENT_RESULT_SUCCESS && lastResult != EVENT_RESULT_ACKED)
			{
				uu_showSendErrorMessage(tr("Could not send chat request"), tr("Sending of the request to start a chat failed"), lastResult, manager->parentUser->owner->info->status);
				request->stopWaitForRequestAnswer();
			}
			break;
			
		case UC_CHATEVENT:
			event = (chatEventInfo*)info;
		
			if (lastResult == CEVENT_CONNECT)
			{
				// show a warning when there are too much users and we're not in IRC-mode
				if (g_list_length(chatViews) == 5)
				{
					if (!isIRCStyle)
					{
						u_showYNRememberDialog(0, tr("Too much chatters"), gtk_label_new(tr("There are many participants in the chat\nToo keep the display lucid the view\nswitches to ICQ-style now!")), 
										   tr("Okay"), GTK_STOCK_OK, NULL);
						isIRCStyle = TRUE;
						rebuildViewMode();
					}
					if (ownerView->ircButton)
						gtk_widget_set_sensitive(ownerView->ircButton, FALSE);
				}
				
				// create the view
				view = new chatWindowRemoteView(event->userName, event->userID, this);
				chatViews = g_list_append(chatViews, view);
								
				// find its charset
				ch = charset_findCharsetByNumber(event->encoding);
				if (!ch)
					u_showAlertMessage(tr("character set error"), tr("The character set of the user couldn't be found.\nMessages from this user might look strange..."), GTK_STOCK_DIALOG_ERROR);
				else
					view->charset = ch->id;
				
				// add it to the list
				addViewToList((chatWindowView*)view);	
				
				// set up its font context 
				view->createFontContext();
				view->updateStyle(event);
				
				// create its window if neccessary
				if (!isIRCStyle)
					addChatView((chatWindowView*)view);	
				
				ircView->insertLine(view->alias, tr("%s has entered the chat\n"), NULL, TRUE);
			} else
			{
				view = (chatWindowRemoteView*)findViewByID(event->userID);
				
				if (!view)
					break;
				switch(lastResult)
				{	
					
					case CEVENT_KICKFAIL:
					case CEVENT_KICKPASS:
						if (lastResult == CEVENT_KICKFAIL)
							line = 	tr_NOOP("%s will not be kicked out of the chat. The majority wasn't reached!");
						else
							line = 	tr_NOOP("%s was kicked out of the chat!");						
					
						if (!isIRCStyle)
						{
							toKick = g_strdup_printf(tr(line), view->alias);
							u_showAlertMessage(tr("Kick a user!"), toKick, GTK_STOCK_DIALOG_INFO);
							g_free(toKick);
						} else
						ircView->insertLine(view->alias, line, NULL, TRUE);
						break;
						
												
					
					case CEVENT_KICK:
						// find the name to the person that should be kicked
						toKick = NULL;
						search = chatViews;
						while(search)
						{
							if ( strtoul(((chatWindowView*)search->data)->userID, NULL, 10) == event->toKick)
							{						
								toKick = ((chatWindowView*)search->data)->alias;
								break;
							}
							
							search = search->next;
						}							
					
						if (toKick)
							view->kickUser(toKick, event->toKick);
						else
							fprintf(stderr, "someone shall be kicked but is not in our list ...\n");
						break;
					
					case CEVENT_KICKED:
					case CEVENT_DISCONNECT:
						if (ownerView->ircButton && g_list_length(chatViews)==5)
							gtk_widget_set_sensitive(ownerView->ircButton, TRUE);
					
						ircView->insertLine(view->alias, tr("%s has left the chat\n"), NULL, TRUE);
						chatViews = g_list_remove(chatViews, view);
						view->destroyChatView();
						removeViewFromList((chatWindowView*)view);
						delete view;
						if (!isIRCStyle)
							rebuildViewMode();
						break;		
					
					case CEVENT_ERR_BIND:
					case CEVENT_ERR_CONNECT:
					case CEVENT_ERR_RES:
						break;
					
					case CEVENT_KICKYOU:
						u_showAlertMessage(tr("Terrific!"), tr("Congratulations!\nYour misbehavoir was finally recognized and has\nbrought you a kick out of the chat!\nsee you later!"), GTK_STOCK_DIALOG_INFO);
						break;
					
					case CEVENT_CHAR:
						view->insertCharacter(event->character);
						break;
			
					case CEVENT_CHAR_NEWLINE:
						view->insertCharacter("\n");						
						line = view->getCurrentLineAndClear();
						ircView->insertLine(view->alias,
											line,
											view->fontProperties,
										    FALSE
											);
						g_free(line);					
						break;

					case CEVENT_CHAR_BACKSPACE:
						view->deleteCharacter();
						break;
					
					case CEVENT_FONT_FAMILY:
						ch = charset_findCharsetByNumber(event->encoding);
						if (ch)
							view->charset = ch->id;
						break;
					
					case CEVENT_FONT_FACE:
					case CEVENT_FONT_SIZE:
					case CEVENT_FONT_FGCOLOR:
					case CEVENT_FONT_BGCOLOR:
						view->updateStyle(event);				
						break;
				}
			}
			break;					
	}
	
	if (request)
		return request->eventCallback(command, lastResult, info);
	else
		return TRUE;
}



gchar* chatWindow::getChatWindowTitle()
{
	GString *toReturn;
	GList	*search;
	
	// window wasn't initialized yet -> return nothing
	if (!ownerView)
		return NULL;
	
	toReturn = g_string_new("");
	
	if (chatViews)
	{
		search = chatViews;
		while(search)
		{
			if (strlen(toReturn->str)<60)
			{
				g_string_append(toReturn, ((chatWindowView*)(search->data))->alias);
				
				if (search != g_list_last(chatViews))
					g_string_append(toReturn, ", ");
			}else
			{
				g_string_append(toReturn, "...");
				break;
			}
			
			search = search->next;
		} 
	} else
		g_string_append(toReturn, tr("no participants yet"));
	
	return g_string_free(toReturn, FALSE);
}



GtkWidget* chatWindow::createSpecialWidget(gboolean canMultiparty)
{
	GSList	*group;
	GList	*chats;
	GtkWidget 	*vbox,
				*scrolled,
				*alignment;
	GtkListStore *store;
	GtkTreeIter  iter;
	GtkTreePath  *focus;
	gchar		*chatTitle,
				*tmp;
	gint		i;
	
	
	group = NULL;

	// create the buttons
	startNewChatButton = gtk_radio_button_new_with_label(group, tr("Open a new chat room"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(startNewChatButton), TRUE);
	g_signal_connect_swapped(startNewChatButton, "toggled", G_CALLBACK(this->cb_chatModeSelectionClicked), this);
	
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(startNewChatButton));
	joinExistingChatButton = gtk_radio_button_new_with_label(group, tr("Invite into an existing chat room"));
//	g_signal_connect(joinExistingChatButton, "toggled", G_CALLBACK(this->cb_chatModeSelectionClicked), this);
	
	
	store = gtk_list_store_new(2, G_TYPE_POINTER, G_TYPE_STRING);
	
	if (canMultiparty)
	{
		// create the list
		chats = basicWindow_getAllWindows();
	
		i = 0;
		while(chats)
		{
			if ( ((basicWindow*)chats->data)->wType == UE_CHAT ||
				 ((basicWindow*)chats->data)->wType == UE_CHATREQUEST)
			{
				i++;
			
				chatTitle = ((chatWindow*)chats->data)->getChatWindowTitle();
				if (chatTitle)
				{
					tmp = g_strdup_printf("<b>%d.</b> %s", i, chatTitle);
				
					gtk_list_store_append(store, &iter);
					gtk_list_store_set(store, &iter,
									    0, chats->data,
									    1, tmp,
									   -1);
					g_free(chatTitle);
					g_free(tmp);
				}
			}
			chats = chats->next;
		}
	}
	
	chatRoomList = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
	gtk_widget_set_sensitive(chatRoomList, FALSE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(chatRoomList),
								gtk_tree_view_column_new_with_attributes(
										tr("Existing rooms"), 
										gtk_cell_renderer_text_new(),
										"markup", 1,
										NULL
										)
								);
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC );
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled),
										GTK_SHADOW_IN);
								   
	gtk_container_add(GTK_CONTAINER(scrolled), chatRoomList);

	alignment = gtk_alignment_new(0, 0, 1, 1);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 20, 0);
	gtk_container_add(GTK_CONTAINER(alignment), scrolled);	
	
	
	if (!gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), NULL))
	{
		gtk_list_store_append(store, &iter);
		if (canMultiparty)
			gtk_list_store_set(store, &iter,
							    0, NULL,
							    1, tr(" - You havn't opened a chat room yet -"),
							   -1);	
		else
			gtk_list_store_set(store, &iter,
							   0, NULL,
							   1, tr("You cannot invite the user, because\nhe will open a room!"),
							   -1);		
		gtk_widget_set_sensitive(joinExistingChatButton, FALSE);
	} else
	{
		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
		focus = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
		gtk_tree_view_set_cursor(GTK_TREE_VIEW(chatRoomList), focus, NULL, FALSE);
		gtk_tree_path_free(focus);
	}
	
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), startNewChatButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), joinExistingChatButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), alignment, TRUE, TRUE, 0);
	
	return vbox;
}

	
	
GtkWidget* chatWindow::createButtonbar()
{
	GtkWidget	*hbox,
				*cancelButton,
				*saveButton,
				*inviteButton;
		
	// create the buttons
	cancelButton = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	
	saveButton = u_createTextStockImageButton(tr("Save chat"), GTK_STOCK_SAVE);
	
	inviteButton = u_createTextStockImageButton(tr("Invite user"), GTK_STOCK_JUMP_TO);
	
	
	// pack everything in a hbox
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), saveButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), inviteButton, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), cancelButton, FALSE, FALSE, 0);
	
	return hbox;
}



void chatWindow::addViewToList(chatWindowView* view)
{
	gtk_list_store_append(GTK_LIST_STORE(usersStore), &view->listIter);
	
	gtk_list_store_set(GTK_LIST_STORE(usersStore), &view->listIter,
						0, view->alias,
						1, view,
						-1);	
}



void chatWindow::removeViewFromList(chatWindowView* view)
{
	gtk_list_store_remove(GTK_LIST_STORE(usersStore), &view->listIter);
}


GtkWidget* chatWindow::createUserList()
{
	GtkCellRenderer		*renderer;
	GtkTreeViewColumn	*column;
	GtkWidget			*scroll;
		

	usersStore = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);	
	
	userList = gtk_tree_view_new_with_model(GTK_TREE_MODEL(usersStore));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (userList), TRUE);
	g_signal_connect(userList, "button_press_event", G_CALLBACK(this->cb_userListClicked), this);
	
	
	// create a renderer for the name
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "xalign", 0.0, NULL);
	column = gtk_tree_view_column_new_with_attributes(tr("Alias"), renderer, 
													  "markup", 0,
													  NULL );
	gtk_tree_view_append_column(GTK_TREE_VIEW(userList), column);

		
	// create a scrolled window around it
	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC );
	gtk_container_add(GTK_CONTAINER(scroll), userList);
	
	
	// finally create a frame for the tree view
	userListFrame = gtk_frame_new(0);
	gtk_widget_set_size_request(userListFrame, 80, -1);
	gtk_frame_set_shadow_type(GTK_FRAME(userListFrame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(userListFrame), scroll);	
	
	
	return userListFrame;	
}



	
GtkWidget* chatWindow::createUserSelection()
{
	return gtk_label_new("Work is in progress ...");	
}



	
void chatWindow::addChatView(chatWindowView* cwv)
{
	GtkWidget	*paned;
	
	GList		*children;
	
	
	if (!(children = gtk_container_get_children(GTK_CONTAINER(mainChatBox))))
	{
		// no chat views in list -> create a first one
		gtk_container_add(GTK_CONTAINER(mainChatBox), cwv->createChatView());
	} else
	{
		// there are already chat views -> create a vpaned to split them from the new
		paned = gtk_vpaned_new();
		gtk_paned_pack1(GTK_PANED(paned), cwv->createChatView(), TRUE, TRUE);
		
		// move the original child of mainChatBox into the paned view
		gtk_widget_reparent((GtkWidget*)children->data, paned);
		
		// move the paned view into the mainChatBox
		gtk_container_add(GTK_CONTAINER(mainChatBox), paned);
		
		g_list_free(children);
	}
	
	gtk_widget_show_all(mainChatBox);
}





chatFontInfo* chatWindow::getFontInfo()
{
	chatFontInfo *toReturn;
	PangoWeight weight;
	PangoStyle  style;
	PangoUnderline underline;
	GdkColor *colb, *colf;
	
	if (!ownerView || !ownerView->fontProperties)
		return NULL;
	
	toReturn = (chatFontInfo*)g_malloc0(sizeof(chatFontInfo));
	
	g_object_get(G_OBJECT(ownerView->fontProperties),
				 "family", &toReturn->family,
				 "size", &toReturn->size,
				 "weight", &weight,
				 "style", &style,
				 "underline", &underline,
				 "strikethrough", &toReturn->strikeout,
				 "foreground-gdk", &colf,
				 "background-gdk", &colb,
				 NULL);
	
	memcpy(&toReturn->foreground, colf, sizeof(GdkColor));
	memcpy(&toReturn->background, colb, sizeof(GdkColor));
	
	weight == PANGO_WEIGHT_NORMAL ? toReturn->bold = FALSE : toReturn->bold = TRUE;
	style  == PANGO_STYLE_NORMAL  ? toReturn->italic = FALSE : toReturn->italic = TRUE;
	underline == PANGO_UNDERLINE_NONE ? toReturn->underline = FALSE : toReturn->underline = TRUE;	

	toReturn->size = toReturn->size / PANGO_SCALE;
	
	return toReturn;
}




chatWindow* chatWindow::getMultipartyWindow()
{
	GtkTreePath *path;
	GtkTreeIter iter;
	chatWindow* multiChat;
	
	gtk_tree_view_get_cursor (GTK_TREE_VIEW(chatRoomList), &path, NULL);

	if (!path)
		return NULL;
	
	multiChat = NULL;
	
	gtk_tree_model_get_iter(gtk_tree_view_get_model(GTK_TREE_VIEW(chatRoomList)), &iter, path);
	gtk_tree_model_get(gtk_tree_view_get_model(GTK_TREE_VIEW(chatRoomList)), &iter, 
	   				   0, &multiChat, -1);
	gtk_tree_path_free(path);
	
	
	return multiChat;
}




void chatWindow::cb_chatModeSelectionClicked(chatWindow* chw)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chw->startNewChatButton)))
		gtk_widget_set_sensitive(chw->chatRoomList, FALSE);
	else
	{
		gtk_widget_set_sensitive(chw->chatRoomList, TRUE);
		gtk_widget_grab_focus(chw->chatRoomList);
	}
}

	
	
void chatWindow::cb_changeViewButtonClicked(chatWindow* chw)
{	
	chw->isIRCStyle = !chw->isIRCStyle;
		
	chw->rebuildViewMode();
}



		
void chatWindow::cb_requestCallback(gint result, gpointer info, chatWindow* cw)
{
	chatFontInfo *fontInfo;
	chatWindow	 *multiChat;
	
	
	if (cw->isServer)
	{
		switch(result)
		{
			case REQUEST_CANCELLED:
				cw->request->stopWaitForRequestAnswer();
				cw->manager->cancelInitatedEvent();
				break;
			
			case REQUEST_CLOSED:
				cw->request = NULL;
				delete cw;
				break;
			
			case REQUEST_ACCEPTED:
				// a new room should be opened
				if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw->startNewChatButton)))
				{
					((IMChatManager*)cw->manager)->sendRequest((gchar*)info, NULL, 0);
				} else
				// an existing room should be joined
				{
					multiChat = cw->getMultipartyWindow();	
					// FIXME: check whether the chat is still active
					((IMChatManager*)cw->manager)->sendRequest((gchar*)info, 
															   NULL, 
															   ((IMChatManager*)multiChat->manager)->getChatPort());
					cw->multipartyWindow = multiChat;
				}
				
				cw->request->waitForRequestAnswer();
				break;
		}
	} else
	{
		switch(result)
		{
			// window was destroyed without any response
			case REQUEST_CLOSED:
				cw->request = NULL;
				delete cw; 
				break;
		
			// request was refused
			case REQUEST_REFUSED:
				((IMChatManager*)cw->manager)->answerRemoteRequest(FALSE, (gchar*)info);
				cw->request->destroyWindow();
				delete cw->request;
				cw->request = NULL;
				delete cw;
				break;
		
			// request was accepted
			case REQUEST_ACCEPTED:
				cw->createWindow();
				fontInfo = cw->getFontInfo();
				if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw->startNewChatButton)))
				{
					((IMChatManager*)cw->manager)->answerRemoteRequest(TRUE, NULL, fontInfo, 0);
				} else
				{
					multiChat = cw->getMultipartyWindow();
					((IMChatManager*)cw->manager)->answerRemoteRequest(TRUE, NULL, fontInfo, ((IMChatManager*)multiChat->manager)->getChatPort());
					cw->multipartyWindow = multiChat;
				}
				g_free(fontInfo);
	
				cw->request->destroyWindow();
				delete cw->request;
				cw->request = NULL;
				break;
		}
	}
	
}



gboolean chatWindow::cb_userListClicked(GtkTreeView* uList, GdkEventButton* event, chatWindow* cw)
{
	GtkTreePath			*path;
	GtkTreeIter 		iter;
	chatWindowRemoteView *view;
	
	if (event->button != 3)
		return FALSE;
	
	if (!gtk_tree_view_get_path_at_pos(uList, (gint)event->x, (gint)event->y, &path, NULL, NULL, NULL))
		return FALSE;

	
	gtk_tree_model_get_iter(gtk_tree_view_get_model(uList), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(gtk_tree_view_get_model(uList), &iter, 
					   1, &view, -1);
	

	if ((gpointer)view != (gpointer)cw->ownerView)
	{
		gtk_menu_popup(GTK_MENU(view->createUserMenu()), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
	}
	
	return FALSE;
}





// ================== chatWindowView ==========================================
chatWindowView::chatWindowView(gchar* _alias, gchar* _userID, chatWindow* _parentWindow)
{
	_alias ? alias = g_markup_escape_text(_alias, strlen(_alias)) : alias = NULL;
	_userID ? userID = g_strdup(_userID) : userID = NULL;
	parentWindow = _parentWindow;
	textBox = NULL;
	viewType = CWVT_PROTO; 
	fontProperties = NULL;
	encodingsMenu = NULL;
	currentLine = g_string_new("");
	history = g_string_new("");
	charset = NULL;
	useUserColors = TRUE;
	useUserFont = TRUE;
}



chatWindowView::~chatWindowView()
{
	if (alias) g_free(alias);
	if (userID) g_free(userID);
	g_string_free(currentLine, TRUE);
	g_string_free(history, TRUE);
}



void chatWindowView::createFontContext()
{
	GtkWidget *tmpbox;
	GtkTextAttributes *defProps; 
	GdkColor  chatBackgroundColor,
			  chatFontColor;
	memset(&chatBackgroundColor, 0xFFFF, sizeof(GdkColor));
	memset(&chatFontColor, 0x0000, sizeof(GdkColor));
	
	tmpbox = gtk_text_view_new();
	
	// create a default properties tag and apply all settings
	defProps = gtk_text_view_get_default_attributes(GTK_TEXT_VIEW(tmpbox));
	fontProperties = gtk_text_tag_new(NULL);
	g_object_set(G_OBJECT(fontProperties),
				 "size", pango_font_description_get_size(defProps->font),
				 "size-set", useUserFont,
				 "family", pango_font_description_get_family(defProps->font),
				 "family-set", useUserFont,	
				 "foreground-gdk", &chatFontColor,
				 "foreground-set", useUserColors,
				 "background-gdk", &chatBackgroundColor, 
				 "background-set", useUserColors,
				 "style", pango_font_description_get_style(defProps->font),
				 "style-set", useUserFont,
				 "weight", pango_font_description_get_weight(defProps->font),
				 "weight-set", useUserFont,
				 "underline", defProps->appearance.underline,
				 "underline-set", useUserFont,
				 "strikethrough", (gboolean)defProps->appearance.strikethrough,
				 "strikethrough-set", useUserFont,
				 "editable", FALSE,
				 NULL);						
				
	g_free(defProps);	
	
	gtk_widget_destroy(tmpbox);
}



GtkWidget* chatWindowView::createChatView()
{
	GtkWidget *vbox,
			  *scroll;
	GdkColor  *background;
	
	GtkTextIter iter;
	GtkTextBuffer* buf;

	
	if (!fontProperties)
		createFontContext();		
	
	g_object_get(G_OBJECT(fontProperties),
				 "background-gdk", &background,
				 NULL);
	
	// create the text box
	scroll = u_createTextView(&textBox);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textBox), GTK_WRAP_WORD_CHAR);
	
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textBox));
	
	// add the property to the list
	gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(buf),
						   fontProperties);
	
	// set the old text
	gtk_text_buffer_get_start_iter(buf, &iter);
	gtk_text_buffer_insert_with_tags(buf, &iter, history->str, strlen(history->str), fontProperties, NULL);

	// scroll down
	gtk_text_buffer_get_end_iter(buf, &iter);
	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textBox), &iter, 0, FALSE, 0, 0);	
	

	// set correct background color
	gtk_widget_modify_base(textBox, GTK_STATE_NORMAL, background);	


	vbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), createToolBar(), FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
	
	
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 2);
	
	boxFrame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(boxFrame), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(boxFrame), vbox);

	
	return boxFrame;
}



void chatWindowView::destroyChatView()
{	
	// this chatview was already destroyed -> return
	if (!textBox)
		return;
	
	textBox = NULL;
}




GtkWidget* chatWindowView::createToolBar()
{
	GtkWidget	*tb;
	
	tb = gtk_hbox_new(FALSE, 5);
	
	return tb;
}




void chatWindowView::setSelectedEncodingsMenuItem(gchar* id)
{
	guint i;
	charsetType	*charsets;
	GList	*children;
	GtkWidget *item;
	
	
	if (!encodingsMenu)
		return;
	
	charsets = charset_getCharsetList();
	children = gtk_container_get_children(GTK_CONTAINER(encodingsMenu));
	
	
	for (i=0; i<g_list_length(children);i++)
	{
		if (!g_ascii_strcasecmp(charsets[i].id, id))
		{
			item = (GtkWidget*)g_list_nth_data(children, i);
			// block emission of the "activate" signal (to prevent another call of encodingChanged(...) )
			g_signal_handlers_block_by_func(item, (gpointer)this->cb_selectEncoding, this);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
			g_signal_handlers_unblock_by_func(item, (gpointer)this->cb_selectEncoding, this);
			break;
		}
			
	}
		
	g_list_free(children);
	return;
}



void chatWindowView::encodingChanged(charsetType* enc)
{
	
}



void chatWindowView::updateStyle(chatEventInfo *ev)
{	
	GdkColor *col;
	gboolean setBackground;
	
	if (!fontProperties)
		return;
		
	if (ev)
	{
		g_object_set(G_OBJECT(fontProperties),
					 "family", ev->font.family,
					 "family-set", useUserFont,
					 "foreground-gdk", &ev->font.foreground,
					 "foreground-set", useUserColors,
					 "background-gdk", &ev->font.background,
					 "background-set", useUserColors,
					 "weight", ev->font.bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL,
					 "weight-set", useUserFont,
					 "style", ev->font.italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL,
					 "style-set", useUserFont,
					 "underline", ev->font.underline ? PANGO_UNDERLINE_SINGLE : PANGO_UNDERLINE_NONE,
					 "underline-set", useUserFont,
					 "strikethrough", ev->font.strikeout,
					 "strikethrough-set", useUserFont,
					 "size", ev->font.size * PANGO_SCALE,
					 "size-set", useUserFont,
					NULL);
	}
	
	// don't take the color from ev, because it could be NULL
	g_object_get(G_OBJECT(fontProperties),
				 "background-gdk", &col,
				 "background-set", &setBackground,
				 NULL);
	
	if (textBox)
	{
		if (setBackground)
			gtk_widget_modify_base(textBox, GTK_STATE_NORMAL, col);
		else
			gtk_widget_modify_base(textBox, GTK_STATE_NORMAL, NULL);
	}
}



void chatWindowView::cb_showSubmenu(GtkWidget* subMenu)
{
	gtk_menu_popup(GTK_MENU(subMenu), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
}




void chatWindowView::cb_selectEncoding(GtkWidget* item, chatWindowView *cwv)
{
	charsetType* charsets;
	guint		 num;
	
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item)))
	{
		charsets = charset_getCharsetList();
	
		num = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(item), "icqnd-menu"));

		cwv->encodingChanged(&charsets[num]);
	}
}





// ================== chatWindowIRCView ========================================

chatWindowIRCView::chatWindowIRCView(chatWindow* _parentWindow):
chatWindowView::chatWindowView(NULL, NULL, _parentWindow)
{
	viewType = CWVT_IRC;
	
	textBuffer = gtk_text_buffer_new(NULL);
	
	fontComment = gtk_text_buffer_create_tag(textBuffer, "comment",
											 "weight", PANGO_WEIGHT_BOLD,
											 NULL);
	
	fontAlias = gtk_text_buffer_create_tag(textBuffer, "alias",
										   "weight", PANGO_WEIGHT_BOLD,
										   NULL);
	
	gtk_text_buffer_create_tag(textBuffer, "newline", 
								"size", 3*PANGO_SCALE, NULL);
}


chatWindowIRCView::~chatWindowIRCView()
{
	g_object_unref(textBuffer);
}



GtkWidget* chatWindowIRCView::createChatView()
{
	GtkWidget *v;
	GtkTextIter endIter;
	
	v = chatWindowView::createChatView();
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW(textBox), FALSE);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(textBox), textBuffer);
	
	gtk_text_buffer_get_end_iter(textBuffer, &endIter);
	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textBox), &endIter, 0, FALSE, 0, 0);	

	
	return v;
}



void chatWindowIRCView::insertLine(gchar* alias, gchar* line, GtkTextTag* font, gboolean isComment)
{
	gchar *tmp;
	GtkTextIter endIter;
		
	if (isComment)
		tmp = g_strdup_printf(line, alias);
	else
	{
		tmp = g_strdup_printf("%s: ", alias);
		gtk_text_buffer_get_end_iter(textBuffer, &endIter);
		gtk_text_buffer_insert_with_tags(textBuffer, &endIter, tmp, strlen(tmp), fontAlias, NULL);
		g_free(tmp);
		
		tmp = g_strdup(line);
	}
	
	gtk_text_buffer_get_end_iter(textBuffer, &endIter);
	
	if (isComment)
		gtk_text_buffer_insert_with_tags(textBuffer, &endIter, tmp, strlen(tmp), fontComment, NULL);
	else
		gtk_text_buffer_insert(textBuffer, &endIter, tmp, strlen(tmp));

	g_free(tmp);	
	
	tmp = "\n\n";
	gtk_text_buffer_get_end_iter(textBuffer, &endIter);
	gtk_text_buffer_insert_with_tags_by_name(textBuffer, &endIter, tmp, strlen(tmp), "newline", NULL);

	// scroll down
	if (textBox)
	{
		gtk_text_buffer_get_end_iter(textBuffer, &endIter);
		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textBox), &endIter, 0, FALSE, 0, 0);	
	}
}



// ================= chatWindowLocalView =======================================

chatWindowLocalView::chatWindowLocalView(chatWindow* _parentWindow):
chatWindowView::chatWindowView(_parentWindow->manager->parentUser->owner->info->alias, NULL, _parentWindow)
{
	charsetType*	ch;
	
	viewType = CWVT_OWNER;
	ircButton = NULL;
	
	ch = charset_findCharsetById(_parentWindow->manager->info->charset);
	if (ch)
		currentEncoding = ch->number;
	else
		currentEncoding = ENCODING_DEFAULT;
}



chatWindowLocalView::~chatWindowLocalView()
{
	// chatWindowView::~chatWindowView();
}



GtkWidget* chatWindowLocalView::createChatView()
{		
	chatWindowView::createChatView();
	
	gtk_frame_set_shadow_type(GTK_FRAME(boxFrame), GTK_SHADOW_OUT);
	
	g_signal_connect_after(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textBox)), "insert-text",G_CALLBACK(this->cb_textInserted), this);
	g_signal_connect(textBox, "key-press-event", G_CALLBACK(this->cb_keyPressed), this);

	return boxFrame;
}



GtkWidget* chatWindowLocalView::createToolBar()
{
	GtkWidget	*tb,
				*tools,
				*comboSize,
				*comboFamily;
	
	GtkToolItem *toolItem;	
	PangoFontFamily **fontFamilies;
	gint			fontCount, i;
	guint			asize;
	gchar			*tmp;
	GtkTooltips		*tips;
	
	guint fontSizes[21] =
	{8, 9, 10, 11, 12, 13, 14, 16, 18, 20, 22, 24, 26, 28, 32, 36, 40, 48, 56, 64, 72};
	
	
	tb = chatWindowView::createToolBar();
	
	tips = gtk_tooltips_new();
		
	// create owner toolbar
	tools = gtk_toolbar_new();
	gtk_toolbar_set_tooltips(GTK_TOOLBAR(tools), TRUE);
	gtk_toolbar_set_style(GTK_TOOLBAR(tools), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_icon_size(GTK_TOOLBAR(tools), GTK_ICON_SIZE_BUTTON);
	
	// font color
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_SELECT_FONT, GTK_ICON_SIZE_BUTTON), "Schriftfarbe");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Font color"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_showSubmenu), createColorMenus(FALSE)); 	
		
	// background color
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_SELECT_COLOR, GTK_ICON_SIZE_BUTTON), "Hintergrundfarbe");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Background color"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_showSubmenu), createColorMenus(TRUE)); 	


	// fonts
	
	// *** create the font family combo box ***
	// FIXME: insert a kind of preview
	// FIXME: make menu item appear in the proxy menu	
	comboFamily = gtk_combo_box_new_text();
	g_signal_connect(comboFamily, "changed", G_CALLBACK(this->cb_selectFont), this);
	gtk_widget_set_size_request(comboFamily, 100, -1);
	
	
	pango_context_list_families(gtk_widget_get_pango_context(comboFamily), &fontFamilies, &fontCount);
	g_qsort_with_data(fontFamilies, fontCount, sizeof(PangoFontFamily *), (GCompareDataFunc)this->cb_sortFamilies, NULL);
	
	g_object_get(G_OBJECT(fontProperties), "family", &tmp, NULL);
	
	for (i=0; i<fontCount; i++)
	{
		gtk_combo_box_append_text(GTK_COMBO_BOX(comboFamily), pango_font_family_get_name(fontFamilies[i]));
		
		if (!g_ascii_strcasecmp(pango_font_family_get_name(fontFamilies[i]), tmp))
			gtk_combo_box_set_active(GTK_COMBO_BOX(comboFamily), i);	
	}
	
	toolItem = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(toolItem), comboFamily); 
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	
	
	
	// *** create the font sizes combo box ***
	// FIXME: make menu item appear in the proxy menu
	comboSize = gtk_combo_box_new_text();
	g_signal_connect(comboSize, "changed", G_CALLBACK(this->cb_selectSize), this);
	
	
	g_object_get(G_OBJECT(fontProperties), "size", &asize, NULL);
	asize = asize / PANGO_SCALE;
	
	for (i=0; i<21; i++)
	{
		tmp = g_strdup_printf("%d", fontSizes[i]);
		gtk_combo_box_append_text(GTK_COMBO_BOX(comboSize), tmp);
		g_free(tmp);
		
		if (fontSizes[i] == asize)
			gtk_combo_box_set_active(GTK_COMBO_BOX(comboSize), i);
	}
	
	toolItem = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(toolItem), comboSize); 
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	
	/*
	// font
	toolItem = gtk_tool_item_new();
	tmp = gtk_font_button_new();
	gtk_font_button_set_show_style(GTK_FONT_BUTTON(tmp), FALSE);
	gtk_button_set_relief(GTK_BUTTON(tmp), GTK_RELIEF_NONE);
	gtk_container_add(GTK_CONTAINER(toolItem), tmp);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect(tmp, "font-set", G_CALLBACK(this->cb_selectFont), this); 	
*/


	// separator
	gtk_toolbar_insert(GTK_TOOLBAR(tools), gtk_separator_tool_item_new(), -1);
	
	// bold
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_BOLD, GTK_ICON_SIZE_BUTTON), "Fett");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Bold text"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);	
	g_signal_connect(toolItem, "clicked", G_CALLBACK(this->cb_selectBold), this);
	
	// italic
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_ITALIC, GTK_ICON_SIZE_BUTTON), "Kursiv");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Italic text"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);	
	g_signal_connect(toolItem, "clicked", G_CALLBACK(this->cb_selectItalic), this);


	// underlined
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_UNDERLINE, GTK_ICON_SIZE_BUTTON), "Unterstrichen");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Underlined text"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);	
	g_signal_connect(toolItem, "clicked", G_CALLBACK(this->cb_selectUnderline), this);
	
	// strikedout
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_STRIKETHROUGH, GTK_ICON_SIZE_BUTTON), "Durchgestrichen");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Strokedout text"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);	
	g_signal_connect(toolItem, "clicked", G_CALLBACK(this->cb_selectStrikeout), this);
	
	// separator
	gtk_toolbar_insert(GTK_TOOLBAR(tools), gtk_separator_tool_item_new(), -1);
	
	// encoding 
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_JUSTIFY_LEFT, GTK_ICON_SIZE_BUTTON), "Zeichenkodierung");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Character set"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);	
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_showSubmenu), createEncodingsMenu());
	
	
	// separator
	gtk_toolbar_insert(GTK_TOOLBAR(tools), gtk_separator_tool_item_new(), -1);
	
	// IRC
	ircButton = GTK_WIDGET(gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_CONVERT, GTK_ICON_SIZE_BUTTON), "IRC-Modus"));
	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(ircButton), tips, tr("IRC mode on/off"), NULL);
	g_signal_connect_swapped(ircButton, "clicked", G_CALLBACK(parentWindow->cb_changeViewButtonClicked), parentWindow);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), GTK_TOOL_ITEM(ircButton), -1);	


	gtk_box_pack_start(GTK_BOX(tb), tools, TRUE, TRUE, 0);
		
	return tb;
}



void	chatWindowLocalView::updateFontInformation(guint toUpdate)
{
	IMChatManager* man;
	GdkColor *col;
	gchar* family;
	gint size;
	gint weight;
	PangoStyle style;
	PangoUnderline underline;
	gboolean strikethrough;
	
	
	
	man = (IMChatManager*)parentWindow->manager;
	
	if (toUpdate & CF_FGCOLOR)
	{
		g_object_get(G_OBJECT(fontProperties), "foreground-gdk", &col, NULL);
		man->setColor(FALSE, col->red>>8, col->green>>8, col->blue>>8);
	}

	if (toUpdate & CF_BGCOLOR)
	{
		g_object_get(G_OBJECT(fontProperties), "background-gdk", &col, NULL);
		man->setColor(TRUE, col->red>>8, col->green>>8, col->blue>>8);
	}
		
	if (toUpdate & CF_FAMILY)
	{
		g_object_get(G_OBJECT(fontProperties), "family", &family, NULL);
		man->setFontFamily( family, 
							currentEncoding, 
							STYLE_DONTCARE | STYLE_VARIABLExPITCH); // FIXME: make this variable
	}
	
	if (toUpdate & CF_SIZE)	
	{
		g_object_get(G_OBJECT(fontProperties), "size", &size, NULL);
		man->setFontSize( size / PANGO_SCALE);
	}
	
	if (toUpdate & CF_FACE)
	{
		g_object_get(G_OBJECT(fontProperties),
					 "weight", &weight,
					 "style", &style,
					 "underline", &underline,
					 "strikethrough", &strikethrough, NULL);
		man->setFontFace( weight >= PANGO_WEIGHT_BOLD,
						  style  != PANGO_STYLE_NORMAL,
						  underline != PANGO_UNDERLINE_NONE,
						  strikethrough);
	}
	
}



void chatWindowLocalView::clearTextBuffer()
{
	GtkTextIter startIter, endIter;
	GtkTextBuffer *buf;
	
	if (!textBox)
		return;
	
	// only show the unfinished line and delete the rest 
	// (the insert-handler has to be blocked, otherwise a loop would be created)
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textBox));
	
	gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	gtk_text_buffer_delete(buf, &startIter, &endIter);
	gtk_text_buffer_get_start_iter(buf, &startIter);
	
	g_signal_handlers_block_by_func(buf, (gpointer)this->cb_textInserted, this);

	if (parentWindow->isIRCStyle)
		gtk_text_buffer_insert_with_tags(buf, &startIter, currentLine->str, strlen(currentLine->str), fontProperties, NULL);
	else
		gtk_text_buffer_insert_with_tags(buf, &startIter, history->str, strlen(history->str), fontProperties, NULL);
	
	g_signal_handlers_unblock_by_func(buf, (gpointer)this->cb_textInserted, this);

	gtk_text_buffer_get_end_iter(buf, &endIter);
	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textBox), &endIter, 0, FALSE, 0, 0);	
}




GtkWidget* chatWindowLocalView::createEncodingsMenu()
{
	guint i1;
	charsetType	*charsets;
	GtkWidget	*item;
	GSList		*group;
	gchar		*txt;
	gboolean	itemSet;
	
	if (encodingsMenu)
		return encodingsMenu;
	
	i1=0;
	charsets = charset_getCharsetList();
	encodingsMenu = gtk_menu_new();
	group = NULL;
	itemSet = FALSE;
	
	while(charsets[i1].name)
	{
		txt = g_strdup_printf("%s (%s)", tr(charsets[i1].name), charsets[i1].id);
		item = gtk_radio_menu_item_new_with_label(group, txt);
		
		if (!itemSet && charsets[i1].number == currentEncoding)
		{
			itemSet = TRUE;
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
		} else
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), FALSE);
		
		g_signal_connect(G_OBJECT(item),"activate", G_CALLBACK(this->cb_selectEncoding), this);
		g_object_set_data(G_OBJECT(item), "icqnd-menu", GUINT_TO_POINTER(i1));
		group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
		g_free(txt);
		gtk_menu_shell_append(GTK_MENU_SHELL(encodingsMenu), item);	
		i1++;
	}
	
	gtk_widget_show_all(encodingsMenu);
	
	return encodingsMenu;
}




GtkWidget* chatWindowLocalView::createColorMenus(gboolean createBG)
{
	guint i;
	GdkPixbuf	*bgCol;
	GtkWidget	*menu,
				*item,
				*img,
				*label;
	gchar		*txt;
	guint32		curCol;
		
	menu = gtk_menu_new();
	
	
	for (i=0; i<NUM_COLORS; i++)
	{
		item = gtk_menu_item_new();
		
		
		// create the colored pixbuf
		if (createBG)
		{
			bgCol = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 
								   FALSE,
								   8,
								   40,
								   15);
			curCol = (chatColors[i][0]<<24) + (chatColors[i][1]<<16) + (chatColors[i][2]<<8);
			gdk_pixbuf_fill(bgCol, curCol);
			img = gtk_image_new_from_pixbuf(bgCol);
			g_object_unref(bgCol);
			gtk_container_add(GTK_CONTAINER(item), img);
			g_object_set_data(G_OBJECT(item), "icqnd-bcol", (gpointer)&chatColors[i]);
			g_signal_connect(item, "activate", G_CALLBACK(this->cb_selectColor), this);
		} else
		{
			txt = g_strdup_printf("<span color=\"#%02X%02X%02X\">abcd</span>", chatColors[i][0], chatColors[i][1], chatColors[i][2]);
			label = gtk_label_new(txt);
			g_free(txt);
			gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
			gtk_container_add(GTK_CONTAINER(item), label);
			g_object_set_data(G_OBJECT(item), "icqnd-fcol", (gpointer)&chatColors[i]);
			g_signal_connect(item, "activate", G_CALLBACK(this->cb_selectColor), this);
		}
		
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	}
	
	gtk_widget_show_all(menu);
	
	return menu;
}



void chatWindowLocalView::encodingChanged(charsetType* enc)
{
	currentEncoding = enc->number;
	updateFontInformation(CF_FAMILY);
}



gint chatWindowLocalView::cb_sortFamilies(PangoFontFamily** a, PangoFontFamily** b)
{
	const gchar *name1 = pango_font_family_get_name(*a);
	const gchar *name2 = pango_font_family_get_name(*b);
	
	return g_utf8_collate(name1, name2);
}



gboolean chatWindowLocalView::cb_keyPressed(GtkWidget* w, GdkEventKey* event, chatWindowLocalView *cwv)
{
	GtkTextIter startIter, endIter, insertIter;
	GtkTextBuffer *buf;
	gchar* tmp;
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cwv->textBox));	
	
	// get end iter and insertion point
	gtk_text_buffer_get_end_iter(buf, &endIter);
	gtk_text_buffer_get_iter_at_mark(buf, &insertIter, gtk_text_buffer_get_mark(buf, "insert"));		

	// a character was typed before the end -> don't allow writing
	if (event->length>0 && !gtk_text_iter_equal(&endIter, &insertIter))
		return TRUE;
	
	if (event->keyval != 0xFF08)
		return FALSE;
		
	if (gtk_text_buffer_get_char_count(buf)>0 && gtk_text_iter_equal(&endIter, &insertIter))
	{
		startIter = endIter;
		gtk_text_iter_backward_cursor_position(&startIter);

		// one character can be longer than one byte, so more text has to be deleted
		tmp = gtk_text_buffer_get_text(buf, &startIter, &endIter, TRUE);
		g_string_truncate(cwv->currentLine, strlen(cwv->currentLine->str)-strlen(tmp));
		g_string_truncate(cwv->history, strlen(cwv->history->str)-strlen(tmp));	
		
		g_free(tmp);
		
		// knowing that the other chat programs are intelligent we send the delete-signal only once
		((IMChatManager*)cwv->parentWindow->manager)->sendInformation(CST_BACKSPACE, 0);		
			
		
		gtk_text_buffer_delete(buf, &startIter, &endIter);
		
		


	}
	
	return FALSE;
}



void chatWindowLocalView::cb_textInserted(GtkTextBuffer *textbuffer, GtkTextIter *arg1, gchar *arg2, gint arg3, chatWindowLocalView* cw)
{
	GtkTextIter startIter, endIter;
	guint i;
	
	for (i=0; i<strlen(arg2);i++)
	{
		g_string_append_c(cw->currentLine, arg2[i]);
		
		if (arg2[i]==10)
		{
			((IMChatManager*)cw->parentWindow->manager)->sendInformation(CST_NEWLINE, 0);
			// FIXME: i don't like that
			cw->parentWindow->ircView->insertLine(cw->parentWindow->manager->parentUser->owner->info->alias, cw->currentLine->str, cw->fontProperties, FALSE);
			g_string_truncate(cw->currentLine, 0);
			
			if (cw->parentWindow->isIRCStyle)
				cw->clearTextBuffer();
			
		}
		else
			((IMChatManager*)cw->parentWindow->manager)->sendInformation(CST_CHAR, arg2[i]);	
	}
	
	g_string_append(cw->history, arg2);
	
	gtk_text_buffer_get_bounds(gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->textBox)),
								&startIter,
								&endIter);
	gtk_text_buffer_apply_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->textBox)),
							  cw->fontProperties,
							  &startIter,
							  &endIter);
}





void chatWindowLocalView::cb_selectColor(GtkWidget* item, chatWindowLocalView *cwv)
{
	GdkColor col;
	guchar *colInt;
	gboolean doBackground;
	
	if (!cwv->textBox)
		return;
	
	if ( (colInt = (guchar*)g_object_get_data(G_OBJECT(item), "icqnd-bcol")))
		doBackground = TRUE;
	else
	{
		colInt = (guchar*)g_object_get_data(G_OBJECT(item), "icqnd-fcol");
		doBackground = FALSE;
	}

	col.red = (colInt[0]<<8);
	col.green = (colInt[1]<<8);
	col.blue = (colInt[2]<<8);
	
	
	if (doBackground)
	{
		if (cwv->useUserColors)
			gtk_widget_modify_base(cwv->textBox, GTK_STATE_NORMAL, &col);
		g_object_set(G_OBJECT(cwv->fontProperties), 
					 "background-gdk", &col, 
					 NULL);
		cwv->updateFontInformation(CF_BGCOLOR);
	}
	else
	{
		g_object_set(G_OBJECT(cwv->fontProperties), 
					 "foreground-gdk", &col,	
					 NULL);
		cwv->updateFontInformation(CF_FGCOLOR);
	}

	// ((IMChatManager*)cwv->parentWindow->manager)->setColor(doBackground, colInt[0], colInt[1], colInt[2]);	
}




void chatWindowLocalView::cb_selectFont(GtkWidget* box, chatWindowLocalView *cwv)
{
	GtkTreeIter iter;
	gchar		*name;
	
	if (!cwv->fontProperties || gtk_combo_box_get_active_iter(GTK_COMBO_BOX(box), &iter))
	{
		gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(box)),
						   &iter,
						   0, &name,
						   -1);
		
		g_object_set(G_OBJECT(cwv->fontProperties), "family", name, NULL);
		
		cwv->updateFontInformation(CF_FAMILY);
	}
}




void chatWindowLocalView::cb_selectSize(GtkWidget* box, chatWindowLocalView *cwv)
{
	GtkTreeIter iter;
	gchar		*name;
	
	if (!cwv->fontProperties || gtk_combo_box_get_active_iter(GTK_COMBO_BOX(box), &iter))
	{
		gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(box)),
						   &iter,
						   0, &name,
						   -1);
		
		g_object_set(G_OBJECT(cwv->fontProperties), "size", strtoul(name, NULL, 10) * PANGO_SCALE, NULL);
		
		cwv->updateFontInformation(CF_SIZE);
	}
	
}




void chatWindowLocalView::cb_selectBold(GtkWidget* btn, chatWindowLocalView *cwv)
{
	gint weight;
	
	g_object_get(G_OBJECT(cwv->fontProperties), "weight", &weight, NULL);
	
	if (weight>=PANGO_WEIGHT_BOLD)
		weight = PANGO_WEIGHT_NORMAL;
	else
		weight = PANGO_WEIGHT_BOLD;
	
	g_object_set(G_OBJECT(cwv->fontProperties), 
				 "weight", weight, 
				 NULL);
	
	cwv->updateFontInformation(CF_FACE);
}



void chatWindowLocalView::cb_selectItalic(GtkWidget* btn, chatWindowLocalView *cwv)
{
	PangoStyle style;
	
	g_object_get(G_OBJECT(cwv->fontProperties), 
				 "style", &style, 
				 NULL);
	
	if (style != PANGO_STYLE_ITALIC)
		style = PANGO_STYLE_ITALIC;
	else
		style = PANGO_STYLE_NORMAL;
	
	g_object_set(G_OBJECT(cwv->fontProperties), 
				 "style", style,
				 NULL);
	
	cwv->updateFontInformation(CF_FACE);
}



void chatWindowLocalView::cb_selectUnderline(GtkWidget* btn, chatWindowLocalView *cwv)
{
	PangoUnderline underline;
	
	g_object_get(G_OBJECT(cwv->fontProperties), "underline", &underline, NULL);
	
	if (underline != PANGO_UNDERLINE_NONE)
		underline = PANGO_UNDERLINE_NONE;
	else
		underline = PANGO_UNDERLINE_SINGLE;
	
	g_object_set(G_OBJECT(cwv->fontProperties), 
				 "underline", underline, 
				 NULL);	
	
	cwv->updateFontInformation(CF_FACE);

}



void chatWindowLocalView::cb_selectStrikeout(GtkWidget* btn, chatWindowLocalView *cwv)
{
	gboolean strikethrough;
	
	g_object_get(G_OBJECT(cwv->fontProperties), "strikethrough", &strikethrough, NULL);
	
	strikethrough = !strikethrough;
	
	g_object_set(G_OBJECT(cwv->fontProperties), 
				 "strikethrough", strikethrough, 
				 NULL);
	
	cwv->updateFontInformation(CF_FACE);

}



// ================= chatWindowRemoteView ======================================


chatWindowRemoteView::chatWindowRemoteView(gchar* _alias, gchar* _userID, chatWindow* _parentWindow):
chatWindowView::chatWindowView(_alias, _userID, _parentWindow)
{
	viewType = CWVT_REMOTE;
	userMenu = NULL;
}


chatWindowRemoteView::~chatWindowRemoteView()
{
	
}



GtkWidget* chatWindowRemoteView::createChatView()
{
	GtkWidget* v;
	
	v = chatWindowView::createChatView();
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW(textBox), FALSE);	
	return v;
}



GtkWidget* chatWindowRemoteView::createToolBar()
{
	GtkWidget	*tb,
				*tools,
				*name;
	GtkToolItem *toolItem;
	GtkTooltips *tips;
	
	tb = chatWindowView::createToolBar();

	
	// create remote name entry	
	name = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(name), alias);
 	gtk_widget_set_size_request(name, 100, -1);
	gtk_widget_set_sensitive(name, FALSE);
	// gtk_editable_set_editable(GTK_EDITABLE(name), FALSE);
	

	tips = gtk_tooltips_new();	

	
	// create remote toolbar
	tools = gtk_toolbar_new();
	gtk_toolbar_set_tooltips(GTK_TOOLBAR(tools), TRUE);
	gtk_toolbar_set_style(GTK_TOOLBAR(tools), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_icon_size(GTK_TOOLBAR(tools), GTK_ICON_SIZE_BUTTON);
	
	// add items
	
	// user menu
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON), "Benutzermenu");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Send an event to the user"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_showSubmenu), createUserMenu());

	// separator
	gtk_toolbar_insert(GTK_TOOLBAR(tools), gtk_separator_tool_item_new(), -1);
	
	
	// toggle user font
	toolItem = gtk_tool_button_new(fontButtonImg = gtk_image_new_from_stock(GTK_STOCK_SELECT_FONT, GTK_ICON_SIZE_BUTTON), "Benutzerschrift an/aus");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Enable/disable the user font"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_setUseFont), this);

	// toggle user colors
	toolItem = gtk_tool_button_new(colButtonImg = gtk_image_new_from_stock(GTK_STOCK_SELECT_COLOR, GTK_ICON_SIZE_BUTTON), "Benutzerfarben an/aus");
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Enable/disable the user colors"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_setUseColors), this);

	// separator
	gtk_toolbar_insert(GTK_TOOLBAR(tools), gtk_separator_tool_item_new(), -1);


	// kick user
	toolItem = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_STOP, GTK_ICON_SIZE_BUTTON), "Benutzer rauswerfen");
	// implement this feature when it is working in the licq-daemon
	gtk_widget_set_sensitive(GTK_WIDGET(toolItem), FALSE);
	gtk_tool_item_set_tooltip(toolItem, tips, tr("Kick the user out of this chat"), NULL);
	gtk_toolbar_insert(GTK_TOOLBAR(tools), toolItem, -1);
	g_signal_connect_swapped(toolItem, "clicked", G_CALLBACK(this->cb_kickUserButtonClicked), this);

	gtk_box_pack_start(GTK_BOX(tb), name, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(tb), tools, TRUE, TRUE, 0);
	
	return tb;	
}



// FIXME: works only with icq contacts
void chatWindowRemoteView::kickUser(gchar* tokick, gulong uin)
{
	gint res;
	gchar *tmp;
	
	if (!tokick)
		tmp = g_strdup_printf(tr_NOOP("Are you sure that you want to throw %s\nout of the chat session?"), alias);
	else
		tmp = g_strdup_printf(tr_NOOP("%s wants to throw %s out of the chat\nDo you agree?"), alias, tokick);

	res = u_showYesNoDialog(tr("Kick someone"), tr(tmp), GTK_STOCK_DIALOG_QUESTION, FALSE);


	if (tokick)
	{
		if (res == GTK_RESPONSE_YES)
			((IMChatManager*)parentWindow->manager)->sendKickAnswer(uin, TRUE);
		else
		if (res == GTK_RESPONSE_NO)
			((IMChatManager*)parentWindow->manager)->sendKickAnswer(uin, FALSE);
	} else
	if (res == GTK_RESPONSE_YES)
		((IMChatManager*)parentWindow->manager)->kickUser(strtoul(userID, NULL, 10));
		
}


GtkMenu* chatWindowRemoteView::createUserMenu()
{
	if (userMenu)
		return userMenu;
	
	u_menuItem userMenuList[]=
	{
		{UMENU_NORMAL, UE_MESSAGE				, ICON_MESSAGE	,tr_NOOP("Message")					, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_URL					, ICON_URL		,tr_NOOP("URL")						, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_FILETRANSFERREQUEST	, ICON_FILE		,tr_NOOP("Files")					, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_CONTACTSREQUEST		, ICON_INFO		,tr_NOOP("Contacts")				, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, UE_UNDEFINED, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_AUTHORIZATIONREQUEST	, ICON_INFO		,tr_NOOP("Authorization Request") 	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_AUTHORIZATIONGRANT	, ICON_INFO		,tr_NOOP("Authorization") 			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, UE_UNDEFINED, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_USERINFO				, ICON_INFO		,tr_NOOP("User information")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, UE_HISTORY				, ICON_HISTORY	,tr_NOOP("History")					, NULL, TRUE, FALSE, 0, NULL}
	};

	return (userMenu = u_createMenuList(userMenuList, 10, (GtkSignalFunc)this->cb_menuItemClicked, 0, this, TRUE));
}


	


void chatWindowRemoteView::insertCharacter(gchar* character)
{
	GtkTextBuffer *buf;
	GtkTextIter	  iter;
	gchar		  *tmp;
	
	tmp = convertToSystemCharset(character, charset);
	
	g_string_append(currentLine, character);
	g_string_append(history, character);
	
	if (textBox)
	{		
		buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textBox));
		gtk_text_buffer_get_end_iter(buf, &iter);
		gtk_text_buffer_insert_with_tags(buf, &iter, tmp, strlen(tmp), fontProperties, NULL);
		
		// move down
		gtk_text_buffer_get_end_iter(buf, &iter);
		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textBox), &iter, 0, FALSE, 0, 0);
	}
	
	g_free(tmp);
}




void chatWindowRemoteView::deleteCharacter()
{
	GtkTextBuffer *buf;
	GtkTextIter    startIter, endIter;
	gchar		  *tmp;
	
	if (!textBox)
		return;

	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textBox));
	gtk_text_buffer_get_end_iter(buf, &endIter);
	
	startIter = endIter;
	if (gtk_text_iter_backward_cursor_position(&startIter))
	{
		// one character can be longer than one byte, so more text has to be deleted
		tmp = gtk_text_buffer_get_text(buf, &startIter, &endIter, TRUE);
		g_string_truncate(currentLine, strlen(currentLine->str)-strlen(tmp));
		g_string_truncate(history, strlen(history->str)-strlen(tmp));

		g_free(tmp);
		gtk_text_buffer_delete(buf, &startIter, &endIter);
		
		
		// move down
		gtk_text_buffer_get_end_iter(buf, &endIter);
		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textBox), &endIter, 0, FALSE, 0, 0);
	}
}


gchar* chatWindowRemoteView::getCurrentLineAndClear()
{
	gchar* tmp;
	
	tmp = g_string_free(currentLine, FALSE);
	
	currentLine = g_string_new("");
	
	return tmp;
}





void chatWindowRemoteView::cb_setUseFont(chatWindowRemoteView *crv)
{
	if (!crv->fontProperties)
		return;
	
	if (crv->useUserFont)
	{
		g_object_set(G_OBJECT(crv->fontProperties),
					 "weight-set", FALSE,
					 "style-set", FALSE,
					 "underline-set", FALSE,
					 "strikethrough-set", FALSE,
					 "size-set", FALSE,
					 "family-set", FALSE,
					 NULL);
		
		crv->useUserFont = FALSE;
	} else
	{
		g_object_set(G_OBJECT(crv->fontProperties),
					 "weight-set", TRUE,
					 "style-set", TRUE,
					 "underline-set", TRUE,
					 "strikethrough-set", TRUE,
					 "size-set", TRUE,
					 "family-set", TRUE,
					 NULL);		
	
		crv->useUserFont = TRUE;
	}
	
	gtk_widget_set_sensitive(crv->fontButtonImg, crv->useUserFont);	
}




void chatWindowRemoteView::cb_setUseColors(chatWindowRemoteView *crv)
{
	if (!crv->fontProperties)
		return;
	
	if (crv->useUserColors)
	{
		g_object_set(G_OBJECT(crv->fontProperties),
					 "foreground-set", FALSE,
					 "background-set", FALSE,
					 NULL);
		
		crv->useUserColors = FALSE;
	} else
	{
		g_object_set(G_OBJECT(crv->fontProperties),
					 "foreground-set", TRUE,
					 "background-set", TRUE,
					 NULL);		

		crv->useUserColors = TRUE;
	}	

	gtk_widget_set_sensitive(crv->colButtonImg, crv->useUserColors);	
	crv->updateStyle(NULL);	
}




void  chatWindowRemoteView::cb_menuItemClicked(GtkMenuItem* item, chatWindowRemoteView* crv)
{
	guint menu, 
		  selection;
	
	menu = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(item), "ICQmenuMENU"));
	selection = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(item), "ICQmenuID"));
		
	switch(menu)
	{
		case 0:
			crv->parentWindow->manager->callUser(selection, NULL);
			break;
	}
}



void chatWindowRemoteView::cb_kickUserButtonClicked(chatWindowRemoteView *crv)
{
	crv->kickUser(NULL, 0);
}
