#include "contactlist.h"

gboolean c_useRealNames = FALSE;
gboolean c_useColors = FALSE;
gboolean c_blinkEvents = FALSE;
GdkColor c_vlistCol,
		 c_noMsgCol,
		 c_idleCol,
		 c_groupsCol;
GList	 *c_sortRules = NULL;
gulong	 c_extendedIconsMask = 0;

contactListEntry::contactListEntry(gchar* _id, gpointer _data)
{
	_id ? id = g_strdup(_id): id = NULL;
	data 	 = _data;
	children = NULL;
	iter 	 = NULL;
	model 	 = NULL;
	parent	 = NULL;
	selected = FALSE;
	type	 = CE_UNDEFINED;
}



contactListEntry::~contactListEntry()
{
	contactListEntry	*entry;
	GList				*search,
						*childrenCopy;
		
	if (id)
		g_free(id);
	
	childrenCopy = g_list_copy(children);
	
	// delete all children
	for (search = childrenCopy; search; search = search->next)
	{
		entry = (contactListEntry*)search->data;
		delete entry;
	}
	
	// unrealize the entry and free everything
	unrealizeEntry();
	g_list_free(children); 
	g_list_free(childrenCopy);
	
	if (parent)
		parent->removeChild(this);
}

	

void  contactListEntry::addChild(contactListEntry* child)
{
	child->parent	= this;
	child->widget	= widget;
	child->viewType	= viewType;
	child->model	= model;
	children = g_list_append(children, child);
}



void contactListEntry::removeChild(contactListEntry* child)
{
	child->model = NULL;
	children = g_list_remove(children, child);
}


void contactListEntry::deleteAllChildren()
{
	GList				*search,
						*childrenCopy;
	contactListEntry	*entry;
	
	childrenCopy = g_list_copy(children);
	
	for (search = childrenCopy; search; search = search->next)
	{		
		entry = (contactListEntry*)search->data;
		delete entry;
	}

	g_list_free(children);
	g_list_free(childrenCopy);
	children = NULL;
}


gboolean contactListEntry::hasChildren()
{
	return children != NULL;
}


void contactListEntry::childUpdated(contactListEntry* child)
{
	// child was updated - do something
}
		

contactListEntry* contactListEntry::findChildByID(gchar* _id, contactType cType)
{
	GList				*search;
	contactListEntry	*child;
	
	for (search = children; search; search = search->next)
	{
		child = (contactListEntry*)search->data;
		
		if (child->type == cType && !strcmp(child->id, _id))
			return child;
		
		if ((child = child->findChildByID(_id, cType)))
			return child;
	}
	
	return NULL;
}


contactListEntry* contactListEntry::getParent()
{
	return parent;
}



GList* contactListEntry::getAllChildrenOfType(contactType toFind, GList* startList)
{
	GList				*search;
	contactListEntry	*entry;
	
	for (search = children; search; search = search->next)
	{
		entry = (contactListEntry*)search->data;
		if (entry->type == toFind)
			startList = g_list_append(startList, entry);
		if (entry->hasChildren())
			startList = entry->getAllChildrenOfType(toFind, startList);
	}
	
	return startList;
}


GList* contactListEntry::getChildren()
{
	return children;
}
		

void contactListEntry::realizeEntry(gboolean realizeChildren, gboolean realizeThisEntry)
{
	GList		*search;
	

	
	// inform the parent that something has changed
	if (parent)
		parent->childUpdated(this);
	
	if (parent && parent->type == CE_GROUP && !((contactListGroup*)parent)->visible)
		return;
	
	
	if (realizeThisEntry && model && canBeVisible() && !iter)
	{
		iter = new GtkTreeIter;
	
		if (GTK_IS_TREE_STORE(model))
			gtk_tree_store_append(GTK_TREE_STORE(model), iter, parent? parent->iter: NULL);
		else
			gtk_list_store_append(GTK_LIST_STORE(model), iter);
	
		updateEntry(FALSE);
	}
		
	if (realizeChildren)
		for (search = children; search; search = search->next)
			((contactListEntry*)search->data)->realizeEntry(TRUE);
	/*	
	// update the parent
	if (parent)
		parent->updateEntry(FALSE); */
}


void contactListEntry::unrealizeEntry()
{
	GList	*search;
	GtkTreeIter	*iterTmp;
	
	iterTmp = iter;
	iter = NULL;	
	
	// also unrealize the children
	for (search = children; search; search = search->next)
		((contactListEntry*)search->data)->unrealizeEntry();	
	
	if (iterTmp && model)
		gtk_list_store_remove(GTK_LIST_STORE(model), iterTmp);
	
	// update the parent
	if (parent)
		parent->updateEntry(FALSE);

	delete iterTmp;	
}

void contactListEntry::updateEntry(gboolean updateChildren)
{
	GList	*search;

	if (updateChildren)
		for (search = children; search; search = search->next)
			((contactListEntry*)search->data)->updateEntry(TRUE);
}


void contactListEntry::updateParents()
{
	if (!parent)
		return;
	
	parent->updateEntry(FALSE);
	parent->updateParents();
}


guint contactListEntry::getNumOnline()
{
	GList	*search;
	guint	num;
	
	num = 0;
	
	for (search = children; search; search = search->next)
		num+=((contactListEntry*)search->data)->getNumOnline();

	return num;	
}


guint contactListEntry::getNumRecursiveChildren()
{
	GList	*search;
	guint	num;
	
	num = 0;
	
	// don't count groups
	if (type != CE_GROUP)
		num++;
	
	for (search = children; search; search = search->next)
		num+=((contactListEntry*)search->data)->getNumRecursiveChildren();
		
	return num;
}


gboolean contactListEntry::canBeVisible()
{
	return FALSE;
}




void contactListEntry::setViewMode(contactViewType _viewType)
{
	GList	*search;
	
	viewType = _viewType;
	
	for (search = children; search; search = search->next)
		((contactListEntry*)search->data)->setViewMode(viewType);
}


void contactListEntry::setTreeModel(GtkTreeModel* _model)
{
	GList	*search;
	
	model = _model;
	
	for (search = children; search; search = search->next)
		((contactListEntry*)search->data)->setTreeModel(model);	
}



void contactListEntry::setSelected(gboolean setting)
{
	GList	*search;
	
	selected = setting;
	
	if (model && iter)
	{
		gtk_list_store_set(GTK_LIST_STORE(model), iter,
						   CL_SELECTED, setting,
						   -1);
	}
	
	// also select the children
	for (search = children; search; search = search->next)
		((contactListEntry*)search->data)->setSelected(setting);
}


gboolean contactListEntry::getSelected()
{
	return selected;
}


GList* contactListEntry::getSelectedChildren(contactType toFind, GList* startList)
{
	GList				*search;
	contactListEntry	*child;

	for(search = children; search; search = search->next)
	{
		child = (contactListEntry*)search->data;
		if (child->type == toFind && child->selected)
			// don't get entries more than once
			// if the child is a user and not the first in the shadows list
			// then it's a shadow of an entry that is already added
			if (!(child->type == CE_USER && ((contactListUser*)child)->shadows && ((contactListUser*)child)->shadows->data != child))
			startList = g_list_append(startList, child);
		
		startList = child->getSelectedChildren(toFind, startList);
	}
	
	return startList;
}

		

// =============================================================================

contactListGroup::contactListGroup(gchar* _name, gchar* _id, gpointer _data):
				  contactListEntry(_id, _data)
{
	_name ? name 	= g_markup_escape_text(_name, strlen(_name)): name = NULL;
	isOpen 			= FALSE;
	type 			= CE_GROUP;
	textAttributes 	= NULL;
	arrow 			= i_getIcons()->getEventPixbuf(ARROW_CLOSED1);
	arrowSelected 	= FALSE;
	visible			= TRUE;
	
	quickMenuEntry = gtk_check_menu_item_new_with_label(_name);
	g_object_set_data(G_OBJECT(quickMenuEntry), "icqnd-qm-owner", this);
}


contactListGroup::~contactListGroup()
{
	if (name)
		g_free(name);
	
	gtk_widget_destroy(quickMenuEntry);
}
		

void contactListGroup::openGroup(gboolean open)
{
	GList		*search;

	isOpen = open;
	
	if (iter && model)
	{

		// stop sorting (by using a column id with no sorting function specified)
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 2, GTK_SORT_ASCENDING);

		for(search = children; search; search = search->next)
			if (open)
				((contactListEntry*)search->data)->realizeEntry(TRUE, TRUE);
			else
				((contactListEntry*)search->data)->unrealizeEntry();	
			
		// go on sorting
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 0, GTK_SORT_ASCENDING);
			
	}
	
	if (isOpen)
	{
		if (arrowSelected)
			arrow = i_getIcons()->getEventPixbuf(ARROW_OPEN2);
		else
			arrow = i_getIcons()->getEventPixbuf(ARROW_OPEN1);
	}
	else
	{
		if (arrowSelected)
			arrow = i_getIcons()->getEventPixbuf(ARROW_CLOSED2);
		else
			arrow = i_getIcons()->getEventPixbuf(ARROW_CLOSED1);
	}
	
	updateEntry(FALSE);
}


gboolean contactListGroup::isGroupOpen()
{
	return isOpen;
}


void contactListGroup::removeChild(contactListEntry* child)
{
	contactListEntry::removeChild(child);
	
	// may we still be visible?
	if (!canBeVisible())
		unrealizeEntry();
}

		

void contactListGroup::updateEntry(gboolean updateChildren)
{
	gchar			*newText,
					*strippedText;
	guint			numOnline,
					numChildren;
	
	// only update group when our store is a tree store
	if (iter && model)
	{
		numOnline = getNumOnline();	
		numChildren = getNumRecursiveChildren();

		newText = g_strdup_printf("<b>%s</b> <small>(%d / %d)</small>",
					   		 	 name,
					   		 	 numOnline,
					   		 	 numChildren);
		
		if (textAttributes)
			pango_attr_list_unref(textAttributes);
		pango_parse_markup(newText, -1, 0, &textAttributes, &strippedText, NULL, NULL);
		
		
		gtk_list_store_set(GTK_LIST_STORE(model), iter,
				  	  	 CL_ALIAS, strippedText,
						CL_TEXTATTR, textAttributes,
					   	CL_ENTRYPOINTER, this,
					   	CL_ICONVISIBLE, TRUE,
						CL_ICON, arrow,	
					    CL_BGCOLORACTIVE, TRUE,
					    CL_BGCOLOR, &c_groupsCol,		
				   	   	-1);
		
		g_free(newText);
		g_free(strippedText);
		
		// also update the quick tool bar entry
		gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(quickMenuEntry))), name);
	}
	
	contactListEntry::updateEntry(updateChildren);
}


gboolean contactListGroup::canBeVisible()
{
	// are we generally visible?
	if (!visible)
		return FALSE;
	
	// do we want to show groups?
	if (!(viewType & CVIEW_GROUPS))
		return FALSE;
	
	// do we want to show offline users
	if (viewType & CVIEW_NOEMPTYGROUPS && !children)
		return FALSE;
	
	if (viewType & CVIEW_HIDEOFFLINEGROUPS && !getNumOnline())
		return FALSE;
	
	return TRUE;
}





void contactListGroup::setVisible(gboolean _visible)
{
	visible = _visible;
	if (visible && !iter)
		realizeEntry(TRUE, TRUE);
	
	if (!visible && iter)
		unrealizeEntry();
}


void contactListGroup::childUpdated(contactListEntry* child)
{
	gboolean	cbv;
	// can we still be visible?
	
	cbv = canBeVisible();
	if (cbv && !iter)
		realizeEntry(TRUE);
	else
	if (!cbv && iter)
		unrealizeEntry();
	
	// just update
	updateEntry(FALSE);
}



// =============================================================================

contactListUser::contactListUser(IMUserDaemon *user, GList *_shadows):
				 contactListEntry(user? user->info->licqID: NULL, user)
{
	icon 	  	 	= NULL;
	iconBlink 	 	= NULL;
	status	  	 	= BUDDY_STATUS_OFFLINE;
	type	  	 	= CE_USER;
	shadows	  		= _shadows;
	blinkSource  	= 0;
	flashSource		= 0;
	flashLetters	= 0;
	enableBgColor	= FALSE;
	enableFgColor	= FALSE;
	stateMask		= 0;
	
	
	textAttributes  = pango_attr_list_new();
	
	textItalic = pango_attr_style_new(PANGO_STYLE_ITALIC);
	textItalic->start_index = 0;
	textItalic->end_index = 0;
	textBold = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
	textBold->start_index = 0;
	textBold->end_index = 0;
	textColor = pango_attr_foreground_new(0, 0, 0);
	textColor->start_index = 0;
	textColor->end_index = 0;
	
	pango_attr_list_insert(textAttributes, textItalic);
	pango_attr_list_insert(textAttributes, textBold);
	pango_attr_list_insert(textAttributes, textColor);
	
}


contactListUser::~contactListUser()
{
	if (blinkSource)
		g_source_remove(blinkSource);
	pango_attr_list_unref(textAttributes);
}
		

void contactListUser::updateEntry(gboolean updateChildren)
{
	IMUserDaemon	*user;
	gchar			*nameToInsert,
					*flashName;
		
	if (!iter || !model)
		return;
	
	user = (IMUserDaemon*)data;
	
	if (c_useRealNames)
		nameToInsert = uu_getRealNameOrAlias(user->info, FALSE);
	else
		nameToInsert = g_strdup(user->info->alias);
	
	// shall we change the letters?
	if (flashLetters != 0)
	{
		if (flashLetters == 1)
			flashName = g_utf8_strup(nameToInsert, -1);
		else
			flashName = g_utf8_strdown(nameToInsert, -1);
		
		g_free(nameToInsert);
		nameToInsert = flashName;
	}
	
	gtk_list_store_set(GTK_LIST_STORE(model), iter,
					   CL_ICON, iconBlink? iconBlink: icon,
					   CL_ICONVISIBLE, TRUE,
					   CL_ALIAS, nameToInsert,
					   CL_TEXTATTR, textAttributes,
					   CL_ORDER, 0,
					   CL_BGCOLORACTIVE, enableBgColor,
					   CL_BGCOLOR, enableBgColor? &backgroundColor: NULL,
					   CL_SELECTED, selected,
					   CL_STATEICONS, stateMask,
					   CL_ENTRYPOINTER, this,
					   -1);
	
	g_free(nameToInsert);
}


void contactListUser::updateColors()
{
	IMUserDaemon	*user;
	GdkColor		*col;
	
	user = (IMUserDaemon*)data;
		
	if (!user->info->stats.lastReceived)
		col = &c_noMsgCol;	
	else
	if (user->info->stats.idleSince)
		col = &c_idleCol;
	else		
	if (user->info->onVisibleList)
		col = &c_vlistCol;
	else
		col = NULL;
	
	if (c_useColors && col)
	{
		setForegroundColor(*col);
		setForegroundColorEnabled(TRUE);
	} else
		setForegroundColorEnabled(FALSE);
}



gboolean contactListUser::canBeVisible()
{
	IMUserDaemon		*user;
	contactListGroup	*group;
	GList				*search;
	// check whether our entry can be visible
	
	// can our parent be visible?
//	if (parent && !parent->canBeVisible())
//		return FALSE;
	
	user = (IMUserDaemon*)data;
	
	// do we hide offline users and we ARE offline?
	if (viewType & CVIEW_HIDEOFFLINE && 
		user && 
		user->info->status == BUDDY_STATUS_OFFLINE)
		return FALSE;
	
	// is our group closed?
	group = (contactListGroup*)parent;
	
	if (viewType & CVIEW_GROUPS && group && group->type == CE_GROUP && !group->isGroupOpen())
		return FALSE;
	
	// do we want to see groups? if not only one instance may be shown
	
	if (!(viewType & CVIEW_GROUPS))
	for (search = shadows; search; search = search->next)
		if (((contactListEntry*)search->data)->iter)
			return FALSE;

	
	// everything okay - we should be able to be visible
	return TRUE;
}



void contactListUser::setSelected(gboolean setting)
{
	GList	*search;
	
	// FIXME: normally a loop is created when every shadow calls every
	//		  other shadow. This will avoid it. find a better solution
	if (selected == setting)
		return;
	
	
	contactListEntry::setSelected(setting);
	
	// Also set all shadows selected
	for (search = shadows; search; search = search->next)
		((contactListUser*)search->data)->setSelected(setting);
}



guint contactListUser::getNumOnline()
{
	guint num;
	IMUserDaemon* user;
	
	user = (IMUserDaemon*)data;
	if (user && user->info->status != BUDDY_STATUS_OFFLINE)
		num = 1;
	else
		num = 0;
	
	num+=contactListEntry::getNumOnline();
	
	return num;
}


void contactListUser::setStatus(guint _status, GdkPixbuf *_icon)
{
	gboolean	cbv;
	
	icon = _icon;
	
	// check whether we still can be visible
	cbv = canBeVisible();
	
	// can we be visible but don't have an iter?
	if (cbv && !iter)
		realizeEntry(TRUE, TRUE);
	else
	// can't we be visible but DO have an iter?
	if (!cbv && iter)
		unrealizeEntry();

	if (parent)
		parent->childUpdated(this);	
		
	if (!iter || !model)
		return;
	
		
	gtk_list_store_set(GTK_LIST_STORE(model), iter,
					   CL_ICON, icon,
					   -1);	
}


void contactListUser::setStateMask(gulong newMask)
{
	stateMask = newMask;
	
	if (model && iter)
	{
		gtk_list_store_set(GTK_LIST_STORE(model), iter,
						   CL_STATEICONS, stateMask,
						   -1);
	}
}
		

void contactListUser::blinkIcon(GdkPixbuf *_iconBlink)
{
	iconBlink = _iconBlink;
	
	if (!iconBlink)
	{
		if (model && iter)
		{
			updateEntry(FALSE);
		}
		
		if (blinkSource)
		{
			g_source_remove(blinkSource);
			blinkSource = 0;
		}			
	} else
	{
		blinkTimeoutIcon = FALSE;
		if (!c_blinkEvents && blinkSource)
		{
			g_source_remove(blinkSource);
			blinkSource = 0;
		}
		if (c_blinkEvents && !blinkSource)
			blinkSource = g_timeout_add(300, (GSourceFunc)this->cb_blinkIconTimeout, this);
	}
}


void contactListUser::flashName(contactFlashType ftType)
{
	flashRules = ftType;
	
	if (flashSource)
		g_source_remove(flashSource);
	
	flashIterator = 0;
	
	flashSource = g_timeout_add(300, (GSourceFunc)this->cb_flashNameTimeout, this);
}


void contactListUser::stopFlashName()
{
	if (flashSource)
	{
		g_source_remove(flashSource);
		flashSource = 0;
	}
	
	textItalic->end_index = 0;
	textBold->end_index = 0;
	flashLetters = 0;
	
	((PangoAttrColor*)textColor)->color.red = foregroundColor.red;
	((PangoAttrColor*)textColor)->color.green = foregroundColor.green;
	((PangoAttrColor*)textColor)->color.blue = foregroundColor.blue;
	if (enableFgColor)
		textColor->end_index = 0xFFFF;
	else
		textColor->end_index = 0;
	
	updateEntry(FALSE);
	
}


void contactListUser::setFlashColor(GdkColor col)
{
	flashColor.red = col.red;
	flashColor.green = col.green;
	flashColor.blue = col.blue;
}


void contactListUser::setBackgroundColor(GdkColor bgColor)
{
	backgroundColor.red 	= bgColor.red;
	backgroundColor.blue 	= bgColor.blue;
	backgroundColor.green 	= bgColor.green;
	
	if (model && iter && enableBgColor)
	{
		gtk_list_store_set(GTK_LIST_STORE(model), iter,
						   CL_BGCOLOR, &backgroundColor,
						   -1);
	}
}


void contactListUser::setBackgroundColorEnabled(gboolean setting)
{
	enableBgColor = setting;
		
	if (model && iter)
	{
		gtk_list_store_set(GTK_LIST_STORE(model), iter,
						   CL_BGCOLORACTIVE, enableBgColor,
						   CL_BGCOLOR, enableBgColor? &backgroundColor: NULL,
						   -1);
	}
}


gboolean contactListUser::getBackgroundColorEnabled()
{
	return enableBgColor;
}


void contactListUser::setForegroundColor(GdkColor fgColor)
{
	foregroundColor.red = fgColor.red;
	foregroundColor.blue = fgColor.blue;
	foregroundColor.green = fgColor.green;
	
	((PangoAttrColor*)textColor)->color.red = fgColor.red;
	((PangoAttrColor*)textColor)->color.green = fgColor.green;
	((PangoAttrColor*)textColor)->color.blue = fgColor.blue;
}


void contactListUser::setForegroundColorEnabled(gboolean setting)
{
	enableFgColor = setting;
	textColor->start_index = 0;
	
	if (setting)
		textColor->end_index = 0xFFFF;
	else
		textColor->end_index = 0;
}


gboolean contactListUser::getForegroundColorEnabled()
{
	return enableFgColor;
}


gboolean contactListUser::cb_flashNameTimeout(contactListUser* self)
{
	guchar			flashMult;
		
	self->flashIterator++;
	
	flashMult = self->flashIterator%2;
		
	// make text bold
	if (self->flashRules & CF_WEIGHT)
	{
		self->textBold->start_index = 0;
		self->textBold->end_index = 0xFFFF * flashMult;
	}
	
	// make text italic
	if (self->flashRules & CF_STYLE)
	{
		self->textItalic->start_index = 0;
		self->textItalic->end_index = 0xFFFF * flashMult;
	}
	
	// set the color
	if (self->flashRules & CF_COLOR)
	{
		if (flashMult)
		{
			((PangoAttrColor*)self->textColor)->color.red = self->flashColor.red;
			((PangoAttrColor*)self->textColor)->color.green = self->flashColor.green;
			((PangoAttrColor*)self->textColor)->color.blue = self->flashColor.blue;
			self->textColor->end_index = 0xFFFF;
		} else
		{
			((PangoAttrColor*)self->textColor)->color.red = self->foregroundColor.red;
			((PangoAttrColor*)self->textColor)->color.green = self->foregroundColor.green;
			((PangoAttrColor*)self->textColor)->color.blue = self->foregroundColor.blue;
			if (self->enableFgColor)
				self->textColor->end_index = 0xFFFF;
			else
				self->textColor->end_index = 0;
		}
	}
	
	// set the letters
	if (self->flashRules & CF_LETTERS)
		self->flashLetters = flashMult+1;
	
	self->updateEntry(FALSE);
	
	
	if (self->flashIterator < 8)
		return TRUE;
	else
	{
		self->stopFlashName();
		return FALSE;
	}
}



gboolean contactListUser::cb_blinkIconTimeout(contactListUser* self)
{
	self->blinkTimeoutIcon = !self->blinkTimeoutIcon;
	
	// FIXME: this is not very efficient
	if (!self->model || !self->iter)
		return TRUE;
	
	if (self->blinkTimeoutIcon)
	{
		gtk_list_store_set(GTK_LIST_STORE(self->model), self->iter,
						   CL_ICON, self->iconBlink,
						   -1);
		
	} else
	{
		gtk_list_store_set(GTK_LIST_STORE(self->model), self->iter,
						   CL_ICON, self->icon,
						   -1);
	}
	
	return TRUE;
}
	

// =============================================================================


contactList::contactList():
			 contactListEntry(NULL, NULL)
{
	GtkCellRenderer		*renderer;
	gboolean			showExtendedIcons;
	GtkWidget			*scroll;
	
	
	// get all settings and add us to the changed function
	settings_getSettings()->getProperties("contactlist", 
										  "blinkEvents", &c_blinkEvents,
										  "showRealNames", &c_useRealNames,
										  "showExtendedIcons", &showExtendedIcons,
										  "extendedIconsMask", &c_extendedIconsMask,
										  "showColors", &c_useColors,
										  "contactVListColor", &c_vlistCol,
										  "contactNoMsgColor", &c_noMsgCol,
										  "contactIDLEColor", &c_idleCol,
										  "contactGroupsColor", &c_groupsCol,
										  "contactsFont", &font,
										  "contactsUseOwnFont", &useOwnFont,
										  NULL);

	contactList_updateSortRules();
	
	settings_getSettings()->installListener("contactlist", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
	
	
	// create the list store
	listStore = gtk_list_store_new(10,
								   G_TYPE_BOOLEAN,
								   GDK_TYPE_PIXBUF,
								   G_TYPE_BOOLEAN,
								   G_TYPE_STRING,
								   G_TYPE_ULONG,
								   GDK_TYPE_COLOR,
								   G_TYPE_BOOLEAN,
								   G_TYPE_ULONG,
								   PANGO_TYPE_ATTR_LIST,
								   G_TYPE_POINTER);
	
	// append the sorting function
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(listStore), 0,
									(GtkTreeIterCompareFunc)this->cb_sortRowsFunction,
									this, NULL);
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(listStore), 0, GTK_SORT_ASCENDING);
									
	
	
	widget	= gtk_tree_view_new();
	g_signal_connect_swapped(widget, "row-collapsed", G_CALLBACK(this->cb_rowExpandedCollapsed), this);
	g_signal_connect_swapped(widget, "row-expanded", G_CALLBACK(this->cb_rowExpandedCollapsed), this);
	g_signal_connect(widget, "row-activated", G_CALLBACK(this->cb_rowActivated), this);
	g_signal_connect_swapped(widget, "motion-notify-event", G_CALLBACK(this->cb_viewMotionEvent), this);
	g_signal_connect_swapped(widget, "leave-notify-event", G_CALLBACK(this->cb_viewMotionEvent), this);
	g_signal_connect_swapped(widget, "button-press-event", G_CALLBACK(this->cb_rowClicked), this);

	// create the selection checkbox
	checkboxColumn = gtk_tree_view_column_new();

	renderer = gtk_cell_renderer_toggle_new();
	g_signal_connect(renderer, "toggled", G_CALLBACK(this->cb_checkboxToggled), this);	
	
	gtk_tree_view_column_pack_start(checkboxColumn, renderer, FALSE);
	gtk_tree_view_column_set_attributes(checkboxColumn, renderer,
												"active", CL_SELECTED,
												"cell-background-gdk", CL_BGCOLOR,
												"cell-background-set", CL_BGCOLORACTIVE,
												NULL);	
	gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(checkboxColumn), selected);											
	gtk_tree_view_append_column(GTK_TREE_VIEW(widget), checkboxColumn);	 
	
					
	// create the other column
	column  = gtk_tree_view_column_new();
	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(widget), TRUE);
	gtk_tree_view_set_search_column(GTK_TREE_VIEW(widget), 3);	
	
	// make the icon field
	renderer = gtk_cell_renderer_pixbuf_new();
	g_object_set(G_OBJECT(renderer), "xpad", 1, NULL);
	gtk_tree_view_column_pack_start(column, renderer, FALSE);
	
	gtk_tree_view_column_set_attributes(column, renderer,
										"pixbuf", CL_ICON,
										"visible", CL_ICONVISIBLE,
										"cell-background-gdk", CL_BGCOLOR,
										"cell-background-set", CL_BGCOLORACTIVE,
										NULL);
										
		
	// make the alias field
	aliasRenderer = gtk_cell_renderer_text_pixbuf_new();
	// aliasRenderer = gtk_cell_renderer_text_new();

	stateIconList = NULL;
	updateExtendedIconsList();

	gtk_cell_renderer_text_pixbuf_set_distance(GTK_CELL_RENDERER_TEXT_PIXBUF(aliasRenderer), 5);
	gtk_cell_renderer_text_pixbuf_set_padding(GTK_CELL_RENDERER_TEXT_PIXBUF(aliasRenderer), 1);
	gtk_cell_renderer_text_pixbuf_set_use_mask(GTK_CELL_RENDERER_TEXT_PIXBUF(aliasRenderer), TRUE);
	
	setShowExtendedIcons(showExtendedIcons);
	
	gtk_tree_view_column_pack_start(column, aliasRenderer, TRUE);
	
	gtk_tree_view_column_set_attributes(column, aliasRenderer,
										"text", CL_ALIAS,
										"attributes", CL_TEXTATTR, 
										"mask", CL_STATEICONS,
										"cell-background-gdk", CL_BGCOLOR,
										"cell-background-set", CL_BGCOLORACTIVE,
										NULL); 
					
	if (useOwnFont)					
		g_object_set(G_OBJECT(aliasRenderer), "font", font, NULL);


	// append the column
	gtk_tree_view_append_column(GTK_TREE_VIEW(widget), column);	 	

	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(widget), FALSE);



	// set sorting function
/*	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), CSORT_STATUS, 
									cb_sortingFunction,
                                    NULL, NULL);
									
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), CSORT_STATUS, GTK_SORT_ASCENDING);
*/
	
	createQuickBar(); // (tm)
	
	
	// make a scrolling-window around the tree view
	
	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC );
								   
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scroll), widget);
	

	// pack the view and the quickBar into a vbox
	frame = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(frame), quickBar, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(frame), scroll, TRUE, TRUE, 0);
	
	
	
	// set the view mode according to the settings
	updateViewMode();
	
	
	oldPath = NULL;
	
}


contactList::~contactList()
{
	settings_getSettings()->removeListenerByData(this);
}


void contactList::createQuickBar()
{
	GtkToolItem		*showGroups,
					*showOffline;
	GtkWidget		*item;
	gchar			*labelText;
	
	quickBar = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(quickBar), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_icon_size(GTK_TOOLBAR(quickBar), GTK_ICON_SIZE_MENU);
	
	showOffline = gtk_tool_button_new_from_stock(ICON_SHOWOFFLINE);
	g_signal_connect_swapped(showOffline, "clicked", G_CALLBACK(cb_showOfflineButtonClicked), this);
	gtk_toolbar_insert(GTK_TOOLBAR(quickBar), showOffline, 0);
	gtk_tool_item_set_homogeneous(showOffline, FALSE);
		
	showGroups = gtk_tool_button_new_from_stock(ICON_GROUPS);
	g_signal_connect_swapped(showGroups, "clicked", G_CALLBACK(cb_showGroupsButtonClicked), this);
	gtk_toolbar_insert(GTK_TOOLBAR(quickBar), showGroups, 1);
	gtk_tool_item_set_homogeneous(showGroups, FALSE);
		
	
	labelText = g_strdup_printf("<small>%s</small>", tr("All Groups"));
	filterLabel = gtk_label_new(labelText);
	g_free(labelText);
	gtk_label_set_use_markup(GTK_LABEL(filterLabel), TRUE);
	gtk_misc_set_alignment(GTK_MISC(filterLabel), 0, 0);
	

	groupFilter = gtk_tool_button_new(filterLabel, tr("All Groups"));
	g_signal_connect_swapped(groupFilter, "clicked", G_CALLBACK(cb_groupFilterButtonClicked), this);
	gtk_tool_item_set_expand(groupFilter, TRUE);
	
	filterMenu = gtk_menu_new();
	
	item = gtk_separator_menu_item_new();
	gtk_menu_shell_append(GTK_MENU_SHELL(filterMenu), item);
	
	filterAllItem = gtk_check_menu_item_new_with_label(tr("All Groups"));
	g_signal_connect_swapped(filterAllItem, "activate", G_CALLBACK(cb_groupAllFilterButtonClicked), this);
	gtk_menu_shell_append(GTK_MENU_SHELL(filterMenu), filterAllItem);
		
	gtk_toolbar_insert(GTK_TOOLBAR(quickBar), groupFilter, 2);
}
	

void contactList::updateExtendedIconsList()
{
	iconManager*	iconMan;
	gint			i;
	
	if (stateIconList)
	{
		g_list_free(stateIconList);
		stateIconList = NULL;
	}
	
	iconMan = i_getIcons();
	for (i=0; i<IET_NUM; i++)
		if (c_extendedIconsMask & (1<<i))
			stateIconList = g_list_append(stateIconList, iconMan->getExtendedPixbuf(1<<i));
		else
			stateIconList = g_list_append(stateIconList, NULL);
	
	gtk_cell_renderer_text_pixbuf_set_list(GTK_CELL_RENDERER_TEXT_PIXBUF(aliasRenderer), stateIconList);
}



void contactList::updateViewMode()
{
	gboolean	showOfflineUsers,
				showGroups,
				showEmptyGroups;
	
	gint		newView;
	
	// Update the view mode
	settings_getSettings()->getProperties("contactlist", 
					  					 "showOfflineUsers", &showOfflineUsers,				
								 		 "showGroups", &showGroups,
								 		 "showEmptyGroups", &showEmptyGroups,
								 		 NULL);
					
	// Find out the new view mode			
	newView = 0;
	if (showGroups)
		newView = newView | CVIEW_GROUPS;
	if (!showEmptyGroups)
		newView = newView | CVIEW_NOEMPTYGROUPS;
	if (!showOfflineUsers)
		newView = newView | CVIEW_HIDEOFFLINE;
		
	// reset our view
	unrealizeEntry();
	setViewMode((contactViewType)newView);
	realizeEntry(TRUE);
}


void contactList::resortContactlist()
{
	if (model)
	{
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 2, GTK_SORT_ASCENDING);
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 0, GTK_SORT_ASCENDING);
	}
}



void contactList::setMultipleHighlight(gboolean setting)
{
	if (widget)
		 gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW (widget)),
				   					 setting? GTK_SELECTION_MULTIPLE: GTK_SELECTION_SINGLE);
}



contactListEntry* contactList::getFocusedEntry()
{
	contactListEntry* 	toreturn;
	GtkTreePath* 		path;
	GtkTreeIter  		iter;
	
	gtk_tree_view_get_cursor (GTK_TREE_VIEW(widget), &path, NULL);

	if (!path) 
		return NULL;
	
	toreturn = NULL;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
					   CL_ENTRYPOINTER, &toreturn, -1);
	
	return toreturn;	
}


GList* contactList::getMultipleFocusedEntries()
{
	GList 				*rows,
						*toReturn,
						*search;
	GtkTreePath			*path;
	GtkTreeIter			iter;
	contactListEntry	*entry;
	
	rows = gtk_tree_selection_get_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(widget)), NULL);
	
	toReturn = NULL;
	
	for (search = rows; search; search = search->next)
	{
		path = (GtkTreePath*)search->data;
		
		gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path);
		gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
					   	   CL_ENTRYPOINTER, &entry, -1);
		
		toReturn = g_list_append(toReturn, entry);
	}
	
	
	g_list_foreach(rows, (GFunc)gtk_tree_path_free, NULL);
	g_list_free(rows);
	
	return toReturn;
}



contactListEntry* contactList::getEntryAtCursor(gint x, gint y)
{
	contactListEntry* 	toreturn;
	GtkTreePath* 		path;
	GtkTreeIter  		iter;
	
	if (!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), x, y, &path, NULL, NULL, NULL))
		return NULL;

	toreturn = NULL;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
					   CL_ENTRYPOINTER, &toreturn, -1);
	
	return toreturn;		
}


contactListEntry* contactList::getEntryAtDropPosition(gint x, gint y)
{
	contactListEntry* 	toreturn;
	GtkTreePath* 		path;
	GtkTreeIter  		iter;
	
	if (!gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(widget), x, y, &path, NULL))
		return NULL;

	toreturn = NULL;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
					   CL_ENTRYPOINTER, &toreturn, -1);
	
	return toreturn;		
}


contactListEntry* contactList::getContextEntry(GdkDragContext* dc)
{
	GtkTreePath	*path;
	GtkTreeRowReference *ref;
	GtkTreeIter	iter;
	contactListEntry	*toreturn;
	
	ref = (GtkTreeRowReference*)g_object_get_data(G_OBJECT(dc), "gtk-tree-view-source-row");
	
	if (!ref)
		return NULL;
	
	path = gtk_tree_row_reference_get_path(ref);
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
					   CL_ENTRYPOINTER, &toreturn, -1);
	
	return toreturn;
}



void contactList::clearList()
{
	// ...
}


void contactList::setCheckSelectionMode(gboolean setting)
{
	selected = setting;
	gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(checkboxColumn), selected);	
}


void contactList::setShowExtendedIcons(gboolean setting)
{	
	if (setting)
		gtk_cell_renderer_text_pixbuf_set_list(GTK_CELL_RENDERER_TEXT_PIXBUF(aliasRenderer), stateIconList);
	else
		gtk_cell_renderer_text_pixbuf_set_list(GTK_CELL_RENDERER_TEXT_PIXBUF(aliasRenderer), NULL);		

	gtk_widget_queue_draw(widget);
}


void contactList::focusChild(contactListEntry* child)
{
	// ...
}


void contactList::realizeEntry(gboolean realizeChildren)
{
	GList	*search;

	setTreeModel(GTK_TREE_MODEL(listStore));	
	gtk_tree_view_set_model(GTK_TREE_VIEW(widget), GTK_TREE_MODEL(listStore));		
	
	for (search = children; search; search = search->next)
		((contactListEntry*)search->data)->realizeEntry(realizeChildren);
}


void contactList::cb_rowExpandedCollapsed(contactList *self, GtkTreeIter *eventIter, GtkTreePath *path)
{
	contactListEntry* entry;

	gtk_tree_model_get(self->model, eventIter,
					   CL_ENTRYPOINTER, &entry,
					   -1);

	
	if (entry)
		((contactListGroup*)entry)->isOpen = gtk_tree_view_row_expanded(GTK_TREE_VIEW(self->widget), path);
}



void contactList::cb_rowActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, contactList* self)
{
	contactListGroup	*entry;
	
	entry = (contactListGroup*)self->getFocusedEntry();
	
	if (entry->type == CE_GROUP)
	{
		if (entry->isGroupOpen())
			entry->openGroup(FALSE);
		else
			entry->openGroup(TRUE);
	}
}



gboolean contactList::cb_viewMotionEvent(contactList* self, GdkEvent* event)
{
	contactListGroup* 	entry;
	GtkTreePath* 		path;
	GtkTreeIter  		iter;
	GdkEventMotion		*motionEvent;
	gint				x,
						xMin,
						xMax,
						cRes;
	
	x = 0;
	xMin = self->selected? 16: 0;
	xMax = self->selected? 32: 16;
	cRes = 1;
	
	if (event->type == GDK_MOTION_NOTIFY)
	{
		motionEvent = (GdkEventMotion*)event;
		gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(self->widget), (gint)motionEvent->x, (gint)motionEvent->y, &path, NULL, NULL, NULL);

		x = (gint)motionEvent->x;
	} else
		path = NULL;
		
	// are we pointing on no entry or did we change the entry? -> restore the default
	if (self->oldPath && (!path ||
		(cRes = gtk_tree_path_compare(path, self->oldPath) || (x < xMin || x > xMax))))
	{
		gtk_tree_model_get_iter(GTK_TREE_MODEL(self->model), &iter, self->oldPath);
		gtk_tree_model_get(GTK_TREE_MODEL(self->model), &iter,
					   CL_ENTRYPOINTER, &entry, -1);
		
		if (entry->isOpen)
			entry->arrow = i_getIcons()->getEventPixbuf(ARROW_OPEN1);
		else
			entry->arrow = i_getIcons()->getEventPixbuf(ARROW_CLOSED1);
		
		entry->updateEntry(FALSE);		
		entry->arrowSelected = FALSE;
		
		gtk_tree_path_free(self->oldPath);
		self->oldPath = NULL;
	}
	
	
	// check whether we changed the group
	if (!path)
		return FALSE;
	
	// nothing more to do if we are still on the old entry
	if (!cRes)
	{
		gtk_tree_path_free(path);
		return FALSE;
	}

	
	// get the new entry
	gtk_tree_model_get_iter(GTK_TREE_MODEL(self->model), &iter, path);
	
	gtk_tree_model_get(GTK_TREE_MODEL(self->model), &iter, 
					   CL_ENTRYPOINTER, &entry, -1);	
	
	// set the new arrow if neccessary
	if (entry->type == CE_GROUP && (x>=xMin && x<=xMax))
	{	

		if (entry->isOpen)
			entry->arrow = i_getIcons()->getEventPixbuf(ARROW_OPEN2);
		else
			entry->arrow = i_getIcons()->getEventPixbuf(ARROW_CLOSED2);
		
		entry->updateEntry(FALSE);		
		entry->arrowSelected = TRUE;		
		
		self->oldPath = path;		
	} else
		gtk_tree_path_free(path);
	
	
	return FALSE;
}




gboolean contactList::cb_rowClicked(contactList* self, GdkEventButton* event)
{
	GtkTreePath			*path;
	contactListGroup	*entry;
	GtkTreeIter			iter;
	
	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(self->widget), (gint)event->x, (gint)event->y, &path, NULL, NULL, NULL);

	if (!path)
		return FALSE;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(self->model), &iter, path);
	gtk_tree_model_get(GTK_TREE_MODEL(self->model), &iter,
					   CL_ENTRYPOINTER, &entry, -1);	
	
	gtk_tree_path_free(path);	
	
	if (!entry->arrowSelected)
		return FALSE;
	
	if (entry->isOpen)
		entry->openGroup(FALSE);
	else
		entry->openGroup(TRUE);
		
	
	return FALSE;
}




gint contactList::cb_sortRowsFunction(GtkTreeModel* model, GtkTreeIter *iter1, GtkTreeIter *iter2, contactList* self)
{	
	contactListUser	*entry1,
					*entry2;
	IMUserDaemon	*user1,
					*user2;
	GList			*search;
	gchar			*n1,
					*n2;
	gint			nResult;
		
	if (!iter1 || !iter2)
		return 0;

	entry1 = NULL;
	entry2 = NULL;
	
	gtk_tree_model_get(model, iter1, CL_ENTRYPOINTER, &entry1, -1);
	gtk_tree_model_get(model, iter2, CL_ENTRYPOINTER, &entry2, -1);

	if (!entry1 || !entry2)
		return 0;
	
	// group sorting
	if (self->viewType & CVIEW_GROUPS)
	{		
		if (entry1->type == CE_GROUP && entry2->type == CE_GROUP)
		{	
			if (((groupInfo*)entry1->data)->num > ((groupInfo*)entry2->data)->num)
				return 1;
			else
			if (((groupInfo*)entry1->data)->num < ((groupInfo*)entry2->data)->num)
				return -1;
		
			return 0;
		}	
	
		if (entry1->type == CE_GROUP)
		{
			if (entry2->parent == entry1)
				return -1;
		
			nResult = ((groupInfo*)entry1->data)->num - ((groupInfo*)entry2->parent->data)->num;
		
			return nResult;
		}


		if (entry2->type == CE_GROUP)
		{
			if (entry1->parent == entry2)
				return 1;

			nResult = ((groupInfo*)entry1->parent->data)->num - ((groupInfo*)entry2->data)->num;	
		
			return nResult;
		}
	
		
		// check whether the two entries belong to the same group and resort them
		// if not
		if (entry1->parent != entry2->parent)
		{
			entry1 = (contactListUser*)entry1->parent;
			entry2 = (contactListUser*)entry2->parent;
				
			if (((groupInfo*)entry1->data)->num > ((groupInfo*)entry2->data)->num)
				return 1;
			else
			if (((groupInfo*)entry1->data)->num < ((groupInfo*)entry2->data)->num)
				return -1;
		
			return 0;
		}
	}
	

	// sort the status of the entries
	user1 = (IMUserDaemon*)entry1->data;
	user2 = (IMUserDaemon*)entry2->data;
	
	if (!user1 || !user2)
		return 0;
	
	for (search = c_sortRules; search; search = search->next)
	{
		
		switch(GPOINTER_TO_UINT(search->data))
		{
			// sort the status of a user			
			case CS_STATUS:
				if (user1->info->status < user2->info->status)
					return -1;
				else
				if (user1->info->status > user2->info->status)
					return 1;	

				break;
			
			// sort the last received event				
			case CS_LASTRECEIVED:
				if (user1->info->stats.lastReceived > user2->info->stats.lastReceived)
					return -1;
				else
				if (user1->info->stats.lastReceived < user2->info->stats.lastReceived)
					return 1;				
				break;
				
			// sort the last sent event				
			case CS_LASTSENT:
				if (user1->info->stats.lastSent > user2->info->stats.lastSent)
					return -1;
				else
				if (user1->info->stats.lastSent < user2->info->stats.lastSent)
					return 1;				
				break;
				
			// sort alphabetical
			case CS_ALPHABETICAL:
				if (entry1->model && entry1->iter && entry2->model && entry2->iter)
				{
					gtk_tree_model_get(entry1->model, entry1->iter, CL_ALIAS, &n1, -1);
					gtk_tree_model_get(entry2->model, entry2->iter, CL_ALIAS, &n2, -1);
									
					nResult = g_ascii_strcasecmp(n1, n2);
					
					if (nResult)
						return nResult;
				}
				break;
				
			// sort idle time
			case CS_IDLE:
				if (user1->info->stats.idleSince < user2->info->stats.idleSince)
					return -1;
				else
				if (user1->info->stats.idleSince > user2->info->stats.idleSince)
					return 1;
				break;
		}
	}
	
	return 0;
}


void contactList::cb_checkboxToggled(GtkCellRenderer* t, gchar* pathString, contactList* self)
{
	GtkTreePath			*path;
	GtkTreeIter			iter;
	contactListEntry	*entry;
	
	path = gtk_tree_path_new_from_string(pathString);	
	gtk_tree_model_get_iter(self->model, &iter, path);
	gtk_tree_model_get(self->model, &iter,
					   CL_ENTRYPOINTER, &entry,
					   -1);
	gtk_tree_path_free(path);
	
	entry->setSelected(!entry->selected);	
}


void contactList::cb_settingsChangedCallback(gchar* sname, GList *values, contactList* self)
{
	gboolean 			updateAllColors,
						updateSortRules,
						updateAllUsers,
						modeChanged;
	settingsProperty	*prop;
	GdkColor			*col;
	GList				*allChildren,
						*search;
	contactListUser		*cUser;
	
	updateAllColors = FALSE;
	updateSortRules = FALSE;
	updateAllUsers = FALSE;
	modeChanged = FALSE;
	
	for( ; values; values = values->next)
	{
		prop = (settingsProperty*)values->data;
		
		if (!strcmp(prop->name, "showOfflineUsers") ||
			!strcmp(prop->name, "showGroups") ||
			!strcmp(prop->name, "showEmptyGroups"))
		{
			// Continue if we already changed the view mode this time
			if (modeChanged)
				continue;
			
			self->updateViewMode();			
			modeChanged = TRUE;
			
		}
		
		if (!strcmp(prop->name, "blinkEvents"))
		{
			c_blinkEvents = g_value_get_boolean(prop->val);
			allChildren = self->getAllChildrenOfType(CE_USER);
			for (search = allChildren; search; search = search->next)
			{
				cUser = (contactListUser*)search->data;
				cUser->blinkIcon(cUser->iconBlink);
			}
			g_list_free(allChildren);
			
			updateAllUsers = TRUE;
		} else
		if (!strcmp(prop->name, "showRealNames"))
		{
			c_useRealNames = g_value_get_boolean(prop->val);
			updateAllUsers = TRUE;
		} else
		if (!strcmp(prop->name, "showExtendedIcons"))
			self->setShowExtendedIcons(g_value_get_boolean(prop->val));
		else
		if (!strcmp(prop->name, "extendedIconsMask"))
		{
			c_extendedIconsMask = g_value_get_ulong(prop->val);
			self->updateExtendedIconsList();
			gtk_widget_queue_draw(self->frame);
		}
		else
		if (!strcmp(prop->name, "showColors"))
		{
			c_useColors = g_value_get_boolean(prop->val);
			updateAllColors = TRUE;
		}
		else
		if (!strcmp(prop->name, "contactVListColor"))
		{
			col = (GdkColor*)g_value_get_pointer(prop->val);
			c_vlistCol.red = col->red;
			c_vlistCol.green = col->green;
			c_vlistCol.blue = col->blue;
			updateAllColors = TRUE;
		} else
		if (!strcmp(prop->name, "contactNoMsgColor"))
		{
			col = (GdkColor*)g_value_get_pointer(prop->val);
			c_noMsgCol.red = col->red;
			c_noMsgCol.green = col->green;
			c_noMsgCol.blue = col->blue;
			updateAllColors = TRUE;			
		} else
		if (!strcmp(prop->name, "contactIDLEColor"))
		{
			col = (GdkColor*)g_value_get_pointer(prop->val);
			c_idleCol.red = col->red;
			c_idleCol.green = col->green;
			c_idleCol.blue = col->blue;
			updateAllColors = TRUE;		
		} else
		if (!updateSortRules && 
			(!strcmp(prop->name, "sortRulesOrder") || !strcmp(prop->name, "sortRulesEnabled")))
		{
			updateSortRules = TRUE;
			contactList_updateSortRules();
			self->resortContactlist();
		} else
		if (!strcmp(prop->name, "contactsFont"))
		{
			g_free(self->font);
			self->font = g_strdup(g_value_get_string(prop->val));
			
			
			
			if (self->useOwnFont)
				g_object_set(G_OBJECT(self->aliasRenderer),
							 "font", self->font, NULL);	
			else			
				g_object_set(G_OBJECT(self->aliasRenderer), "font", NULL, NULL);
			
			updateAllUsers = TRUE;

		} else
		if (!strcmp(prop->name, "contactsUseOwnFont"))
		{
			self->useOwnFont = g_value_get_boolean(prop->val);
			
			if (self->useOwnFont)
				g_object_set(G_OBJECT(self->aliasRenderer),
							 "font", self->font, NULL);	
			else			
				g_object_set(G_OBJECT(self->aliasRenderer), "font", NULL, NULL);
			
			updateAllUsers = TRUE;
		}
	}	

	if (updateAllColors || updateAllUsers)
	{
		allChildren = self->getAllChildrenOfType(CE_USER);
		for (search = allChildren; search; search = search->next)
		{
			if (updateAllColors)
				((contactListUser*)search->data)->updateColors();
			((contactListUser*)search->data)->updateEntry(FALSE);
		}
		g_list_free(allChildren);
	}		
}



void contactList::addChild(contactListEntry* child)
{
	GtkWidget	*menuItem;
	
	contactListEntry::addChild(child);
	
	if (child->type == CE_GROUP)
	{
		menuItem = ((contactListGroup*)child)->quickMenuEntry;
		g_signal_connect(G_OBJECT(menuItem), "activate", G_CALLBACK(cb_quickItemActivated), this);
		
		gtk_menu_shell_prepend(GTK_MENU_SHELL(filterMenu), menuItem);
		gtk_widget_show_all(filterMenu);
	}
}



// =============================================================================


void contactList_updateSortRules()
{
	gchar	*rulesOrderStr,
			*rulesEnabledStr;
	
	GList	*rulesOrder,
			*rulesEnabled,
			*searchOrder,
			*searchEnabled;
	
	settings_getSettings()->getProperties("contactlist",
										  "sortRulesOrder", &rulesOrderStr,
										  "sortRulesEnabled", &rulesEnabledStr,
										  NULL);
	
	rulesOrder = u_getUNumbersFromString(rulesOrderStr);
	rulesEnabled = u_getUNumbersFromString(rulesEnabledStr);
	
	searchOrder = rulesOrder;
	searchEnabled = rulesEnabled;
	
	if (c_sortRules)
	{
		g_list_free(c_sortRules);
		c_sortRules = NULL;
	}
	
	while(searchOrder)
	{
		if (searchEnabled && searchEnabled->data)
			c_sortRules = g_list_append(c_sortRules, searchOrder->data);
		
		searchOrder = searchOrder->next;
		searchEnabled = searchEnabled->next;
	}
	
	g_list_free(rulesOrder);
	g_list_free(rulesEnabled);
	g_free(rulesOrderStr);
	g_free(rulesEnabledStr);  
}


		
void contactList::cb_showOfflineButtonClicked(contactList* self)
{
	self->viewType = (contactViewType)(self->viewType ^ CVIEW_HIDEOFFLINE);	

	self->unrealizeEntry();
	self->setViewMode(self->viewType);
	self->realizeEntry(TRUE);
}



void contactList::cb_showGroupsButtonClicked(contactList* self)
{	
	self->viewType = (contactViewType)(self->viewType ^ CVIEW_GROUPS);	

	self->unrealizeEntry();
	self->setViewMode(self->viewType);
	self->realizeEntry(TRUE);
	
		
}

void contactList::cb_groupFilterButtonClicked(contactList* self)
{
	gtk_menu_popup(GTK_MENU(self->filterMenu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
}


void contactList::cb_quickItemActivated(GtkMenuItem* item, contactList* self)
{
	GList				*grList,
						*search;
	gpointer			toShow;
	contactListGroup	*cur;
	gchar				*newLabelText,
						*escapedGroupName;
	
	toShow = g_object_get_data(G_OBJECT(item), "icqnd-qm-owner");
	
	grList = self->getAllChildrenOfType(CE_GROUP);
	for (search = grList; search; search = search->next)
	{
		cur = (contactListGroup*)search->data;
		
		g_signal_handlers_block_by_func(cur->quickMenuEntry, (gpointer)self->cb_quickItemActivated, self);
		
		if (search->data == toShow)
		{
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(cur->quickMenuEntry), TRUE);
			cur->visible = TRUE;
		}
		else
		{
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(cur->quickMenuEntry), FALSE);
			cur->visible = FALSE;
		}
		
		g_signal_handlers_unblock_by_func(cur->quickMenuEntry, (gpointer)self->cb_quickItemActivated, self);
	}
	
	self->unrealizeEntry();
	self->realizeEntry(TRUE);
	
	escapedGroupName = g_markup_escape_text(((contactListGroup*)toShow)->name, -1);
	newLabelText = g_strdup_printf("<small>%s</small>", escapedGroupName);
	gtk_label_set_markup(GTK_LABEL(self->filterLabel), newLabelText);
	g_free(escapedGroupName);
	g_free(newLabelText);
	
	g_signal_handlers_block_by_func(self->filterAllItem, (gpointer)self->cb_groupAllFilterButtonClicked, self);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(self->filterAllItem), FALSE);
	g_signal_handlers_unblock_by_func(self->filterAllItem, (gpointer)self->cb_groupAllFilterButtonClicked, self);
	
	g_list_free(grList);
	
	gtk_tool_button_set_label(GTK_TOOL_BUTTON(self->groupFilter), ((contactListGroup*)toShow)->name);
}


void contactList::cb_groupAllFilterButtonClicked(contactList* self)
{
	GList	*grList,
			*search;
	gchar	*newLabelText;
	contactListGroup	*cur;
	
	
	grList = self->getAllChildrenOfType(CE_GROUP);
	
	for (search = grList; search; search = search->next)
	{
		cur = (contactListGroup*)search->data;
		
		g_signal_handlers_block_by_func(cur->quickMenuEntry, (gpointer)self->cb_quickItemActivated, self);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(cur->quickMenuEntry), FALSE);
		g_signal_handlers_unblock_by_func(cur->quickMenuEntry, (gpointer)self->cb_quickItemActivated, self);

		cur->visible = TRUE;
	}
	
	g_list_free(grList);
	
	self->unrealizeEntry();
	self->realizeEntry(TRUE);
	
	g_signal_handlers_block_by_func(self->filterAllItem, (gpointer)self->cb_groupAllFilterButtonClicked, self);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(self->filterAllItem), TRUE);
	g_signal_handlers_unblock_by_func(self->filterAllItem, (gpointer)self->cb_groupAllFilterButtonClicked, self);

	newLabelText = g_strdup_printf("<small>%s</small>", tr("All Groups"));
	gtk_label_set_markup(GTK_LABEL(self->filterLabel), newLabelText);	
	g_free(newLabelText);
	
	gtk_tool_button_set_label(GTK_TOOL_BUTTON(self->groupFilter), tr("All Groups"));
}
