/***************************************************************************
 *            convwindow.cpp
 *
 *  Fri Jan 21 23:21:09 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "convwindow.h"
#ifdef USE_GTKSPELL
extern "C" 
{
	#include <gtkspell/gtkspell.h>
}
#endif


conversationWindow::conversationWindow()
{
	wType = UE_MESSAGE;
	tabLabelImage = NULL;
	smileysMenu = NULL;
	keyPressTime = 0;
	isURLMode = FALSE;
	hoveredLink = NULL;
	multipleSelection = NULL;
	
	// things that appear in the tool bar only once
	sendAni = NULL;
		
	smallHistory = NULL;
	
	statusEntry = NULL;
	timeEntry = NULL;
	charsetMenu = NULL;
	serverCheckbox = NULL;
	urgentCheckbox = NULL;
	sendButton = NULL;
	multipleButton = NULL;
	toggleSmileysImg = NULL;
	toggleURLImg = NULL;

	timeTimeoutID = 0;
}



conversationWindow::~conversationWindow()
{

}



void conversationWindow::windowActivated()
{	
	if (!hasFocus)
	{		
		basicWindow::windowActivated();
		// window is visible now, so all sunken messages should be seen and can
		// be deleted
		manager->deleteEvent(EO_ALL | EO_NORMAL);
		
		// unset the color
		if (manager->info->typing == 2)
			setTabWidgetColor(&tabTypingColor);
		else
			setTabWidgetColor(NULL);
		
	}
	// focus the text entry	
	gtk_widget_grab_focus(localView);
}



void conversationWindow::windowDeactivated()
{
	basicWindow::windowDeactivated();
}



GtkWidget* conversationWindow::createWindowContent()
{
	GtkWidget	*localbox,
				*remotebox,
				*entrybuttonbox,
				*vbox;

	
	// create the tool bars
	settings_getSettings()->getProperties("conversations",
										  "showSmileys", &interpretSmileys,
										  "showURIs", &interpretURIs,
										  "dontApplyColors", &dontApplyColors,
										  "sendTyping", &sendTyping,
										  "sendByReturn", &sendByReturn,
										  NULL);
										  
	settings_getSettings()->getProperties("appearance",
										  "tabMessageColor", &tabMessageColor,
										  "tabTypingColor", &tabTypingColor,
										  NULL);			

	settings_getSettings()->getProperties("contactlist",
										  "showRealNames", &useRealName,
										  NULL);
							
	preferredEncoding = NULL;						
	settings_getSettings()->installHatch("conversations", "standardEncoding", &preferredEncoding);							
	
	toolbar1Container = gtk_hbox_new(FALSE, 0);
	toolbar2Container = gtk_hbox_new(FALSE, 0);
	toolbar3Container = gtk_hbox_new(FALSE, 0);
	
	createToolbars();
	
	
	// create all boxes
	if (!(remotebox = createRemoteEntryBox())) return NULL;
	if (!(localbox  = createLocalEntryBox()) ) return NULL;		

	
	// create a bar to hold the URL field
	URLbox = gtk_hbox_new(FALSE, 2);	
	
	
	// create the box for the menu + entry
	entrybuttonbox = gtk_vbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(entrybuttonbox), toolbar2Container, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(entrybuttonbox), localbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(entrybuttonbox), URLbox, FALSE, TRUE, 0);
	
		
	
	// create divider
	localRemoteDivider = gtk_vpaned_new();
	
	
	//if (lastPanedPosition)
	//	gtk_paned_set_position(GTK_PANED(textEntryDivider), lastPanedPosition - 1);
	
	gtk_paned_pack1(GTK_PANED(localRemoteDivider),remotebox	,TRUE,TRUE);
	gtk_paned_pack2(GTK_PANED(localRemoteDivider),entrybuttonbox,FALSE,TRUE);
		
	
	
	
	// create the main vertical box
	vbox = gtk_vbox_new(FALSE, 0);	

	gtk_box_pack_start(GTK_BOX(vbox), toolbar1Container   ,FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), localRemoteDivider,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox), toolbar3Container	,FALSE, FALSE, 0);


	// create a hbox around the vbox (so the multiple send window can be added)
	mainHBox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(mainHBox), vbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(mainHBox), 5);
	
	
	updateLocalColors();
	updateTitle();

	setWindowSize(550, 450);
	
	addSettingsListener("conversations");
	addSettingsListener("appearance");
	addSettingsListener("contactlist");
	
	return mainHBox;	
}


#define CW_CLEAR_CHILDREN(c)										\
	children = gtk_container_get_children(GTK_CONTAINER(c));		\
	if (children)													\
	{																\
		g_list_foreach(children, (GFunc)gtk_widget_destroy, NULL);	\
		g_list_free(children);										\
	}


void conversationWindow::createToolbars()
{
	GList		*children;
	gchar		*remoteToolbarRules,
				*localToolbarRules,
				*buttonbarRules;
	gboolean	showRemoteToolbar,
				showLocalToolbar,
				showButtonbar;
	gulong		remoteToolbarSpacing,
				localToolbarSpacing,
				buttonbarSpacing;
	GtkWidget	*toolbar1,
				*toolbar2,
				*toolbar3,
				*alignment1,
				*alignment3;
	
	
	settings_getSettings()->getProperties("conversations",
										  "remoteToolbarRules", &remoteToolbarRules,
										  "localToolbarRules", &localToolbarRules,
										  "buttonbarRules", &buttonbarRules,
										  "remoteToolbarSpacing", &remoteToolbarSpacing,
										  "localToolbarSpacing", &localToolbarSpacing,
										  "buttonbarSpacing", &buttonbarSpacing,
										  "showRemoteToolbar", &showRemoteToolbar,
										  "showLocalToolbar", &showLocalToolbar,
										  "showButtonbar", &showButtonbar,
										  "messageTag", &messageTag,
										  NULL);
										  
	u_stripStringByNeedle(messageTag, "|||");
	
	
	if (timeTimeoutID)
		g_source_remove(timeTimeoutID);
	
	if (sendAni)
	{
		delete sendAni;
		sendAni = NULL;
	}
	
	// NULL these things so they won't be recognized
	statusEntry = NULL;
	timeEntry = NULL;
	charsetMenu = NULL;
	serverCheckbox = NULL;
	urgentCheckbox = NULL;
	sendButton = NULL;
	multipleButton = NULL;
	toggleSmileysImg = NULL;
	toggleURLImg = NULL;
	alignment1 = NULL;
	alignment3 = NULL;

	// clear the children
	CW_CLEAR_CHILDREN(toolbar1Container);
	CW_CLEAR_CHILDREN(toolbar2Container);
	CW_CLEAR_CHILDREN(toolbar3Container);

	
	// remote toolbar
	if (showRemoteToolbar)
	{
		toolbar1 = createToolbar(remoteToolbarRules, remoteToolbarSpacing);
		alignment1 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
		gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 5, 0, 0);
		gtk_container_add(GTK_CONTAINER(alignment1), toolbar1);	
	} else
		toolbar1 = NULL;
	
	// local toolbar
	if (showLocalToolbar)
	{
		toolbar2 = createToolbar(localToolbarRules, localToolbarSpacing);
		gtk_container_set_border_width(GTK_CONTAINER(toolbar2), 2);
	} else
		toolbar2 = NULL;
	
	// buttonbar
	if (showButtonbar)
	{
		toolbar3 = createToolbar(buttonbarRules, buttonbarSpacing);
		alignment3 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
		gtk_alignment_set_padding(GTK_ALIGNMENT(alignment3), 3,0,0,0);
		gtk_container_add(GTK_CONTAINER(alignment3), toolbar3);
	}
	else
		toolbar3 = NULL;
	
	if (toolbar1)
		gtk_container_add(GTK_CONTAINER(toolbar1Container), alignment1);
	if (toolbar2)
		gtk_container_add(GTK_CONTAINER(toolbar2Container), toolbar2);
	if (toolbar3)
		gtk_container_add(GTK_CONTAINER(toolbar3Container), alignment3);

	gtk_widget_show_all(toolbar1Container);
	gtk_widget_show_all(toolbar2Container);
	gtk_widget_show_all(toolbar3Container);
}



void conversationWindow::enableURLMode()
{
	GtkTooltips	*tips;
	
	
	if (!isURLMode)
	{
		goToButton = u_createStockImageButton(GTK_STOCK_JUMP_TO);
		g_signal_connect_swapped(goToButton, "clicked", G_CALLBACK(this->cb_openURLButtonClicked), this);
		//gtk_widget_set_sensitive(goToButton, FALSE);
		URLentry = gtk_entry_new();
		//g_signal_connect_swapped(URLentry, "key-release-event", G_CALLBACK(this->cb_urlEntryTyped), this);
		//g_signal_connect_swapped(URLentry, "paste-clipboard", G_CALLBACK(this->cb_urlEntryTyped), this);
		
		gtk_box_pack_start(GTK_BOX(URLbox), goToButton, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(URLbox), URLentry, TRUE, TRUE, 0);	
	
		tips = gtk_tooltips_new();
		gtk_tooltips_set_tip(tips, goToButton, 
							 tr("Open the entered URL"), NULL);
	
		gtk_tooltips_enable(tips);	
		
		gtk_widget_show_all(URLbox);
		
		gtk_widget_grab_focus(URLentry);
		
		isURLMode = TRUE;
	};

}


void conversationWindow::disableURLMode()
{
	GList	*children,
			*search;
	
	if (isURLMode)
	{
		children = gtk_container_get_children(GTK_CONTAINER(URLbox));
		
		search = children;
		while(search)
		{
			gtk_container_remove(GTK_CONTAINER(URLbox), GTK_WIDGET(search->data));
			search = search->next;
		}
		
		g_list_free(children);

		isURLMode = FALSE;		
	}
}



GtkWidget* conversationWindow::createContainerTabWidget()
{
	GtkWidget  		*hbox,
					*eventBox;
	
	gchar	   		*tmp;
	PangoAttrList	*labelAttr;
	
	tabLabelImage = gtk_image_new_from_pixbuf(i_getIcons()->getUserStatusPixbuf(manager->parentUser));
	
	if (useRealName)
		tmp = uu_getRealNameOrAlias(manager->info, FALSE);
	else
		tmp = g_strdup(manager->info->alias);
	tabLabelText = gtk_label_new(tmp);
	g_free(tmp);
	
	labelAttr = pango_attr_list_new();
	tabLabelColor = pango_attr_foreground_new(0, 0, 0);
	tabLabelColor->start_index = 0;
	tabLabelColor->end_index = 0;
	pango_attr_list_insert(labelAttr, tabLabelColor);
	gtk_label_set_attributes(GTK_LABEL(tabLabelText), labelAttr);
	
	gtk_widget_ensure_style(tabLabelText);
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), tabLabelImage, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), tabLabelText, TRUE, TRUE, 0);
	
	eventBox = gtk_event_box_new();
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(eventBox), FALSE);
	gtk_container_add(GTK_CONTAINER(eventBox), hbox);
	
	tooltips = new userTooltips();
	tooltips->setPopupDelay(500);
	tooltips->attachWidget(eventBox, (UTWidgetFunc)this->cb_tooltipsFunction, this);
	
	return eventBox;
}



void conversationWindow::destroyWindowContent()
{
	if (sendAni)
	{
		sendAni->stop();
		delete sendAni;
	}
	
	if (timeTimeoutID)
		g_source_remove(timeTimeoutID);
	((IMMessageManager*)manager)->sendTypingNotification(0);
	
	settings_getSettings()->removeHatch("conversations", "standardEncoding", &preferredEncoding);							
	
	
	if (manager)
		delete manager;
	
	delete tooltips;
}


void conversationWindow::updateTitle()
{
	gchar	*newTitle;
	
	newTitle = g_strdup_printf(tr("Conversation - %s"), manager->info->alias);
	setWindowTitle(newTitle);
	g_free(newTitle);
}


void conversationWindow::setTypedText(gchar* text)
{
	GtkTextBuffer		*buf;

	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));
	
	gtk_text_buffer_insert_at_cursor(buf, text, -1);
	cb_keyPressed(this, NULL);
}




void conversationWindow::sendMessage()
{	
	// user clicked when not sending	
	if (GTK_WIDGET_SENSITIVE(localView))
	{
		// get all neccessary information
		toSendText = getTextWithoutSmileys();
		if (isURLMode)
			toSendLink = g_strdup((gchar*)gtk_entry_get_text(GTK_ENTRY(URLentry)));
		else
			toSendLink = NULL;
		
		// don't send the message if it's empty
		if ((!strlen(toSendText) && !isURLMode) || (isURLMode && !strlen(toSendLink)))
		{
			g_free(toSendText);
			if (toSendLink)
				g_free(toSendLink);
			
			return;
		}

		if (sendAni)		
			sendAni->play();

		if (sendButton)
			gtk_button_set_label(GTK_BUTTON(sendButton), tr("_Cancel"));
		
		gtk_widget_set_sensitive(localView,FALSE);
		if (serverCheckbox)
			gtk_widget_set_sensitive(serverCheckbox, FALSE);
		if (multipleButton)
			gtk_widget_set_sensitive(multipleButton, FALSE);

		if (isURLMode)
				gtk_widget_set_sensitive(URLentry, FALSE);		
		
		
		
		// is there a multiple selection? -> start multiple sending of 5 entries
		if (multipleSelection)
		{
			multipleSelection->getSelectedEntries();
			sendMessageMultiple(5);
		} else
		// no multiple selection -> send the appropriate event
		{
			if (toSendLink)
			{
				((IMMessageManager*)manager)->sendURL(toSendLink, toSendText);
				g_free(toSendLink);
			} else
				((IMMessageManager*)manager)->sendMessage(toSendText);
			
			g_free(toSendText);
		}
		
		// FIXME: auto send messages urgent ?
		// FIXME: check serverCheckbox state		
	} else
	{
		stopSendMessage();
		// user clicked while sending -> cancel sending	
		manager->cancelInitatedEvent();	
	}
}



void conversationWindow::sendMessageMultiple(gint count)
{	
	GList	*search;
	contactListUser* entry;
	IMMessageManager* man;
	IMUserDaemon* user;
	
	if (!multipleSelection)
		return;
	
	search = multipleSelection->selectedEntries;
	while(search && count)
	{
		entry = (contactListUser*)search->data;

		if (entry->getSelected() && !entry->getBackgroundColorEnabled())
		{
			entry->setBackgroundColorEnabled(TRUE);
			
			user = (IMUserDaemon*)entry->data;
		
			if (user != manager->parentUser)
			{
				man = new IMMessageManager(user->info);
				man->info->sendViaServer = TRUE;
				man->addCallback((IMEventCallback)this->cb_eventCallback, this);
				user->addManager(man);
			} else
				man = (IMMessageManager*)manager;
		
			if (isURLMode)
				man->sendURL(toSendLink, toSendText);
			else
				man->sendMessage(toSendText);
			
			count--;
		} 
		
		search = search->next;
	}
}




void conversationWindow::stopSendMessage()
{
	GList* search;
	contactListEntry *entry;
	
	if (multipleSelection)
	{
		search = multipleSelection->selectedEntries;
		while(search)
		{
			entry = (contactListEntry*)search->data;
			if ((IMUserDaemon*)entry->data != manager->parentUser)
				((IMUserDaemon*)entry->data)->removeManagerByCallback((gpointer)this->cb_eventCallback);
			search = search->next;
		}
		
		if (toSendLink)
			g_free(toSendLink);
		g_free(toSendText);
	}
	
	if (sendAni)
		sendAni->stop();
	if (sendButton)
		gtk_button_set_label(GTK_BUTTON(sendButton), tr("_Send"));
	// gtk_label_set_markup_with_mnemonic(GTK_LABEL(sendLabel), "_Senden");
	gtk_widget_set_sensitive(localView,TRUE);
	if (serverCheckbox)
		gtk_widget_set_sensitive(serverCheckbox, TRUE);
	if (multipleButton)
		gtk_widget_set_sensitive(multipleButton, TRUE);
	
	gtk_widget_grab_focus(localView);
	
	if (isURLMode)
		gtk_widget_set_sensitive(URLentry, TRUE);
}




void conversationWindow::finishSendMessage(messageEventInfo* messageEvent, gint result)
{
	GtkTextIter 	start, 
					end;
	GtkTextBuffer	*tb;
	GList			*search;
	contactListUser	*entry;
	IMUserDaemon	*user;
	GString			*errorMsg;
	
	if (result != EVENT_RESULT_SUCCESS && result != EVENT_RESULT_ACKED)
	{
		errorMsg = g_string_new(tr("The message could not be sent"));
		errorMsg = g_string_append(errorMsg, uu_getSendErrorMessage(result, manager->parentUser->owner->info->status));
		insertMessage(time(NULL), errorMsg->str, NULL, NULL, MT_ERROR, TRUE);
		g_string_free(errorMsg, TRUE);
		stopSendMessage();
		
		// if there is a multisending running stop it and destroy all managers
		if (multipleSelection)
		{
			search = multipleSelection->selectedEntries;
			while(search)
			{
				entry = (contactListUser*)search->data;
				entry->setBackgroundColorEnabled(FALSE);
				user = (IMUserDaemon*)entry->data;
				if (user != manager->parentUser)
					user->removeManagerByCallback((gpointer)this->cb_eventCallback);
				search = search->next;
			}
		}
		
		return;
	}
	
	// is there a multi send running? -> send the next 5 events
	if (multipleSelection && !messageEvent->isSplit)
	{
		// delete the entry from the list of selected entries		
		search = multipleSelection->selectedEntries;
		while(search)
		{
			entry = (contactListUser*)search->data;
			if (!strcmp(entry->id, messageEvent->id))
			{
				entry->setSelected(FALSE);
				entry->setBackgroundColorEnabled(FALSE);
				if ((IMUserDaemon*)entry->data != manager->parentUser)
					((IMUserDaemon*)entry->data)->removeManagerByCallback((gpointer)this->cb_eventCallback);
				multipleSelection->selectedEntries = g_list_remove(multipleSelection->selectedEntries, entry);
				break;
			}
			
			search = search->next;
		}
		
		sendMessageMultiple(1);
	}
	
	// there is no multiple selection or there are no more entries in it
	if (!multipleSelection || (multipleSelection && !multipleSelection->selectedEntries))
	{
		if (result == EVENT_RESULT_SUCCESS || result == EVENT_RESULT_ACKED)
		{
			user = ((IMOwnerDaemon*)manager->parentUser->owner)->findUserByID(messageEvent->id);
			insertMessage(time(NULL), messageEvent->text, messageEvent->url, user->owner, MT_OUTGOING, TRUE);
		}
		// clear text buffer
		tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));	
		gtk_text_buffer_get_bounds(tb, &start, &end);
		gtk_text_buffer_delete(tb, &start, &end);
		gtk_widget_set_sensitive(sendButton, FALSE);
		
		if (hasFocus)
			gtk_widget_grab_focus(localView);	

		keyPressTime = 0;
		((IMMessageManager*)manager)->sendTypingNotification(0);
		
		if (!messageEvent->isSplit)
			stopSendMessage();
	}
}




void conversationWindow::insertMessage(time_t msgTime, gchar* msg, gchar* url, IMUserDaemon* user, messageType msgType, gboolean add2History)
{
	GtkTextIter 		iter;
	GtkTextBuffer* 		tb;
	gchar				ftime[100] = {0, };
	gchar				*toInsert;
	gchar				*typeStr;
	GtkTextMark			*mark;
	smallHistoryEntry	*hEntry;
	gchar				*lastPos,
						*currentPos,
						*currentTag;
	tm					localFormattedTime,
						*msgFormattedTime;
	time_t				localTime;
	

	/* add a new entry for the small history (used for message recreation)
	   -------------------------------------------------------------------
	*/
	if (add2History)
	{
		hEntry = (smallHistoryEntry*)g_malloc0(sizeof(smallHistoryEntry));
		hEntry->type = msgType;
		hEntry->time = msgTime;
		hEntry->msg = g_strdup(msg);
		hEntry->user = user;
		if (url)
			hEntry->url = g_strdup(url);
		else
			hEntry->url = NULL;
		smallHistory = g_list_append(smallHistory, hEntry);
	}
	
	// find the right text tag and beginning "script" for the text
	currentPos = messageTag;
	
	switch(msgType)
	{
		case MT_INCOMING: typeStr = "incoming"; break;
		case MT_OUTGOING: typeStr = "outgoing"; break;
		case MT_HINCOMING: typeStr = "history_incoming"; break;
		case MT_HOUTGOING: typeStr = "history_outgoing"; break;
		case MT_ERROR: typeStr = "error"; currentPos = "%t "; break;
		default: typeStr = "notification"; currentPos = "%t %a "; break;
	}
	
	
	tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView));
	gtk_text_buffer_get_end_iter(tb, &iter);
	
	// create a gap
	gtk_text_buffer_insert_with_tags_by_name(tb, &iter, "\n", -1, "newline", NULL);		

	
	// insert the formatted message beginning	
	lastPos = currentPos;
	
	while((currentPos = strchr(lastPos, '%')))
	{
		// insert non-token stuff
		if (lastPos != currentPos)
			gtk_text_buffer_insert_with_tags_by_name(tb, &iter, lastPos, currentPos - lastPos, "HL",  typeStr, NULL);

		toInsert = NULL;
		currentTag = "HL";
		
		// insert the appropriate tag
		switch(currentPos[1])
		{
			case 't': 
				// get the current time
				localTime = time(NULL);
				memcpy(&localFormattedTime, localtime(&localTime), sizeof(tm));
				msgFormattedTime = localtime(&msgTime);
			
				// do we still have the same day? If not print the date also
				if (localFormattedTime.tm_yday == msgFormattedTime->tm_yday &&
					localFormattedTime.tm_year == msgFormattedTime->tm_year)			
					strftime(ftime, 100, tr("%H:%M:%S"), msgFormattedTime);
				else
					strftime(ftime, 100, tr("%m/%d/%y %H:%M:%S"), msgFormattedTime);
				toInsert = g_strdup_printf("(%s)", ftime);
				currentTag = "timestamp";
				break;

			case 'a': 
				toInsert = g_strdup(user->info->alias);
				break;

			case 'f':
				toInsert = g_strdup(user->info->firstname);
				break;

			case 'l':
				toInsert = g_strdup(user->info->lastname);
				break;

			case 'n':
				toInsert  = g_strdup_printf("%s %s", user->info->firstname, user->info->lastname);			
				break;

			case 'b':
				toInsert = g_strdup("\n");
				break;
			
			default:
				fprintf(stderr, "conversationWindow::insertMessage(): Unknown token '%c'\n", currentPos[1]);
				break;
		}
				
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, toInsert, -1, currentTag, typeStr, NULL);
		g_free(toInsert);		
		
		lastPos = currentPos+2;
	}
	
	// insert the rest
	gtk_text_buffer_insert_with_tags_by_name(tb, &iter, lastPos, -1, "HL", typeStr, NULL);
	
	// eventually show an url
	if (url)
	{
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, url, -1, "url", dontApplyColors? NULL: typeStr, "margin", NULL);
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, "\n", -1, dontApplyColors? NULL: typeStr, NULL);
	}
	
	
	// show text
	toInsert = g_strdup_printf("%s\n", msg);
	if (msgType != MT_NOTIFICATION && msgType != MT_ERROR && !(!interpretSmileys && !interpretURIs))
		u_insertTextInterpretedIntoView(toInsert, dontApplyColors? NULL: typeStr, url != NULL, interpretURIs, interpretSmileys, &iter, GTK_TEXT_VIEW(remoteView));
//	insertTextInterpreted(toInsert, dontApplyColors? NULL: typeStr, url != NULL, &iter);
	else
		gtk_text_buffer_insert_with_tags_by_name(tb, &iter, toInsert, -1, dontApplyColors? NULL: typeStr, NULL);	
	g_free(toInsert);
		
	// scroll down
	gtk_text_buffer_get_end_iter(tb, &iter);
	mark = gtk_text_buffer_get_mark(tb, "last_pos");
	gtk_text_buffer_move_mark(tb, mark, &iter);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(remoteView), mark, 0, FALSE, 0, 0);
	
}




// ----------------------- helper functions ------------------------------------

#define CW_CREATESTDTOOL(w, id, stock)												\
	w = u_createStockImageButton(stock);											\
	g_object_set_data(G_OBJECT(w), "icqnd-menuid", GUINT_TO_POINTER(id));			\
	if (!preview)																	\
		g_signal_connect(w, "clicked", G_CALLBACK(this->cb_toolbarClicked), this);


GtkWidget* conversationWindow::createToolbar(gchar *description, guint spacing, gboolean preview, GList **entries)
{
	guint					 element;
	GtkWidget				 *toAdd,
							 *toolbar,
							 *label;
	gboolean				 packRight,
							 fillSpace;
	guint					 padding;
	GtkTooltips				 *toolTips;
	gchar					 *tip;
	GList					 *nums,
							 *search;
	
	
	toolbar = gtk_hbox_new(FALSE, spacing);
	toolTips = gtk_tooltips_new();
	
	// extract the numbers out of the string and create the elements
	packRight = FALSE;
	
	nums = u_getUNumbersFromString(description);
	
	if (entries)
		*entries = g_list_copy(nums);
	

	for (search = nums; search; search = search->next)
	{
		element 	= GPOINTER_TO_UINT(search->data);
		fillSpace 	= FALSE;
		padding 	= 0;
		toAdd 		= NULL;
		tip			= NULL;
				
		switch(element)
		{
			case TOOL_SEPARATOR:
				toAdd = gtk_vseparator_new();
				padding = 2;
				break;
				
			case TOOL_ALIGNLEFT: 
				packRight = FALSE; 
				break;
				
			case TOOL_ALIGNRIGHT:
				packRight = TRUE;
				break;
				
			case TOOL_SENDANI:
				if (sendAni) 
					break;
				sendAni = new processingAni();
				sendAni->setupAnimation(i_getIcons()->images.searchingAni, 22, 22);
				toAdd = sendAni->widget;
				break;
					
			case TOOL_ENCODING:
				CW_CREATESTDTOOL(toAdd, TOOL_ENCODING, GTK_STOCK_UNINDENT);
				tip = tr("Set the encoding for receiving / sending messages");
				if (!preview)
					createCharsetList();
				break;
				
			case TOOL_HISTORY:
				CW_CREATESTDTOOL(toAdd, TOOL_HISTORY, ICON_HISTORY);
				tip = tr("Show users history");
				break;
				
			case TOOL_INFO:
				CW_CREATESTDTOOL(toAdd, TOOL_INFO, ICON_INFO);
				tip = tr("Show users personal information");
				break;
				
			case TOOL_ENCRYPT:
				CW_CREATESTDTOOL(toAdd, TOOL_ENCRYPT, GTK_STOCK_DIALOG_AUTHENTICATION);
				tip = tr("Encrypt the connection to the user");
				break;
					
			case TOOL_STATUS:
				if (statusEntry)
					break;
				label = gtk_label_new(tr("Status:"));
					
				statusEntry = gtk_entry_new();
				gtk_widget_set_size_request(statusEntry, 100, -1);
				gtk_editable_set_editable(GTK_EDITABLE(statusEntry), FALSE);
				if (preview)
					gtk_entry_set_text(GTK_ENTRY(statusEntry), tr("Online"));
				else
					gtk_entry_set_text(GTK_ENTRY(statusEntry), uu_getStatusDescription(manager->info->status));

				toAdd = gtk_hbox_new(FALSE, 2);
				gtk_box_pack_start(GTK_BOX(toAdd), label, FALSE, FALSE, 0);
				gtk_box_pack_start(GTK_BOX(toAdd), statusEntry, TRUE, TRUE, 0);
				fillSpace = TRUE;
				break;
				
			case TOOL_TIME:
				if (timeEntry)
					break;
				label = gtk_label_new(tr("Time:"));
				timeEntry = gtk_entry_new();
				gtk_widget_set_size_request(timeEntry, 100, -1);
				gtk_editable_set_editable(GTK_EDITABLE(timeEntry), FALSE);
				if (preview)
					gtk_entry_set_text(GTK_ENTRY(timeEntry), "12:00:00");
				else
				{
					cb_setUserTime(this);
					timeTimeoutID = g_timeout_add(1000, (GSourceFunc)this->cb_setUserTime, this);					
				}
						
				toAdd = gtk_hbox_new(FALSE, 2);
				gtk_box_pack_start(GTK_BOX(toAdd), label, FALSE, FALSE, 0);
				gtk_box_pack_start(GTK_BOX(toAdd), timeEntry, TRUE, TRUE, 0);
				fillSpace = TRUE;	
				break;

			case TOOL_SMILEYS:
				CW_CREATESTDTOOL(toAdd, TOOL_SMILEYS, ICON_SMILEYS);
				tip = tr("Insert a smiley");
				break;
					
			case TOOL_SENDFILE:
				CW_CREATESTDTOOL(toAdd, TOOL_SENDFILE, ICON_FILE);
				tip = tr("Send files to the user");
				break;
					
			case TOOL_SENDLINK:
				CW_CREATESTDTOOL(toAdd, TOOL_SENDLINK, ICON_URL);
				tip = tr("Send a URL to the user");
				break;
					
			case TOOL_STARTCHAT:					
				CW_CREATESTDTOOL(toAdd, TOOL_STARTCHAT, ICON_CHAT);
				tip = tr("Start a chat session with the user");
				break;
				
			case TOOL_FGCOLOR:
				CW_CREATESTDTOOL(toAdd, TOOL_FGCOLOR, GTK_STOCK_SELECT_FONT);
				tip = tr("Set the font color");
				break;
				
			case TOOL_BGCOLOR:
				CW_CREATESTDTOOL(toAdd, TOOL_BGCOLOR, GTK_STOCK_SELECT_COLOR);
				tip = tr("Set the background color");
				break;
				
			case TOOL_SENDSERVER:
				if (serverCheckbox)
					break;
				serverCheckbox = gtk_check_button_new_with_label(tr("Send via server"));	
				if (!preview)
					g_signal_connect_swapped(serverCheckbox, "clicked", G_CALLBACK(this->cb_sendServerButtonClicked), this);
				toAdd = serverCheckbox;
				tip = tr("Send messages via the server and not directly to the user");
				break;
					
			case TOOL_SENDURGENT:
				if (urgentCheckbox)
					break;
				urgentCheckbox = gtk_check_button_new_with_label(tr("Send urgent"));
				gtk_widget_set_sensitive(urgentCheckbox, FALSE);
				toAdd = urgentCheckbox;
				tip = tr("Mark a message as urgent");
				break;
					
			case TOOL_SEND:
				if (sendButton)
					break;
				sendButton = gtk_button_new_with_mnemonic(tr("_Send"));
				gtk_widget_set_size_request(sendButton, 80, -1);
				gtk_widget_set_sensitive(sendButton, FALSE);
				if (!preview)
					g_signal_connect_swapped(sendButton, "clicked", G_CALLBACK(this->cb_sendButtonClicked), this);
				toAdd = sendButton;
				tip = tr("Send this message");
				break;
					
			case TOOL_MULTIPLE:
				if (multipleButton)
					break;
				CW_CREATESTDTOOL(multipleButton, TOOL_MULTIPLE, GTK_STOCK_GO_FORWARD);
				toAdd = multipleButton;
				tip = tr("Send messages to multiple users at once");
				break;
				
			case TOOL_TOGGLESMILEYS:
				if (toggleSmileysImg)
					break;
				
				toggleSmileysImg = gtk_image_new_from_stock(ICON_SMILEYS, GTK_ICON_SIZE_SMALL_TOOLBAR);
				gtk_widget_set_sensitive(toggleSmileysImg, interpretSmileys);
				
				toAdd = gtk_button_new();
				gtk_container_add(GTK_CONTAINER(toAdd), toggleSmileysImg);
				
				g_object_set_data(G_OBJECT(toAdd), "icqnd-menuid", GUINT_TO_POINTER(TOOL_TOGGLESMILEYS));			
				if (!preview)																	
					g_signal_connect(toAdd, "clicked", G_CALLBACK(this->cb_toolbarClicked), this);

				
				tip = tr("Show smiley images on/off");
				break;
			
			case TOOL_TOGGLEURL:
				if (toggleURLImg)
					break;
				
				toggleURLImg = gtk_image_new_from_stock(ICON_URL, GTK_ICON_SIZE_SMALL_TOOLBAR);
				gtk_widget_set_sensitive(toggleURLImg, interpretURIs);
				
				toAdd = gtk_button_new();
				gtk_container_add(GTK_CONTAINER(toAdd), toggleURLImg);
				
				g_object_set_data(G_OBJECT(toAdd), "icqnd-menuid", GUINT_TO_POINTER(TOOL_TOGGLEURL));			
				if (!preview)																	
					g_signal_connect(toAdd, "clicked", G_CALLBACK(this->cb_toolbarClicked), this);
				
				tip = tr("Show URLs on/off");
				break;
		}
			
		if (toAdd)
		{
			if (preview)
				gtk_widget_set_sensitive(toAdd, FALSE);
			
			if (packRight)
				gtk_box_pack_end(GTK_BOX(toolbar), toAdd, fillSpace, fillSpace, padding);
			else
				gtk_box_pack_start(GTK_BOX(toolbar), toAdd, fillSpace, fillSpace, padding);
			
			if (tip)
				gtk_tooltips_set_tip(toolTips, toAdd, tip, NULL);
		}	
	}
	
	g_list_free(nums);	
	
	gtk_tooltips_enable(toolTips);
	
	return toolbar;
}
	
	
	
	
void conversationWindow::createCharsetList()
{
	charsetType	*charsets;
	GSList		*charsetGroup;
	GtkWidget	*charsetMenuItem;
	guint		i;
	
	gchar		*tmp;
	
	if (charsetMenu)
		return;
	
	// create the charset list
	charsetMenu = gtk_menu_new();
	
	charsets = charset_getCharsetList();
	charsetGroup = NULL;
	i=0;

	
	while(charsets[i].name)
	{
		tmp = g_strdup_printf("%s (%s)", tr(charsets[i].name), charsets[i].id);
		
		charsetMenuItem = gtk_radio_menu_item_new_with_label(charsetGroup, tmp);
		
		if ((manager->info->charset && 
			 *manager->info->charset && 
			 !strcmp(charsets[i].id, manager->info->charset)) ||
			!strcmp(preferredEncoding, charsets[i].id))
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(charsetMenuItem), TRUE);

		g_signal_connect(charsetMenuItem, "activate", G_CALLBACK(this->cb_charsetListItemClicked), this);
		g_object_set_data(G_OBJECT(charsetMenuItem), "icqnd-charset-set", charsets[i].id);
		charsetGroup = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(charsetMenuItem));

		gtk_menu_shell_append(GTK_MENU_SHELL(charsetMenu), charsetMenuItem);
		
		g_free(tmp);
		i++;
	}
		
}
	
	


GtkWidget* conversationWindow::createRemoteEntryBox()
{
	GtkWidget	*scroll;		
	
	GtkTextBuffer* tb;		
	GtkTextIter    iter;	
	
	// set up cText	
	// (cText from global definition)
	remoteView = gtk_text_view_new();
	gtk_text_view_set_left_margin(GTK_TEXT_VIEW(remoteView), 5);
	gtk_text_view_set_right_margin(GTK_TEXT_VIEW(remoteView), 5);
	
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW(remoteView), FALSE);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(remoteView), GTK_WRAP_WORD);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(remoteView), FALSE);

	tb = gtk_text_buffer_new(settings_getSettings()->textTags);
	
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(remoteView), tb);
	
	gtk_text_buffer_get_end_iter(tb, &iter);
	gtk_text_buffer_create_mark(tb, "last_pos", &iter, TRUE);


	g_signal_connect(remoteView, "motion-notify-event", G_CALLBACK(this->cb_remoteViewMouseMoved), this);
	g_signal_connect(remoteView, "visibility-notify-event", G_CALLBACK(this->cb_remoteViewVisibilityChanged), this);
	g_signal_connect_swapped(remoteView, "button-press-event", G_CALLBACK(this->cb_remoteViewURLPressed), this);
	g_signal_connect_swapped(remoteView, "populate-popup", G_CALLBACK(this->cb_popupPopulated), this);
	
	// create the scrolling window 
	scroll = gtk_scrolled_window_new(0, 0);
	gtk_scrolled_window_set_policy(	GTK_SCROLLED_WINDOW(scroll),
				       				GTK_POLICY_AUTOMATIC,
				       				GTK_POLICY_ALWAYS);
	gtk_container_add(GTK_CONTAINER(scroll), remoteView);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);

	return scroll;	
}



GtkWidget* conversationWindow::createLocalEntryBox()
{
	GtkWidget	*scroll;
	GtkTextBuffer* buf;			
	
	// set up cEntry
	// (cEntry from global definition)
	localView = gtk_text_view_new();
#ifdef USE_GTKSPELL
	gtkspell_new_attach( GTK_TEXT_VIEW( localView ), NULL, NULL );
#endif

	gtk_text_view_set_editable(GTK_TEXT_VIEW(localView), TRUE);
  	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(localView), GTK_WRAP_WORD);
	g_signal_connect_swapped(G_OBJECT(localView), "key-press-event", G_CALLBACK(this->cb_keyPressed), this);
	g_signal_connect_swapped(G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView))), "changed", G_CALLBACK(this->cb_localEntryChanged), this);
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));

	gtk_widget_set_size_request(localView, -1, 70);

		
	scroll = gtk_scrolled_window_new(0, 0);
	gtk_scrolled_window_set_policy(	GTK_SCROLLED_WINDOW(scroll),
				       				GTK_POLICY_AUTOMATIC,
				       				GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scroll), localView);

	
	
	return scroll;
	
}





void conversationWindow::selectColor(gboolean foreground)
{
	GtkWidget	*dialog,
				*useCheckbox;
	GtkColorSelection *colorSel;
	gint		res;
	GdkColor	col;
	gchar		*txt,
				*colToGet;
	bool		isSet;
	settings 	*settings;
	

	settings = settings_getSettings();

	if (foreground)
	{
		txt = tr_NOOP("Select a font color");
		colToGet = "foregroundColor";
	} else
	{
		txt = tr_NOOP("Select a background color");
		colToGet = "backgroundColor";
	}
	
	settings->getProperties("conversations",
							colToGet, &col,
							"colorsSet", &isSet,
							NULL);
		
	dialog = gtk_color_selection_dialog_new(tr(txt));
	gtk_widget_set_no_show_all(GTK_COLOR_SELECTION_DIALOG(dialog)->help_button, TRUE);		

	useCheckbox = gtk_check_button_new_with_label(tr("Use own colors"));
	g_signal_connect(useCheckbox, "toggled", G_CALLBACK(this->cb_colorSelectionCheckboxClicked), dialog);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), useCheckbox, FALSE, TRUE, 0);
	gtk_widget_show(useCheckbox);	
	
	colorSel = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel);
	
	gtk_color_selection_set_current_color(colorSel, &col);
	if (isSet)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(useCheckbox), TRUE);
	else
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(useCheckbox), FALSE);
		gtk_widget_set_sensitive(GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel, FALSE);
	}
		
	res = gtk_dialog_run(GTK_DIALOG(dialog));

	if (res == GTK_RESPONSE_OK)
	{
		isSet = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useCheckbox));
		if (isSet)
			gtk_color_selection_get_current_color(colorSel, &col);
		settings->setProperties(TRUE, "conversations",
								colToGet, &col,
								"colorsSet", isSet,
								NULL); 
	}
	
	gtk_widget_destroy(dialog);
}



void conversationWindow::updateLocalColors()
{
	GdkColor	bgColor,
				fgColor;
	gboolean	colorsSet;
	
	settings_getSettings()->getProperties("conversations",
										  "colorsSet", &colorsSet,	
										  "foregroundColor", &fgColor,
										  "backgroundColor", &bgColor,
										  NULL);
	
		
	if (colorsSet)
	{
		gtk_widget_modify_text(localView, GTK_STATE_NORMAL, &fgColor);
		gtk_widget_modify_base(localView, GTK_STATE_NORMAL, &bgColor);
		((IMMessageManager*)manager)->setColorsEnabled(TRUE);
		((IMMessageManager*)manager)->setColors(&fgColor, &bgColor);
	}
	else
	{
		gtk_widget_modify_text(localView, GTK_STATE_NORMAL, NULL); 
		gtk_widget_modify_base(localView, GTK_STATE_NORMAL, NULL);
		((IMMessageManager*)manager)->setColorsEnabled(FALSE);		
	}
}


void conversationWindow::setTabWidgetColor(GdkColor* col)
{
	if (!col)
		tabLabelColor->end_index = 0;
	else
	{
		((PangoAttrColor*)tabLabelColor)->color.red = col->red;
		((PangoAttrColor*)tabLabelColor)->color.green = col->green;
		((PangoAttrColor*)tabLabelColor)->color.blue = col->blue;
	
		tabLabelColor->end_index = 0xFFFF;
	}
		
	// update the label (... *sigh*)
	gtk_label_set_attributes(GTK_LABEL(tabLabelText), 
							 gtk_label_get_attributes(GTK_LABEL(tabLabelText)));
}


void conversationWindow::createSmileysMenu()
{
	GtkWidget	*smileyBackground,
				*smileybox,
				*hrow,
				*asmiley,
				*smileyimg,
				*frame;
	guint i, imagePos;
	settings* config;
	gint 		width, 
				height, 
				posX, 
				posY, 
				maxX, 
				maxY;
	GdkScreen* scr;
	iconManager	*icons;
	
	
	config = settings_getSettings();
	icons = i_getIcons();
	
	// *** create the window ***
	smileysMenu = gtk_window_new(GTK_WINDOW_POPUP);
	g_signal_connect(smileysMenu, "key-release-event", 
					 G_CALLBACK(this->cb_smileysMenuKeyReleased), &smileysMenu);
	g_signal_connect(smileysMenu, "button-release-event", 
					 G_CALLBACK(gtk_widget_destroy), this);
	// don't show a title
	gtk_window_set_decorated(GTK_WINDOW(smileysMenu), FALSE);
	// don't show window in taskbar
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(smileysMenu), TRUE);
	
	// don't show window in pager
	gtk_window_set_skip_pager_hint(GTK_WINDOW(smileysMenu), TRUE); 

	
	// create a frame around the window
	frame = gtk_frame_new(0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
	gtk_container_add (GTK_CONTAINER (smileysMenu),
                      frame);

	


	// create the smiley buttons
	smileyBackground = gtk_event_box_new();
	u_setWidgetBackground(smileyBackground,"#ffffff");


	smileybox = gtk_vbox_new(FALSE,0);
	
		
	i = 0;
	imagePos = 0;
	hrow = NULL;
	if ( icons->images.smileys != NULL )
	while(icons->images.smileys[i].token)
	{
		if (icons->images.smileys[i].isFirstIcon)
		{
			if (!(imagePos % 4))
				hrow=gtk_hbox_new(FALSE,0);
		
			asmiley=gtk_button_new();
			gtk_container_set_border_width(GTK_CONTAINER(asmiley), 0);

			smileyimg=gtk_image_new_from_pixbuf(icons->images.smileys[i].icon);
			gtk_container_add(GTK_CONTAINER(asmiley),smileyimg);
			gtk_box_pack_start(GTK_BOX(hrow),asmiley,FALSE,TRUE,0);
			gtk_button_set_relief(GTK_BUTTON(asmiley),GTK_RELIEF_NONE);
		
			g_object_set_data(G_OBJECT(asmiley), "ICQmenuID", GUINT_TO_POINTER(i));
			g_signal_connect(asmiley,"clicked", G_CALLBACK(this->cb_smileyClicked), this);

			if (imagePos%4==3)
				gtk_box_pack_start(GTK_BOX(smileybox),hrow,FALSE,TRUE,0);
			
			imagePos++;
		}
		
		i++;
	}
			
	
	gtk_container_add(GTK_CONTAINER(smileyBackground),smileybox);	
	gtk_container_add(GTK_CONTAINER(frame),smileyBackground);
	 
	 
	// position the window
	// FIXME: we don't need the mouse position but the position of the pressed button
	scr = gdk_screen_get_default();
	gdk_display_get_pointer(gdk_display_get_default(), &scr, &posX, &posY, NULL);

	// FIXME: returns screen size, not the actual max-window-size
	//		  e.g. gnome-panel shrinkes it
	maxX = gdk_screen_get_width(scr);
	maxY = gdk_screen_get_height(scr);	

	
	gtk_window_get_size(GTK_WINDOW(smileysMenu), &width, &height);
	
	// set position so window doesn't get out of screen
	if (posX + width > maxX) 
		posX-= width;
	
	if (posY + height > maxY) 
		posY-= height;
		
	gtk_window_move(GTK_WINDOW(smileysMenu),posX,posY);

	
	

	// show the window
	gtk_widget_show_all(smileysMenu);	
	
	
	
	// grab mouse and keyboard events
	if (gdk_pointer_grab(smileysMenu->window, TRUE,
						 (GdkEventMask)(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
						 GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
						 GDK_POINTER_MOTION_MASK),
						 NULL, NULL,
						 gtk_get_current_event_time())==0)
	{
		gdk_keyboard_grab(smileysMenu->window, TRUE,
						  gtk_get_current_event_time());
	}
	gtk_grab_add(smileysMenu);
}




void conversationWindow::showOldEntries(gint toSkip)
{
	IMHistoryManager *hman;
	GList			 *entries,
					 *search;
	IMHistoryEntry	 *hentry;
	GtkTextIter		 iter;
	GtkTextMark		 *mark;
	gboolean		 showHistoryEntries;
	gulong			 historyCount;
	
	settings_getSettings()->getProperties("conversations", 
										  "showHistory", &showHistoryEntries,
										  "historyCount", &historyCount,
										  NULL);
	
	// we do not want to see the history? do nothing
	if (!showHistoryEntries)
		return;


	hman = new IMHistoryManager(manager->info);
	
	manager->parentUser->addManager(hman);
	
	// only show these events
	hman->readMask = HRT_MESSAGE | HRT_URL | HRT_INCOMING | HRT_OUTGOING;
	
	
	if (hman->loadHistory())
	{
		hman->setReadReverse(TRUE);
		hman->resetHistoryReading();
		hman->moveHistoryPosition(-toSkip);
		
		
		if (hman->historyLength() - toSkip >= historyCount)		
			entries = hman->getHistoryEntries(historyCount, TRUE);
		else
		{
			if (hman->historyLength()-toSkip>0)
				entries = hman->getHistoryEntries(hman->historyLength()-toSkip, TRUE);
			else
				entries = NULL;
		}

		
		search = g_list_last(entries);

		while(search)
		{		
			hentry = (IMHistoryEntry*)search->data;

			if (hentry->incoming)
				insertMessage(hentry->time, hentry->text, NULL, manager->parentUser, MT_HINCOMING, TRUE);
			else
				insertMessage(hentry->time, hentry->text, NULL, manager->parentUser->owner, MT_HOUTGOING, TRUE);

			search = search->prev;
		}
	}
	
	delete hman;	

	// scroll down
	gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView)), &iter);
	mark = gtk_text_buffer_get_mark(gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView)), "last_pos");
	gtk_text_buffer_move_mark(gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView)), mark, &iter);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(remoteView), mark, 0, FALSE, 0, 0);	
}



void conversationWindow::rebuildMessageView()
{
	GList				*search;
	smallHistoryEntry	*hEntry;
	GtkTextIter			startIter,
						endIter;
	GtkTextBuffer		*buf;
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(remoteView));
	
	// delete the old content
	gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	gtk_text_buffer_delete(buf, &startIter, &endIter);
	
	// reinsert the messages
	search = smallHistory;
	while(search)
	{
		hEntry = (smallHistoryEntry*)search->data;
		
		insertMessage(hEntry->time, hEntry->msg, hEntry->url, hEntry->user, hEntry->type, FALSE);
		
		search = search->next;
	}
}



gboolean conversationWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	messageEventInfo *msg;
	IMUserDaemon	 *user;

	
	switch(command)
	{
		// a new message event was received
		case UC_EVENT_RECEIVED:
			msg = (messageEventInfo*)info;
		
			user = ((IMOwnerDaemon*)manager->parentUser->owner)->findUserByID(msg->id);
			insertMessage(msg->basic->time, msg->text, msg->url, user, MT_INCOMING, TRUE);	
		
			// clear event if current window is active. if this is not the case
			// just drown them (maybe the user doesn't realize that there is an
			// event on another window and closes the program without having read
			// the message. this way the message won't be deleted from the main
			// event buffer but won't be printed again
				
			if (hasFocus)
				manager->deleteEvent(EO_FIRST | EO_NORMAL); 
			else 
			{
				// msg->basic->sunken = TRUE;
				
				
				// set the text color of the alias tab to blue (to inform the user that a message was received)			
				if (tabLabelText)
					setTabWidgetColor(&tabMessageColor);
			}
			break;
		
		case UC_EVENT_FINISHED:
			finishSendMessage((messageEventInfo*)info, lastResult);
			break;
		
		
			
		// the manager was installed -> add the history entries (that won't be shown later)
		case EC_INSTALLED:
			updateTitle();
			showOldEntries(lastResult);	
			oldStatus = manager->info->status;
		// the connection state has changed - update the label
		case UC_SENDSERVER:
			if (serverCheckbox)
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(serverCheckbox), manager->info->sendViaServer);
			break;	
		
			
		case UC_CONVO_ADD:
			insertMessage(time(NULL), tr(" has joined the conversation"), NULL, (IMUserDaemon*)info, MT_NOTIFICATION, TRUE); 
			break;
		
		
		case UC_CONVO_REMOVE:
			insertMessage(time(NULL), tr(" has left the conversation"), NULL, (IMUserDaemon*)info, MT_NOTIFICATION, TRUE);
			break;
						
		
		case UC_UPDATEUSER:
			switch(lastResult)
			{
				case USER_UPDATE_TYPING:
					if (!tabLabelText)
						break;
					
					switch(manager->info->typing)
					{
						// the user isn't typing anything (or stopped typing)
						case 0:
						case 1:
							if (manager->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL))
								setTabWidgetColor(&tabMessageColor);
							else
								setTabWidgetColor(NULL);
							break;

						// the user is typing
						case 2:
							setTabWidgetColor(&tabTypingColor);
							break;
					}
					break; 
					
				case USER_UPDATE_BASIC:
					updateTitle();
					break;
				
				case USER_UPDATE_STATUS:	
					if (oldStatus != manager->info->status)
					{
						if (manager->info->status == BUDDY_STATUS_OFFLINE)
							insertMessage(time(NULL), tr(" has logged off!"), NULL, manager->parentUser, MT_NOTIFICATION, TRUE);
						else
						if (manager->info->status != BUDDY_STATUS_OFFLINE && oldStatus == BUDDY_STATUS_OFFLINE)
							insertMessage(time(NULL), tr(" is online now!"), NULL, manager->parentUser, MT_NOTIFICATION, TRUE);

						if (statusEntry)
							gtk_entry_set_text(GTK_ENTRY(statusEntry), uu_getStatusDescription(manager->info->status));

						if (tabLabelImage)
							gtk_image_set_from_pixbuf(GTK_IMAGE(tabLabelImage), i_getIcons()->getUserStatusPixbuf(manager->parentUser));
				
						oldStatus = manager->info->status;
					}
					break;
					
				default: break;
			}
			break;
		
		
		default:
			fprintf(stderr, "conversationWindow::eventCallback: Unknown event (%d)!\n", command);
	}
	
	return TRUE;
}



void conversationWindow::settingsChangedCallback(gchar* sname, GList *values)
{
	settingsProperty *prop;
	gboolean		 updateColors,
					 updateView;
	gboolean		 recreateBars;
	GdkColor		 *col;
	gchar			 *tmp;
	charsetType		 *charset;
	GList			 *children;
	GtkWidget		 *charsetItem;

	if (!strcmp(sname, "appearance"))
	{
		while(values)
		{
			prop = (settingsProperty*)values->data;
			
			// update the message view with the smileys
			if (!strcmp(prop->name, "SmileyTheme"))
				rebuildMessageView();
			else/*
			// these images are stock icons -> just redraw the toolbars
			if (!strcmp(prop->name, "EventTheme"))
			{
				gtk_widget_queue_draw(toolbar1Container);
				gtk_widget_queue_draw(toolbar2Container);
				gtk_widget_queue_draw(toolbar3Container);
			} else */
			// update the status icon if neccessary
			if (tabLabelImage && !strcmp(prop->name, "StatusTheme"))
				gtk_image_set_from_pixbuf(GTK_IMAGE(tabLabelImage), i_getIcons()->getUserStatusPixbuf(manager->parentUser));
		
			
			values = values->next;
		}
		
	} else
	if (!strcmp(sname, "contactlist"))
	{
		while(values)
		{
			prop = (settingsProperty*)values->data;
			if (!strcmp(prop->name, "showRealNames"))
			{
				useRealName = g_value_get_boolean(prop->val);				
				
				if (useRealName)
					tmp = uu_getRealNameOrAlias(manager->info, FALSE);
				else
					tmp = g_strdup(manager->info->alias);
				gtk_label_set_label(GTK_LABEL(tabLabelText), tmp);
				g_free(tmp);
				
				break;
			}
			
			values = values->next;
		}
	} else
	if (!strcmp(sname, "conversations"))
	{
		updateColors = FALSE;
		updateView = FALSE;
		recreateBars = FALSE;
	
		while(values)
		{	
			prop = (settingsProperty*)values->data;
		
			if (!updateColors &&
				(!strcmp(prop->name, "colorsSet") ||
				 !strcmp(prop->name, "foregroundColor") ||
				 !strcmp(prop->name, "backgroundColor")))
				updateColors = TRUE;
		
			if (!strcmp(prop->name, "showSmileys"))
			{
				interpretSmileys = g_value_get_boolean(prop->val);
				if (toggleSmileysImg)
					gtk_widget_set_sensitive(toggleSmileysImg, interpretSmileys);
				updateView = TRUE;
			} else
			if (!strcmp(prop->name, "showURIs"))
			{
				interpretURIs = g_value_get_boolean(prop->val);
				if (toggleURLImg)
					gtk_widget_set_sensitive(toggleURLImg, interpretURIs);
				updateView = TRUE;
			} else
			if (!strcmp(prop->name, "dontApplyColors"))
			{
				dontApplyColors = g_value_get_boolean(prop->val);
				updateView = TRUE;
			} else
			if (!strcmp(prop->name, "sendByReturn"))
				sendByReturn = g_value_get_boolean(prop->val);
		
			if (!recreateBars &&
				(!strcmp(prop->name, "remoteToolbarRules") ||
				 !strcmp(prop->name, "remoteToolbarSpacing") ||
				 !strcmp(prop->name, "showRemoteToolbar") ||
				 !strcmp(prop->name, "localToolbarRules") ||
				 !strcmp(prop->name, "localToolbarSpacing") ||
				 !strcmp(prop->name, "showLocalToolbar") ||
				 !strcmp(prop->name, "buttonbarRules") ||
				 !strcmp(prop->name, "buttonbarSpacing") ||
				 !strcmp(prop->name, "showButtonbar")))
				recreateBars = TRUE;		
		
			if (!strcmp(prop->name, "tabMessageColor"))
			{
				col = (GdkColor*)g_value_get_pointer(prop->val);				
				tabMessageColor.red = col->red;
				tabMessageColor.green = col->green;
				tabMessageColor.blue = col->blue;
			}
			
			if (!strcmp(prop->name, "tabTypingColor"))
			{
				col = (GdkColor*)g_value_get_pointer(prop->val);
				tabTypingColor.red = col->red;
				tabTypingColor.green = col->green;
				tabTypingColor.blue = col->blue;
			}
		
			if (!strcmp(prop->name, "sendTyping"))
			{
				sendTyping = g_value_get_boolean(prop->val);
				// send a "no typing" notification
				if (!sendTyping)
					((IMMessageManager*)manager)->sendTypingNotification(0);		
			}
		
			if (!strcmp(prop->name, "messageTag"))
			{
				g_free(messageTag);
				
				messageTag = g_strdup(g_value_get_string(prop->val));
				u_stripStringByNeedle(messageTag, "|||");
				updateView = TRUE;
			}
			
			if (!(manager->info->charset && *manager->info->charset) && !strcmp(prop->name, "standardEncoding"))
			{
				charset = charset_findCharsetById((gchar*)g_value_get_string(prop->val));
				if (charset)
				{				
					children = gtk_container_get_children(GTK_CONTAINER(charsetMenu));
					charsetItem = (GtkWidget*)g_list_nth_data(children, charset->order);
					g_signal_handlers_block_by_func(charsetItem, (gpointer)this->cb_charsetListItemClicked, this);
					gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(charsetItem), TRUE);
					g_signal_handlers_unblock_by_func(charsetItem, (gpointer)this->cb_charsetListItemClicked, this);
					
					g_list_free(children);
				}
			}
		
			values = values->next;		
		}

	
		if (recreateBars)
			createToolbars();
	
		if (updateColors)
			updateLocalColors();
	
		if (updateView)
			rebuildMessageView();
	}
}




gchar* conversationWindow::getTextWithoutSmileys()
{
	GtkTextIter 		startIter, 
						endIter;
	gchar				*raw;
	GString				*toReturn;
	gunichar			c;
	GtkTextChildAnchor	*anchor;
	GtkTextBuffer		*buf;
	

	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(localView));
	
	gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	
	toReturn = g_string_new("");
	
	while(!gtk_text_iter_equal(&startIter, &endIter))
	{
		c = gtk_text_iter_get_char(&startIter);
		
		// this could be an image
		if (c == 0xFFFC && (anchor = gtk_text_iter_get_child_anchor(&startIter)))
		{
			raw = (gchar*)g_object_get_data(G_OBJECT(anchor), "ICQsmiley");
			g_string_append(toReturn, raw);
		} else
			g_string_append_unichar(toReturn, c);
		
		gtk_text_iter_forward_char(&startIter);
	}
	
	
	return g_string_free(toReturn, FALSE);
}




void conversationWindow::showMultipleSelectionList()
{
	if (multipleSelection)
		return;
	
	multipleSelection = new contactsSelection(manager->parentUser);
	gtk_widget_set_size_request(multipleSelection->widget, 150, -1);
	allocateExtraSpace(150+4);
	gtk_box_pack_start(GTK_BOX(mainHBox), multipleSelection->widget, FALSE, TRUE, 0);
	
	gtk_widget_show_all(multipleSelection->widget);
	
	u_setStockButtonImage(multipleButton, GTK_STOCK_GO_BACK);
	gtk_button_set_use_stock(GTK_BUTTON(multipleButton), TRUE);
}



void conversationWindow::hideMultipleSelectionList()
{
	if (!multipleSelection)
		return;
	
	gtk_container_remove(GTK_CONTAINER(mainHBox), multipleSelection->widget);
	deallocateExtraSpace();
	delete multipleSelection;
	multipleSelection = NULL;
	
	if (multipleButton)
		u_setStockButtonImage(multipleButton, GTK_STOCK_GO_FORWARD);
}


void conversationWindow::escPressed()
{	
	if (GTK_WIDGET_SENSITIVE(localView))
		cb_destroyWindow(this);
	else
	{
		stopSendMessage();
		// user clicked while sending -> cancel sending	
		manager->cancelInitatedEvent();	
	}
}




// =============================================================================

void conversationWindow::cb_sendButtonClicked(conversationWindow *cw)
{
	cw->sendMessage();
}



void conversationWindow::cb_sendServerButtonClicked(conversationWindow *cw)
{
	cw->manager->setSendViaServer(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw->serverCheckbox)));
}




void conversationWindow::cb_toolbarClicked(GtkWidget* clickedWidget, conversationWindow *cw)
{
	guint toCall;
	
	toCall = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(clickedWidget), "icqnd-menuid"));

	switch(toCall)
	{
		case TOOL_SENDFILE:
			cw->manager->callUser(UE_FILETRANSFERREQUEST, NULL);
			break;
		
		case TOOL_SENDLINK:
			if (!cw->isURLMode)
				cw->enableURLMode();
			else
				cw->disableURLMode();
			break;
		
		case TOOL_STARTCHAT:
			cw->manager->callUser(UE_CHATREQUEST, NULL);
			break;
		
		case TOOL_ENCRYPT:
			cw->manager->callUser(UE_ENCRYPTION, NULL);
			break;
		
		case TOOL_INFO:
			cw->manager->callUser(UE_USERINFO, NULL);
			break;
		
		case TOOL_HISTORY:
			cw->manager->callUser(UE_HISTORY, NULL);
			break;
		
		case TOOL_FGCOLOR:
		case TOOL_BGCOLOR:
			cw->selectColor(toCall == TOOL_FGCOLOR);
			break;
		
		case TOOL_SMILEYS:
			cw->createSmileysMenu();
			break;
		
		case TOOL_ENCODING:
			gtk_widget_show_all(cw->charsetMenu);
			gtk_menu_popup(GTK_MENU(cw->charsetMenu), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
			break;
		
		case TOOL_MULTIPLE:
			if (cw->multipleSelection)
				cw->hideMultipleSelectionList();
			else
				cw->showMultipleSelectionList(); 
			break;
			
			
		case TOOL_TOGGLESMILEYS:
			cw->interpretSmileys = !cw->interpretSmileys;
			gtk_widget_set_sensitive(cw->toggleSmileysImg, cw->interpretSmileys);
			cw->rebuildMessageView();
			break;
		
		case TOOL_TOGGLEURL:
			cw->interpretURIs = !cw->interpretURIs;
			gtk_widget_set_sensitive(cw->toggleURLImg, cw->interpretURIs);
			cw->rebuildMessageView();
			break;
		
		default:
			fprintf(stderr, "conversationWindow::cb_toolbarClicked(): Unknown event clicked\n");
			break;
		
	}		
}



gboolean conversationWindow::cb_setUserTime(conversationWindow *cw)
{
	gchar* tmp;
	time_t curTime;
	struct tm *tmTime;
	
	curTime = time(NULL) + cw->manager->info->localtime;
	tmTime = localtime(&curTime);
	
	
	tmp = g_strdup_printf("%02d:%02d:%02d",
						  tmTime->tm_hour, 
						  tmTime->tm_min, 
						  tmTime->tm_sec);
	gtk_entry_set_text(GTK_ENTRY(cw->timeEntry), tmp);
	g_free(tmp);
	
	if (cw->keyPressTime>0 && time(NULL) - cw->keyPressTime == 3)
		((IMMessageManager*)cw->manager)->sendTypingNotification(1);

	return TRUE;	
}



gboolean conversationWindow::cb_keyPressed(conversationWindow *cw, GdkEventKey *event)
{
	if (!event)
		return FALSE;
	
	if (cw->sendByReturn && 
		event->keyval == GDK_Return && !(event->state & GDK_CONTROL_MASK || event->state & GDK_MOD1_MASK))
		cw->sendMessage();
	else
	if (!cw->sendByReturn && 
		event->keyval == GDK_Return && (event->state & GDK_CONTROL_MASK || event->state & GDK_MOD1_MASK))
		cw->sendMessage();	
	
	return FALSE;
}



void conversationWindow::cb_localEntryChanged(conversationWindow *self)
{
	GtkTextIter	start, end;
	
	gtk_text_buffer_get_bounds(gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->localView)), &start, &end);

	if (self->sendButton)
		gtk_widget_set_sensitive(self->sendButton, !gtk_text_iter_equal(&start, &end));
	
	if (self->sendTyping)
	{
		if (!self->keyPressTime || time(NULL) - self->keyPressTime>=3)
			((IMMessageManager*)self->manager)->sendTypingNotification(2);
			
		self->keyPressTime = time(NULL);
	}
}



void conversationWindow::cb_colorSelectionCheckboxClicked(GtkWidget* w, GtkWidget* d)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
		gtk_widget_set_sensitive(GTK_COLOR_SELECTION_DIALOG(d)->colorsel, TRUE);
	else
		gtk_widget_set_sensitive(GTK_COLOR_SELECTION_DIALOG(d)->colorsel, FALSE);
}



gboolean conversationWindow::cb_smileysMenuKeyReleased(GtkWidget *w, GdkEventKey *event, GtkWidget **toDestroy)
{
	if (event->keyval != GDK_Left &&
		event->keyval != GDK_Right &&
		event->keyval != GDK_Up &&
		event->keyval != GDK_Down &&
		event->keyval != GDK_Tab)
	{
		gtk_widget_destroy(*toDestroy);
		*toDestroy = NULL;
	}
	
	return FALSE;
}



void conversationWindow::cb_smileyClicked(GtkWidget* w, conversationWindow *cw)
{
	GtkTextChildAnchor	*anchor;
	GtkTextBuffer		*buf;
	GtkTextIter			iter;
	GtkWidget			*img;
	guint				num;
	settings			*config;
	
	
	num = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(w), "ICQmenuID"));
	
	
	config = settings_getSettings();
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(cw->localView));
	
	gtk_text_buffer_get_iter_at_mark(buf, &iter,
									 gtk_text_buffer_get_mark(buf, "insert"));
	
	anchor = gtk_text_buffer_create_child_anchor(buf, &iter);
	g_object_set_data_full(G_OBJECT(anchor), "ICQsmiley", g_strdup(i_getIcons()->images.smileys[num].token), g_free);
	
	img = gtk_image_new_from_pixbuf(i_getIcons()->images.smileys[num].icon);
	gtk_widget_show(img);
	
	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(cw->localView), img, anchor);
	
	
	if (cw->smileysMenu)
		gtk_widget_destroy(cw->smileysMenu);
	
	cb_keyPressed(cw, NULL);
}



gboolean conversationWindow::cb_remoteViewMouseMoved(GtkWidget* v, GdkEventMotion* event, conversationWindow *self)
{
	u_checkInterpretedViewForURLCursor((gint)event->x, (gint)event->y, GTK_TEXT_VIEW(self->remoteView), &self->hoveredLink);
	
	gdk_window_get_pointer(self->remoteView->window, NULL, NULL, NULL);
	
	return FALSE;
}



gboolean conversationWindow::cb_remoteViewVisibilityChanged(GtkWidget* v, GdkEventVisibility* event, conversationWindow *self)
{
	gint cX, cY;
		
 	gdk_window_get_pointer (self->remoteView->window, &cX, &cY, NULL);
	u_checkInterpretedViewForURLCursor(cX, cY, GTK_TEXT_VIEW(self->remoteView), &self->hoveredLink);
	
	return FALSE;
}



gboolean conversationWindow::cb_remoteViewURLPressed(conversationWindow* self, GdkEventButton* event)
{	
	if (self->hoveredLink && event->button == 1)
		uu_openURL(self->hoveredLink);
	return FALSE;
}



void conversationWindow::cb_popupPopulated(conversationWindow* self, GtkMenu* menu)
{
	GtkWidget	*item;

	// only do something if we clicked on a URL
	if (self->hoveredLink)
	{
		// prepend a separator
		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
		
		// prepend the copy URL menu item
		item = gtk_image_menu_item_new_with_label(tr("Copy URL into clipboard"));
		g_signal_connect_swapped(item, "activate", G_CALLBACK(self->cb_copyToCBItemClicked), self);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
									  gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU));
		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), item);
		
		// prepend the open URL menu
		item = gtk_image_menu_item_new_with_label(tr("Open URL in Browser"));
		g_signal_connect_swapped(item, "activate", G_CALLBACK(self->cb_openURLItemClicked), self);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
									  gtk_image_new_from_stock(ICON_URL, GTK_ICON_SIZE_MENU));
		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), item);
		
		gtk_widget_show_all(GTK_WIDGET(menu));
	}
}


void conversationWindow::cb_copyToCBItemClicked(conversationWindow* self)
{
	GtkClipboard	*clipboard; 
	
	clipboard = gtk_clipboard_get_for_display(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);

	gtk_clipboard_set_text(clipboard, self->hoveredLink, -1);		
}



void conversationWindow::cb_openURLItemClicked(conversationWindow* self)
{
	uu_openURL(self->hoveredLink);
}



void conversationWindow::cb_charsetListItemClicked(GtkWidget *w, conversationWindow *self)
{
	gchar	*newCharset;
	
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w)))
	{
		newCharset = (gchar*)g_object_get_data(G_OBJECT(w), "icqnd-charset-set");
		self->manager->parentUser->setCharset(newCharset, TRUE);
	}
}



gboolean conversationWindow::cb_urlEntryTyped(conversationWindow* self)
{
	if (!*gtk_entry_get_text(GTK_ENTRY(self->URLentry)))
		gtk_widget_set_sensitive(self->goToButton, FALSE);
	else
		gtk_widget_set_sensitive(self->goToButton, TRUE);
	
	return FALSE;
}


void conversationWindow::cb_openURLButtonClicked(conversationWindow* self)
{
	uu_openURL((gchar*)gtk_entry_get_text(GTK_ENTRY(self->URLentry)));
}


IMUserDaemon* conversationWindow::cb_tooltipsFunction(GtkWidget* w, GdkEventMotion *event, conversationWindow* self)
{
	if (self->manager)
		return self->manager->parentUser;
	
	return NULL;
}
