/***************************************************************************
 *            imuserdaemon.cpp
 *
 *  Thu Jan 20 00:08:25 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "imuserdaemon.h"
#include "imownerdaemon.h"

IMGroupManager	*gMan;
IMSignalSource	*generalSource = NULL;


IMSignalSource::IMSignalSource()
{
	callbacks = NULL;
}



IMSignalSource::~IMSignalSource()
{
	clearCallbacks();
}


void IMSignalSource::clearCallbacks()
{
	g_list_foreach(callbacks, (GFunc)g_free, NULL);
	g_list_free(callbacks);

	callbacks = NULL;	
}


gboolean IMSignalSource::containsCallback(gpointer toCheck)
{
	GList				*search;
	IMSignalSourceCB	*cb;
	
	for (search = callbacks; search; search = search->next)
	{
		cb = (IMSignalSourceCB*)search->data;
		if (cb->cb == toCheck)
			return TRUE;
	}
	
	return FALSE;
}


void IMSignalSource::addCallback(IMEventCallback cb, gpointer data)
{
	
	IMSignalSourceCB	*newcb;

	newcb = (IMSignalSourceCB*)g_malloc0(sizeof(IMSignalSourceCB));	
	newcb->cb = cb;
	newcb->data = data;
	
	callbacks = g_list_append(callbacks, newcb); 
}


void IMSignalSource::removeCallback(IMEventCallback cb)
{
	GList				*search,
						*callbacksCopy;
	IMSignalSourceCB 	*currentCb;
	
	callbacksCopy = g_list_copy(callbacks);
	
	for (search = callbacksCopy; search; search = search->next)
	{
		currentCb = (IMSignalSourceCB*)search->data;
		
		if (currentCb->cb == cb)
		{
			callbacks = g_list_remove(callbacks, currentCb);
			g_free(currentCb);
			break;
		}
	}
	
	g_list_free(callbacksCopy);
}

	
gboolean IMSignalSource::startCallback(userEventType type, gint command, gint lastResult, gpointer data)
{
	GList				*search,
						*callbacksCopy;
	gboolean			success;
	IMSignalSourceCB 	*cb;
	
	success = TRUE;
	
	callbacksCopy = g_list_copy(callbacks);
	
	for (search = callbacksCopy; search; search = search->next)
	{
		cb = (IMSignalSourceCB*)search->data;
		if (cb->cb)
			if (!cb->cb(type, command, lastResult, data, cb->data))
				success = FALSE;
	}
	
	g_list_free(callbacksCopy);
		
	return success; 
}



IMEventManager::IMEventManager(IMUserInfo	*_info)
{
	eventQueue	  = NULL;
		
	initatedEventID = 0;
	isProcessing  = FALSE;	
	
	type 	  	  = UE_UNDEFINED;
	listensTo 	  = NULL;
	
	info 		  = _info;
	
	parentUser	  = NULL;
	
	canDropEvents = TRUE;
}




IMEventManager::~IMEventManager()
{	
	GList				*search;
	generalEventInfo	*ev;
	
	g_list_free(listensTo);
	
	search = eventQueue;
	while(search)
	{
		ev = (generalEventInfo*)search->data;
		
		ev->basic->refCount--;
		
		search = search->next;
	}
	
	if (initatedEventID)
		cancelInitatedEvent();
	
	// tell the parentUser to remove us from the list
	if (parentUser)
		parentUser->removeManager((IMEventManager*)this);
}



	
// adds an event to the queue
gpointer IMEventManager::addEvent(basicEventInfo* _basics)
{
	gpointer newEvent;
	
	newEvent = mergeEvent(_basics->licqEvent);
	((generalEventInfo*)newEvent)->basic = _basics;
	
	eventQueue = g_list_append(eventQueue, newEvent);
	
	return newEvent;
}



	
// returns an event
gpointer IMEventManager::getEvent(gchar returnFlag)
{
	GList *search;
	generalEventInfo* ev;
	
	
	if (returnFlag & EO_LAST)
		search = g_list_last(eventQueue);
	else
		search = eventQueue;
	
	while(search)
	{
		ev = (generalEventInfo*)search->data;

		if ( ( (returnFlag & EO_SYSTEM) && ev->basic->isOwnerEvent) ||
			 ( (returnFlag & EO_SUNKEN) && ev->basic->sunken) ||
			 ( (returnFlag & EO_NORMAL) && !ev->basic->sunken) )
		return search->data;
		
		if (returnFlag & EO_LAST)
			search = search->prev;
		else
			search = search->next;
	}
	
	return NULL;
}



	
// deletes an event
gboolean IMEventManager::deleteEvent(gchar deleteFlag)
{
	GList 			 *search,
					 *eventCopy;
	gboolean 		 deletedSomething;
	generalEventInfo *event;
		
	eventCopy = g_list_copy(eventQueue);
	
	if (deleteFlag & EO_LAST)
		search = g_list_last(eventCopy);
	else
		search = eventCopy;
	
	
	deletedSomething = FALSE;
	
	
	while(search)
	{
		event = (generalEventInfo*)search->data;

		
		if ( ((deleteFlag & EO_SUNKEN) &&  event->basic->sunken) ||
			 ((deleteFlag & EO_NORMAL) && !event->basic->sunken) )
		{					
			eventQueue = g_list_remove(eventQueue, search->data);			
			
			if (parentUser)
				parentUser->clearEvent(event->basic, canDropEvents);
			else
				fprintf(stderr, "IMEventManager::deleteEvent: FATAL: couldn't delete an event because no user was found!\n");
			
			g_free(search->data);
			
			deletedSomething = TRUE;
			
			if (!(deleteFlag & EO_ALL))
				break;
		}
		
		if (deleteFlag & EO_LAST)
			search = search->prev;
		else
			search = search->next;
	}
	
	g_list_free(eventCopy);

	return deletedSomething;	
}



gboolean IMEventManager::eraseEventByBasicInfo(basicEventInfo* basicInfo)
{
	GList			 *search;
	generalEventInfo *ev;
	
	search = eventQueue;
	
	while(search)
	{
		ev = (generalEventInfo*)search->data;
		if (ev->basic == basicInfo)
		{
			eventQueue = g_list_remove(eventQueue, search->data);
			g_free(search->data);
			break;
		}
		search = search->next;
	}
	
	return (search!=NULL);
}



void IMEventManager::cancelInitatedEvent()
{
	isProcessing = FALSE;
	
	if (initatedEventID == 0)
		return;
	
	getLicqDaemon()->CancelEvent(initatedEventID);
	initatedEventID = 0;
}


	

	
// sets the sendViaServer flag
void IMEventManager::setSendViaServer(gboolean newState)
{
	if (info->sendViaServer != newState)
	{
		info->sendViaServer = newState;
		info->sendViaServerByUser = TRUE;
		
		// this is somehow cyclic stuff but however
		if (parentUser)
			parentUser->informAllManagers(UC_SENDSERVER, (gint)newState, NULL);
	}
}





	
	
// merges the event to fit the current manager and returns a filled struct
gpointer IMEventManager::mergeEvent(CUserEvent* event)
{
	generalEventInfo	*newEvent;
	
	newEvent = (generalEventInfo*)g_malloc0(sizeof(generalEventInfo));

	return newEvent;
}



gboolean IMEventManager::callUser(gint command, gpointer data)
{
	if (parentUser)
	{
		parentUser->startCallback(EC_FROMMANAGER, command, data);
		return TRUE;
	}
	return FALSE;
}



gboolean IMEventManager::finishEvent(ICQEvent* event)
{
	generalEventInfo* resultEvent;
	
	initatedEventID = 0;
	isProcessing = FALSE;
	
	// before doing anything -> try to resend the event	
	if (!info->sendViaServer && 
		event->Result() != EVENT_RESULT_SUCCESS && 
		event->Result() != EVENT_RESULT_ACKED)
	{
		// the event was returned. Resend it to the list
		if (!info->sendViaServer)
		{
			// WARNING: can this create a loop?
			if (event->SubResult() == ICQ_TCPxACK_RETURN || !info->sendViaServer)
				retrySendingEvent(event, ICQ_TCPxMSG_LIST);
			// somehow the event couldn't be sent. Try it via server ...
			else
			{
				setSendViaServer(TRUE);
				retrySendingEvent(event);
			}
		}
		
	} else
	
	// this has worked - create a temporary event for checking 
	{
		if (event->UserEvent())
			resultEvent = (generalEventInfo*)mergeEvent(event->UserEvent());
		else
			resultEvent = (generalEventInfo*)IMEventManager::mergeEvent(event->UserEvent());

		resultEvent->basic = parentUser->createBasicEvent(event->UserEvent());
		
		startCallback(UC_EVENT_FINISHED, event->Result(), resultEvent);
		
		return TRUE;
		// FIXME: free allocated event
	}
	
	return FALSE;
}



void IMEventManager::retrySendingEvent(ICQEvent* event, gushort level)
{
	// nothing to do here
}




gboolean IMEventManager::isEventProcessing()
{
	return (initatedEventID != 0);
}



void IMEventManager::setDropEvents(gboolean state)
{
	canDropEvents = state;
}

		

void IMEventManager::conversationUserAdded(IMUserDaemon* user)
{
	startCallback(UC_CONVO_ADD, 0, user);
}
	

void IMEventManager::conversationUserRemoved(IMUserDaemon* user)
{
	startCallback(UC_CONVO_REMOVE, 0, user);
}



// ==================== IMUserUpdateManager ===================================

IMUserUpdateManager::IMUserUpdateManager(IMUserInfo *_info):
IMEventManager::IMEventManager(_info)
{
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)ICQ_CMDxMETA_REQUESTxALLxINFO));	// listens to information update events
	sendUpdateStep = 0;
}




IMUserUpdateManager::~IMUserUpdateManager()
{
	
}




#define IMUT_SAVEINFORMATION(var, cmd)	\
	tmp = convertFromSystemCharset(info->var, info->charset);	\
	u->cmd(tmp);												\
	g_free(tmp);

	
void IMUserUpdateManager::saveUserInfo()
{
	ICQUser			*u;
	gchar			*tmp;
	ICQUserCategory	*cat;
	GList			*searchc,
					*searche;
	GString			*interestEntries;
	IMUserListInfo	*catInfo;
	gint			i;
	
	
	u = parentUser->getLicqUser();
	
	u->SetEnableSave(false);
		
	
	// save "general" info
	IMUT_SAVEINFORMATION(firstname, SetFirstName);
	IMUT_SAVEINFORMATION(lastname, SetLastName);
	IMUT_SAVEINFORMATION(alias, SetAlias);
	u->SetKeepAliasOnUpdate(info->keepAlias);	
	IMUT_SAVEINFORMATION(email1, SetEmailPrimary);
	IMUT_SAVEINFORMATION(email2, SetEmailSecondary);
  	IMUT_SAVEINFORMATION(email3, SetEmailOld);
  	IMUT_SAVEINFORMATION(home.city, SetCity);
  	IMUT_SAVEINFORMATION(home.state, SetState);
  	IMUT_SAVEINFORMATION(home.address, SetAddress);
  	IMUT_SAVEINFORMATION(home.zip, SetZipCode);
  	IMUT_SAVEINFORMATION(home.mobile, SetCellularNumber);
  	IMUT_SAVEINFORMATION(home.phone, SetPhoneNumber);
  	IMUT_SAVEINFORMATION(home.fax, SetFaxNumber);

	u->SetCountryCode(GetCountryByIndex(info->home.country)->nCode);

	
	// save "work" info
	IMUT_SAVEINFORMATION(company.city, SetCompanyCity);
	IMUT_SAVEINFORMATION(company.state, SetCompanyState);
	IMUT_SAVEINFORMATION(company.address, SetCompanyAddress);
	IMUT_SAVEINFORMATION(company.zip,SetCompanyZip);
	IMUT_SAVEINFORMATION(company.phone, SetCompanyPhoneNumber);
	IMUT_SAVEINFORMATION(company.fax, SetCompanyFaxNumber);
	IMUT_SAVEINFORMATION(company.department, SetCompanyDepartment);
	IMUT_SAVEINFORMATION(company.position, SetCompanyPosition);
	IMUT_SAVEINFORMATION(company.name, SetCompanyName);
	IMUT_SAVEINFORMATION(company.website, SetCompanyHomepage);
	u->SetCompanyCountry(GetCountryByIndex(info->company.country)->nCode);
		
	// save "more" info
	IMUT_SAVEINFORMATION(website, SetHomepage);
	u->SetAge(info->age);
	u->SetGender(info->gender);
	u->SetBirthDay(info->birthday.day);
	u->SetBirthMonth(info->birthday.month);
	u->SetBirthYear(info->birthday.year);
	u->SetLanguage1(GetLanguageByIndex(info->language1)->nCode);
	u->SetLanguage2(GetLanguageByIndex(info->language2)->nCode);
	u->SetLanguage3(GetLanguageByIndex(info->language3)->nCode);


	
	// save "about" info
	IMUT_SAVEINFORMATION(about, SetAbout);	

	// save interests info
	interestEntries = g_string_new("");
	
	for (i=0; i<3; i++)
	{
		switch(i)
		{
			case 0: cat = u->GetInterests(); searchc = info->interests; break;
			case 1: cat = u->GetBackgrounds(); searchc = info->background; break;
			case 2: cat = u->GetOrganizations(); searchc = info->organizations; break;
		}

		cat->Clean();

		while(searchc)
		{
			catInfo = (IMUserListInfo*)searchc->data;
			interestEntries = g_string_set_size(interestEntries, 0);
			searche = catInfo->entries;
			while(searche)
			{
				g_string_append_printf(interestEntries, "%s, ", (gchar*)searche->data);
				searche = searche->next;
			}
		
			if (strlen(interestEntries->str))
				interestEntries = g_string_set_size(interestEntries, strlen(interestEntries->str)-2);
		
			cat->AddCategory(catInfo->headlineNumber, convertFromSystemCharset(interestEntries->str, info->charset));
		
			searchc = searchc->next;
		}
	}
	
		
	g_string_free(interestEntries, TRUE);
	
	
	
	u->SetEnableSave(true);
	
	u->SaveGeneralInfo();
	u->SaveMoreInfo();
	u->SaveWorkInfo();
	u->SaveAboutInfo();
	u->SaveExtInfo();
	u->SaveInterestsInfo();
	u->SaveOrganizationsInfo();
	u->SaveBackgroundsInfo();

	// save the user name to the server
	getLicqDaemon()->ProtoRenameUser(info->licqID, info->pluginID);
	
	parentUser->dropLicqUser(u);
	
	parentUser->dispatchInformation(NULL);
}




void IMUserUpdateManager::updateUserInfo(gchar* aliasToKeep)
{
	ICQUser* u;
	gchar* tmp;
	
	u = parentUser->getLicqUser();	

	u->SetEnableSave(false);
	
	if (aliasToKeep)
	{		
		u->SetKeepAliasOnUpdate(true);
		tmp = convertToSystemCharset(aliasToKeep, info->charset);
		u->SetAlias(aliasToKeep);
		g_free(tmp);
	} else
		u->SetKeepAliasOnUpdate(false);
	
	u->SetEnableSave(true);
	u->SaveGeneralInfo();
	
	parentUser->dropLicqUser(u);	
	
	initatedEventID = getLicqDaemon()->ProtoRequestInfo(info->licqID, info->pluginID);
	
	isProcessing = TRUE;
}


void IMUserUpdateManager::sendUserInfo()
{
	sendUpdateStep = 1;
	finishEvent(NULL);
}



gboolean IMUserUpdateManager::finishEvent(ICQEvent* event)
{
	gchar	*alias,
			*firstName,
			*lastName,
			*email,
			*city,
			*state,
			*phone,
			*fax,
			*address,
			*mobile,
			*zip;
	gchar	*homepage;
	gchar	*about;
	gchar	*compState,
			*compCity,
			*compPhone,
			*compFax,
			*compAddress,
			*compZip,
			*compName,
			*compDep,
			*compPos,
			*compSite;
	
	
	if (event)
		if (sendUpdateStep == 0 || event->Result() != EVENT_RESULT_ACKED || event->SubResult() == ICQ_TCPxACK_RETURN)
			return IMEventManager::finishEvent(event);
	
	switch(sendUpdateStep)
	{
		case 1:
			alias = convertFromSystemCharset(info->alias, info->charset);
			firstName = convertFromSystemCharset(info->firstname, info->charset);
			lastName = convertFromSystemCharset(info->lastname, info->charset);
			email = convertFromSystemCharset(info->email1, info->charset);
			city = convertFromSystemCharset(info->home.city, info->charset);
			state = convertFromSystemCharset(info->home.state, info->charset);
			phone = convertFromSystemCharset(info->home.phone, info->charset);
			fax = convertFromSystemCharset(info->home.fax, info->charset);
			address = convertFromSystemCharset(info->home.address, info->charset);
			mobile = convertFromSystemCharset(info->home.mobile, info->charset);
			zip = convertFromSystemCharset(info->home.zip, info->charset);
		
			sendUpdateStep++;
		
			initatedEventID = getLicqDaemon()->ProtoSetGeneralInfo(info->pluginID,
																   alias, firstName, lastName, email,
																   city, state, phone, fax, 
																   address, mobile, zip,
																   info->home.country, false);

			isProcessing = TRUE;
			g_free(alias); g_free(firstName); g_free(lastName); g_free(email);
			g_free(city); g_free(state); g_free(phone); g_free(fax); g_free(address);
			g_free(mobile); g_free(zip);
			break;
		
		case 2:
			homepage = convertFromSystemCharset(info->website, info->charset);
			sendUpdateStep++;
			initatedEventID = getLicqDaemon()->icqSetMoreInfo(info->age,
															  info->gender,
															  homepage,
															  info->birthday.year,
															  info->birthday.month,
															  info->birthday.day,
															  info->language1,
															  info->language2,
															  info->language3);
			isProcessing = TRUE;
			g_free(homepage);
			break;
		
		case 3:
			about = convertFromSystemCharset(info->about, info->charset);
			sendUpdateStep++;
			initatedEventID = getLicqDaemon()->icqSetAbout(about);
			isProcessing = TRUE;
			g_free(about);
			break;
		
		case 4:
			compCity = convertFromSystemCharset(info->company.city, info->charset);
			compState = convertFromSystemCharset(info->company.state, info->charset);
			compPhone = convertFromSystemCharset(info->company.phone, info->charset);
			compFax = convertFromSystemCharset(info->company.fax, info->charset);
			compAddress = convertFromSystemCharset(info->company.address, info->charset);
			compZip = convertFromSystemCharset(info->company.zip, info->charset);
			compName = convertFromSystemCharset(info->company.name, info->charset);
			compDep = convertFromSystemCharset(info->company.department, info->charset);
			compPos = convertFromSystemCharset(info->company.position, info->charset);
			compSite = systemCharsetToLocale(info->company.website);	
		
			sendUpdateStep++;
			initatedEventID = getLicqDaemon()->icqSetWorkInfo(compCity, compState,
										   					  compPhone, compFax,
										   					  compAddress, compZip,
										   					  info->company.country,
										   					  compName, compDep, compPos,
										   					  0,
										   					  compSite);
		
			isProcessing = TRUE;
		
			g_free(compCity); g_free(compState); g_free(compPhone); g_free(compFax);
			g_free(compAddress); g_free(compZip); g_free(compName); g_free(compDep);
			g_free(compPos); g_free(compSite);
			break;
			
		case 5:
			IMEventManager::finishEvent(event);
			sendUpdateStep = 0;
			break;
	}
	
	return TRUE;
}


// ==================== IMHistoryManager =======================================

IMHistoryManager::IMHistoryManager(IMUserInfo* _info):
IMEventManager::IMEventManager(_info)
{
	num = 0;
	numRead = 0;
	entries = NULL;
	readReversely = FALSE;
	readMask = HRT_ALL;
	pattern = NULL;
	threadID = 0;
}



IMHistoryManager::~IMHistoryManager()
{
	stopLoadingHistoryFiltered();
	unloadHistory();
}




gboolean IMHistoryManager::loadHistory()
{
	unloadHistory();
	
	if (!loadLicqHistory())
		return FALSE;
	
	resetHistoryReading();

	return TRUE;	
}




guchar IMHistoryManager::loadHistoryFiltered(gchar* _pattern)
{
	gchar	*wpattern,
			*tmppattern;
	

	if (!loadLicqHistory())
		return 2;
	
	if (!licqList.size())
		return 1;
	
	
	// remove leading and trailing whitespaces
	tmppattern = g_strdup(_pattern);
	wpattern = g_strstrip(tmppattern);
		
	// check whether the history can be scanned
	if (!wpattern || strlen(wpattern)<3)
	{
		resetHistoryReading();
		
		g_free(pattern);
		pattern = NULL;	
		
		if (!*wpattern)
		{
			g_free(tmppattern);
			return 3;
		}
		else
		{
			g_free(tmppattern);
			return 4;
		}
	}		
			
	if (pattern)
		g_free(pattern);
	
	pattern = g_strdup(wpattern);
	
	g_free(tmppattern);
	
	startCallback(EC_LOADHISTORY, 0, NULL);
	
	pthread_create(&threadID, NULL, (void*(*)(void*))this->t_loadHistoryFiltered, this);
		
	return 0;
}



void IMHistoryManager::stopLoadingHistoryFiltered()
{
	if (threadID!=0)
	pthread_cancel(threadID);
	
	threadID = 0;
}



guint IMHistoryManager::historyLength()
{
	return num;
}


gint IMHistoryManager::historyPosition()
{
	return numRead;
}



GList* IMHistoryManager::getHistoryEntries(gint count, gboolean increasePosition)
{
	gint i;
	CUserEvent* ev;
	IMHistoryEntry* hentry;
	HistoryListIter tmpiter;
	
	freeConvertedEntries();
	
	if (!readReversely && (count + numRead > num))
		count = num - numRead;
	
	
	if (readReversely && (numRead - count < 0))
	{
		if (num>=count)
		{
			moveHistoryPosition(count - numRead);
			numRead = count;
		} else
		{
			moveHistoryPosition(num);
			numRead = num;
			count = num;
		}
	}
	

	if (!increasePosition)
		tmpiter = licqListIter;
		
	
	for (i=0; i<count; i++)
	{
		ev = *licqListIter;
		
		hentry = (IMHistoryEntry*)g_malloc0(sizeof(IMHistoryEntry));
		
		hentry->incoming = (ev->Direction() == D_RECEIVER);
		hentry->text	 = convertToSystemCharset((char*)ev->Text(), info->charset);
		hentry->time	 = ev->Time();
		
		entries = g_list_append(entries, hentry);
		
		if (readReversely)
			licqListIter--;
		else
			licqListIter++;
	}	
	
	
	if (!increasePosition)
		licqListIter = tmpiter;
	else
	{
		if (readReversely)
			numRead-=i;
		else
			numRead+=i;		
	}
	
	return entries;
}




void IMHistoryManager::unloadHistory()
{
	freeConvertedEntries();
	num = 0;
	
}



void IMHistoryManager::freeConvertedEntries()
{
	guint i;
	GList* search;
		
	if (!entries)
		return;
		
	i=0;
	
	search = entries;
	
	while(search)
	{
		g_free(((IMHistoryEntry*)search->data)->text);
		search = search->next;
	}
	
	g_list_free(entries);
	entries = NULL;
}



void IMHistoryManager::setReadReverse(gboolean reverse)
{
	readReversely = reverse;
}



gboolean IMHistoryManager::getReadReverse()
{
	return readReversely;
}





gboolean IMHistoryManager::loadLicqHistory()
{
	ICQUser *u;
	gboolean success;

	if (licqList.size()>0)
		licqList.clear();
	
	u = parentUser->getLicqUser();
	
	success = TRUE;
	
	// get history (list is a vector containing CUserEvent s)
	if (!u->GetHistory(licqList))
		// if there is no history file the user doesn't have a history ...
		success = (u->HistoryFile() == NULL);
	
	parentUser->dropLicqUser(u);

	return success;	
}


void IMHistoryManager::resetHistoryReading()
{
	filterLicqHistoryByMask();
	
	if (readReversely)
	{
		licqListIter = ownList.end();
		licqListIter--;
		numRead = ownList.size();
	}
	else
	{
		licqListIter = ownList.begin();
		numRead = 0;
	}
	
	num = ownList.size();	
}




void IMHistoryManager::moveHistoryPosition(gint steps)
{
	gint i;

	if (steps == 0)
		return;
	else
	
	if (steps<0 && numRead+steps<0)
		steps = -numRead;		
	else

	if (steps>0 && numRead+steps>num)
		steps = num-numRead;
		
	
	for (i=0; i<abs(steps); i++)
	{
		if (steps<0)
			licqListIter--;
		else
			licqListIter++;
	}
	
	
	numRead+=steps;
}



void IMHistoryManager::filterLicqHistoryByMask()
{
	HistoryListIter tmpiter;
	guint			i, lsize;
	CUserEvent		*ev;
	gushort			subCommand;
	
	if (ownList.size()>0)
		ownList.clear();
	
	tmpiter = licqList.begin();
		
	lsize = licqList.size();
	
	for (i=0; i<lsize; i++)
	{
		ev = *tmpiter;
		subCommand = ev->SubCommand();
		if ( (ev->Direction() == D_RECEIVER && readMask & HRT_INCOMING) ||
			 (ev->Direction() != D_RECEIVER && readMask & HRT_OUTGOING) )
		{
			if ((subCommand == ICQ_CMDxSUB_MSG && readMask & HRT_MESSAGE) ||
				(subCommand == ICQ_CMDxSUB_URL && readMask & HRT_URL)	  ||
				(subCommand == ICQ_CMDxSUB_SMS && readMask & HRT_SMS)	  ||
				(subCommand == ICQ_CMDxSUB_FILE && readMask & HRT_FILE)	  ||
				(subCommand == ICQ_CMDxSUB_CHAT && readMask & HRT_CHAT) )
				ownList.push_back(ev);			
		} 
			
		tmpiter++;
	}
	
}



void IMHistoryManager::doFilterHistory()
{
	gint i, lsize;
	HistoryList tmplist;
	HistoryListIter tmpiter;
	CUserEvent* ev;
	
	tmpiter = licqList.begin();
	
	lsize = licqList.size();
	
	for (i=0; i< lsize; i++)
	{
		ev = *tmpiter;
		if (g_strrstr(ev->Text(), pattern)!= NULL)
			tmplist.push_back(ev);
		
		tmpiter++;	

		if (i%100 == 0 && (i*100)/lsize != 100 && (i*100)/lsize != 0)
		{
			gdk_threads_enter();
			startCallback(EC_LOADHISTORY, (i*100)/lsize, NULL);
			gdk_threads_leave();
		}	
		
		pthread_testcancel();
	}
	
	licqList = tmplist;
	
	resetHistoryReading();
	
	gdk_threads_enter();
	startCallback(EC_LOADHISTORY, 100, NULL);
	gdk_threads_leave();
}



guchar IMHistoryManager::saveHistory(gchar* fileName, gboolean overwriteExisting)
{
	FILE		*hfile;
	gchar		*sender, *tmp;
	gchar		htime[100];
	CUserEvent *ev;
	gint		i, lsize;
	time_t		evtime;
	HistoryListIter tmpiter;
	struct stat	fattr;
		
	// check if file already exists
	if (stat((char*)fileName, &fattr)==0)
	{
		// file name is a directory
		if (S_ISDIR(fattr.st_mode))
			return 2;
		else
		// file exists
		if (!overwriteExisting)
			return 1;
	}
	
	hfile = fopen(fileName, "w");
	
	if (!hfile)
		return 3;
	
		// FIXME: don't write blanks when there is no first or last name
	if (fprintf(hfile, tr("history from %s (%s %s)\n---\n\n"), info->alias, info->firstname, info->lastname)<0)
	{
		fclose(hfile);
		return 4;
	}
	
	
	lsize = ownList.size();
	
	if (readReversely)
	{
		tmpiter = ownList.end();
		tmpiter--;
	} else
		tmpiter = ownList.begin();
	
	for (i=0; i<lsize; i++)
	{
		ev = *tmpiter;
		
		if (ev->Direction() == D_RECEIVER)
			sender = tr("Received");
		else
			sender = tr("Sent");
		
		evtime = ev->Time();
		strftime(htime, 26, tr("in %m/%d/%y at %H:%M:%S"), localtime(&evtime));
		
		// FIXME: convert to unicode
		tmp = convertToSystemCharset((gchar*)ev->Text(), info->charset);
		if (fprintf(hfile, "%s %s\n %s\n-------\n\n\n\n", sender, htime, tmp)<0)
		{
			g_free(tmp);
			fclose(hfile);
			return 4;
		}
		
		g_free(tmp);
		
		tmpiter++;
	}
	
	fclose(hfile);
	
	return 0;
}




void* IMHistoryManager::t_loadHistoryFiltered(IMHistoryManager* self)
{
	self->doFilterHistory();
	pthread_exit(NULL);
}




// ================= IMAutoResponseManager =====================================

IMAutoResponseManager::IMAutoResponseManager(IMUserInfo	*_info, gboolean isOwner):
IMEventManager::IMEventManager(_info)
{
	isOwnerManager = isOwner;
	type = UE_AUTORESPONSEGET;
}



IMAutoResponseManager::~IMAutoResponseManager()
{
	
}



void IMAutoResponseManager::fetchAutoResponse()
{	
	gboolean sendServer;
	ICQUser	 *u;
	
	u = parentUser->getLicqUser();
	sendServer = (u->SocketDesc(ICQ_CHNxNONE) <= 0 && u->Version() > 6);	
	parentUser->dropLicqUser(u);	
	
	initatedEventID = getLicqDaemon()->icqFetchAutoResponse(info->licqID, info->pluginID, sendServer);
	
	isProcessing = TRUE;	
}


	

gboolean IMAutoResponseManager::fetchTemplates(guint awayMode, GList** contents, GList** names)
{
	SARList				curList;
	CSavedAutoResponse	*curResponse;
	guint				arIndex, i;
	
	
	switch(awayMode)
	{
		case BUDDY_STATUS_AWAY: arIndex = 0; break;
		case BUDDY_STATUS_NA:   arIndex = 1; break;
		case BUDDY_STATUS_OCC:  arIndex = 2; break;
		case BUDDY_STATUS_DND:  arIndex = 3; break;
		case BUDDY_STATUS_FFC:  arIndex = 4; break;
		default: return FALSE;
	}

	curList = gSARManager.Fetch(arIndex);

	if (contents)
		*contents = NULL;
		if (names)
	*names = NULL;
	
	for (i=0;i<curList.size();i++)
	{
		curResponse = (CSavedAutoResponse*)curList[i];
		/*
		*contents = g_list_append(*contents, convertToSystemCharset((gchar*)curResponse->AutoResponse(), info->charset));
		*names = g_list_append(*names, convertToSystemCharset((gchar*)curResponse->Name(), info->charset));		
		*/
		if (contents)
			*contents = g_list_append(*contents, localeToSystemCharset((gchar*)curResponse->AutoResponse()));
		if (names)
			*names = g_list_append(*names, localeToSystemCharset((gchar*)curResponse->Name()));		
		
	}
		
	gSARManager.Drop();

	return TRUE;
}


void IMAutoResponseManager::writeTemplates(guint awayMode, GList* contents, GList* names)
{
	guint				arIndex, i;
	gchar				*newName,
						*newContent;
	
	
	switch(awayMode)
	{
		case BUDDY_STATUS_AWAY: arIndex = 0; break;
		case BUDDY_STATUS_NA:   arIndex = 1; break;
		case BUDDY_STATUS_OCC:  arIndex = 2; break;
		case BUDDY_STATUS_DND:  arIndex = 3; break;
		case BUDDY_STATUS_FFC:  arIndex = 4; break;
		default: return;
	}
		
	SARList& curList = gSARManager.Fetch(arIndex);
	
	i=0;
	while(names)
	{
		newName = systemCharsetToLocale((gchar*)names->data);
		newContent = systemCharsetToLocale((gchar*)contents->data);
		
		delete curList[i];
		curList[i] = new CSavedAutoResponse(newName, newContent);
		
		g_free(newName);
		g_free(newContent);
		
		names = names->next;
		contents = contents->next;
		i++;
	}
	
	gSARManager.Drop();
	gSARManager.Save();
}



void IMAutoResponseManager::setCustomResponse(gchar* msg, gboolean informUser)
{
	ICQUser *u;
	gchar   *txt;
	
	if (info->specialAR)
		g_free(info->specialAR);		
	
	info->specialAR = g_strdup(msg);
	
	txt = convertFromSystemCharset(msg, info->charset);
	
	u = parentUser->getLicqUser();
	if (isOwnerManager)
		u->SetAutoResponse(txt);
	else	
		u->SetCustomAutoResponse(txt);
	
	g_free(txt);
	
	parentUser->dropLicqUser(u);
	
	if (informUser)
		parentUser->startCallback(UC_SETAUTORESPONSE, 0, info->specialAR);
}



gchar* IMAutoResponseManager::getCustomResponse()
{
	gchar *txt1, *txt2;
	ICQUser *u;
	
	u = parentUser->getLicqUser();
	
	if (isOwnerManager)
		txt1 = u->AutoResponse();
	else
		txt1 = u->CustomAutoResponse();
	

	txt2 = convertToSystemCharset(txt1, info->charset);
	
	parentUser->dropLicqUser(u);
	
	return txt2;
}



gboolean IMAutoResponseManager::finishEvent(ICQEvent* event)
{
	gchar			*txt;
	ICQUser			*u;
	
	initatedEventID = 0;
	isProcessing = FALSE;
	
	// was the event successful?
	if (event->Result() == EVENT_RESULT_ACKED || event->Result() == EVENT_RESULT_SUCCESS)
	{
		txt = NULL;
		// do we have a new auto response?
		if (event->ExtendedAck())
		{
			if (event->ExtendedAck()->Accepted())
				txt = convertToSystemCharset((gchar*)event->ExtendedAck()->Response(), info->charset);
			else
				startCallback(EVENT_REQUEST_REFUSED, event->Result(), NULL);
		} else
		// no new response -> take the old one
		{
			u = parentUser->getLicqUser();
			//FIXME: this cannot be correct but isn't working otherwise
			txt = g_strdup(u->AutoResponse());
			// txt = convertToSystemCharset((gchar*)u->AutoResponse(), info->charset);
			parentUser->dropLicqUser(u);
		}
		
		if (txt)
			startCallback(EVENT_REQUEST_ACCEPTED, event->Result(), txt);
		
	} else
		// the event wasn't successful -> finish as usual
		return IMEventManager::finishEvent(event);
		
	return TRUE;
}



// ==================== IMMessageManager =======================================

IMMessageManager::IMMessageManager(IMUserInfo	*_info): IMEventManager(_info)
{
	
	type = UE_MESSAGE;
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_MESSAGE));	// listens to message events
	listensTo = g_list_append(listensTo, GINT_TO_POINTER((gint)USER_EVENT_URL));	// listens to URLs

	colorsEnabled = FALSE;
	licqColor = new CICQColor;
	licqColor->SetToDefault();
	msgCopy = NULL;
	conversationUsers = NULL;
}



IMMessageManager::~IMMessageManager()
{
	
}



void IMMessageManager::sendMessage(gchar* msg, gushort level)
{
	gchar	*toSend;
	gchar	*tmp;
	gint	i;
	
	// does the message has to be splitted?
	if (strlen(msg) > MAX_MESSAGE_SIZE && info->sendViaServer)
	{		
		// look for "."
		for (i=MAX_MESSAGE_SIZE-1; i>0; i--)
			if (msg[i] == '.')
				break;
			
		if (!i)
		for (i=MAX_MESSAGE_SIZE-1; i>0; i--)
			if (msg[i] == ' ')
				break;
		if (!i)
			i = MAX_MESSAGE_SIZE-1;
		
		tmp = g_strdup(msg);
		tmp[i+1]=0;

		toSend = convertFromSystemCharset(tmp, info->charset);

		g_free(tmp);		
		
		if (msgCopy)
			g_free(msgCopy);
		
		msgCopy = g_strdup(msg+i+1);
		
	} 
	else
	{
		toSend = convertFromSystemCharset(msg, info->charset);
		if (msgCopy)
			g_free(msgCopy);
		msgCopy = NULL;
	}
	
	initatedEventID = getLicqDaemon()->ProtoSendMessage(info->licqID, 
														info->pluginID, 
														toSend, 
														!info->sendViaServer, 
														level,
														false,
														colorsEnabled ? licqColor: NULL,
														parentUser?parentUser->getConversationID(): 0);
	
	isProcessing = TRUE;
	
	g_free(toSend);
}


void IMMessageManager::retrySendingEvent(ICQEvent* event,  gushort level)
{
	CUserEvent	*ev;

	ev = event->UserEvent();

	if (ev->SubCommand() == USER_EVENT_MESSAGE)
	{
		if (msgCopy)
			sendMessage(msgCopy, level);
		else
			sendMessage((gchar*)ev->Text(), level);
	}
	else
		sendURL((gchar*)((CEventUrl*)ev)->Url(), (gchar*)((CEventUrl*)ev)->Description(), level);
}




void IMMessageManager::sendURL(gchar* url, gchar* msg, gushort level)
{
	gchar	*toSendMsg,
			*toSendURL;
	
	toSendURL = convertFromSystemCharset(url, info->charset);	
	toSendMsg = convertFromSystemCharset(msg, info->charset);
	
	initatedEventID = getLicqDaemon()->ProtoSendUrl(info->licqID, 
													info->pluginID, 
													toSendURL,
													toSendMsg, 
													!info->sendViaServer, 
													level,
													false,
													colorsEnabled ? licqColor: NULL);
	
	isProcessing = TRUE;
	
	g_free(toSendURL);
	g_free(toSendMsg);	
}



void IMMessageManager::sendTypingNotification(gchar nState)
{
	getLicqDaemon()->ProtoTypingNotification(info->licqID, info->pluginID, nState==2);	
}
	


void IMMessageManager::setColors(GdkColor *fg, GdkColor *bg)
{
	licqColor->SetForeground(fg->red>>8, fg->green>>8, fg->blue>>8);
	licqColor->SetBackground(bg->red>>8, bg->green>>8, bg->blue>>8);
}



void IMMessageManager::setColorsEnabled(gboolean enabled)
{
	colorsEnabled = enabled;
}



gboolean IMMessageManager::finishEvent(ICQEvent* event)
{
	messageEventInfo* gEvent;
	
	if (!msgCopy || (event->Result() != EVENT_RESULT_SUCCESS && 
					 event->Result() != EVENT_RESULT_ACKED) ||
		 event->SubResult() == ICQ_TCPxACK_RETURN)
		return IMEventManager::finishEvent(event);	
	else	
	if (msgCopy)
	{
		sendMessage(msgCopy);
		gEvent = (messageEventInfo*)mergeEvent(event->UserEvent());
		gEvent->isSplit = TRUE;
		gEvent->basic = parentUser->createBasicEvent(event->UserEvent());
		return startCallback(UC_EVENT_FINISHED, event->Result(), gEvent);
	}
	
	return FALSE;
}




gpointer IMMessageManager::mergeEvent(CUserEvent* event)
{
	messageEventInfo	*newEvent;
	CICQColor			*col;
	
	newEvent = (messageEventInfo*)g_malloc0(sizeof(messageEventInfo));
	
	newEvent->id = info->licqID;
	
	if (event->SubCommand() == USER_EVENT_MESSAGE)
	{
		newEvent->text = convertToSystemCharset((gchar*)event->Text(), info->charset);
		newEvent->url = NULL;
		
		col = event->Color();
	
		if (col)
		{
			newEvent->hasColors = TRUE;
			newEvent->fgColor.red = col->ForeRed()<<8;
			newEvent->fgColor.green = col->ForeGreen()<<8;
			newEvent->fgColor.blue = col->ForeBlue()<<8;

			newEvent->bgColor.red = col->BackRed()<<8;
			newEvent->bgColor.green = col->BackGreen()<<8;
			newEvent->bgColor.blue = col->BackBlue()<<8;		
		}
	} else
	{
		newEvent->text = convertToSystemCharset((gchar*)((CEventUrl*)event)->Description(), info->charset);
		newEvent->url = convertToSystemCharset((gchar*)((CEventUrl*)event)->Url(), info->charset);
		newEvent->hasColors = FALSE;		
	}
	
	return newEvent;
}



void IMMessageManager::conversationUserAdded(IMUserDaemon* user)
{
	if (!g_list_find(conversationUsers, user))
		conversationUsers = g_list_append(conversationUsers, user);
	
	IMEventManager::conversationUserAdded(user);
}



void IMMessageManager::conversationUserRemoved(IMUserDaemon* user)
{
	conversationUsers = g_list_remove(conversationUsers, user);
	
	IMEventManager::conversationUserRemoved(user);
}
	

GList *IMMessageManager::getConversationUsers()
{
	return conversationUsers;
}



// ================== IMContactsManager ========================================
IMContactsManager::IMContactsManager(IMUserInfo* _info):
IMEventManager(_info)
{
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_CONTACTS));
}



IMContactsManager::~IMContactsManager()
{
	
}
	


void IMContactsManager::sendContacts(GList* contacts)
{
	UserStringList	uins;
	GList	*search;
	
	for (search = contacts; search; search = search->next)
		uins.push_back(((IMUserDaemon*)search->data)->info->licqID);
	
	if (!uins.size())
		return;
	
	initatedEventID = getLicqDaemon()->icqSendContactList(info->licqID,
															uins,
															!info->sendViaServer,
															ICQ_TCPxMSG_NORMAL,
															false,
															NULL);
}



void IMContactsManager::retrySendingEvent(ICQEvent* event, gushort level)
{
	UinList uins;
	CEventContactList* ce = (CEventContactList*)event->UserEvent();
	ContactList clist = ce->Contacts();
	
	for (ContactList::const_iterator i = clist.begin(); i != clist.end(); i++)
		uins.push_back((*i)->Uin());
	
	initatedEventID = getLicqDaemon()->icqSendContactList(strtoul(info->licqID, (char**)NULL, 10),
															uins,
															!info->sendViaServer,
															level,
															false,
															NULL);	
	
	
}


	
gpointer IMContactsManager::mergeEvent(CUserEvent* event)
{
	IMUserInfoBasic	*newInfo;
	contactsEventInfo	*newEvent;
	
	CEventContactList* ce = (CEventContactList*)event;
	ContactList clist = ce->Contacts();
	
	newEvent = (contactsEventInfo*)g_malloc0(sizeof(contactsEventInfo));
	
	
	
	for (ContactList::const_iterator i = clist.begin(); i != clist.end(); i++)
	{
		newInfo = (IMUserInfoBasic*)g_malloc0(sizeof(IMUserInfoBasic));
		newInfo->alias = g_strdup((*i)->Alias());
		newInfo->licqID = g_strdup((*i)->IdString());
		newInfo->pluginID = (*i)->PPID();
		newEvent->contacts = g_list_append(newEvent->contacts, newInfo);
	}
	
	return newEvent;
}


// ================== IMAdmissionManager =======================================

IMAdmissionManager::IMAdmissionManager(IMUserInfo	*_info):
IMEventManager::IMEventManager(_info)
{
	isServer = FALSE;
	watchSource = 0;
}





IMAdmissionManager::~IMAdmissionManager()
{
	// delete the channel if neccessary
	if (watchSource != 0)
	{
		g_source_remove(watchSource);
		g_io_channel_shutdown(channel, TRUE, NULL);
	}
		
}
	




void IMAdmissionManager::sendRequest(gchar* descr, gpointer data)
{
	// dummy	
}
	



void IMAdmissionManager::answerRemoteRequest(gboolean accept, gchar* charData)
{
	// dummy
}




gpointer IMAdmissionManager::mergeEvent(CUserEvent* event)
{
	generalEventInfo	*newEvent;
	
	newEvent = (generalEventInfo*)g_malloc0(sizeof(generalEventInfo));
		
	return newEvent;
}



void IMAdmissionManager::installPipe(gint pipe, GIOFunc func)
{	
	channel = g_io_channel_unix_new(pipe);
	watchSource = g_io_add_watch(channel, G_IO_IN, (GIOFunc)func, this);
}


// =================== IMAuthorizationManager ===============================

IMAuthorizationManager::IMAuthorizationManager(IMUserInfo *_info):
IMAdmissionManager::IMAdmissionManager(_info)
{
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_AUTH_ACCEPT));
	listensTo = g_list_append(listensTo, GINT_TO_POINTER((gint)USER_EVENT_AUTH_REFUSE));
	listensTo = g_list_append(listensTo, GINT_TO_POINTER((gint)USER_EVENT_AUTH_REQUEST));
}



IMAuthorizationManager::~IMAuthorizationManager()
{
}
	


void IMAuthorizationManager::sendRequest(gchar* descr, gpointer data)
{
	getLicqDaemon()->icqRequestAuth(strtoul(info->licqID, (char**)NULL, 10), descr);
}




void IMAuthorizationManager::answerRemoteRequest(gboolean accept, gchar* charData)
{	
	if (accept)
		initatedEventID = getLicqDaemon()->ProtoAuthorizeGrant(info->licqID, info->pluginID, charData);
	else
		initatedEventID = getLicqDaemon()->ProtoAuthorizeRefuse(info->licqID, info->pluginID, charData);
	
	isProcessing = TRUE;
	
	deleteEvent(EO_FIRST | EO_NORMAL | EO_SYSTEM);
}
	


gpointer IMAuthorizationManager::mergeEvent(CUserEvent* event)
{
	messageEventInfo	*newEvent;
	
	newEvent = (messageEventInfo*)g_malloc0(sizeof(messageEventInfo));
	
	newEvent->text = convertToSystemCharset((gchar*)event->Text(), info->charset);
	
	return newEvent;	
}



// =================== IMFileTransferManager ================================

IMFileTransferManager::IMFileTransferManager(IMUserInfo	*_info):
IMAdmissionManager::IMAdmissionManager(_info)
{
	isTransferring = FALSE;
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_FILE));	// listens to file events
}




IMFileTransferManager::~IMFileTransferManager()
{
	
}
	



void IMFileTransferManager::sendRequest(gchar* descr, gpointer data, gushort level)
{
	ConstFileList licqFileList;
	GList* search;
	gchar  *fileName,
		   *tmp,
		   *tmp2,
		   *tmp3;
	gint   count;

	isServer = TRUE;

	search = (GList*)data;
	
	fileName = NULL;
	count=0;
	
	while(search)
	{
		licqFileList.push_back((char*)g_strdup((gchar*)search->data));
		
		
		// build filename to send
		if (count<3)
		{
			tmp = fileName;
			tmp2 = g_strrstr((gchar*)search->data, "/");
			if (!tmp2)
				tmp2 = (gchar*)search->data;
			
			if (count == 0)
				fileName = g_strdup(tmp2+1);
			else
				fileName = g_strconcat(tmp, ", ", tmp2+1, NULL);
			
			if (tmp)
				g_free(tmp);
			
		} else
		if (count==3)
		{
			tmp = fileName;
			tmp3 = g_strdup_printf(tr(", %d more ..."), g_list_length((GList*)data)-3);
			fileName = g_strconcat(fileName, tmp3, NULL);
			g_free(tmp);
			g_free(tmp3);
			
		}
		
		count++;
		search = search->next;
	}		
	
		
	initatedEventID = getLicqDaemon()->icqFileTransfer(strtoul(info->licqID, (char**)NULL, 10), 
													   fileName, 
													   descr, 
													   licqFileList, 
													   level, 
													   info->sendViaServer);
	
	isProcessing = TRUE;
	
	g_free(fileName);
}




void IMFileTransferManager::answerRemoteRequest(gboolean accept, gchar* charData)
{
	generalEventInfo *event;
	
	
	isServer = FALSE;
	
	event = (generalEventInfo*)getEvent(EO_FIRST | EO_NORMAL);
	
	// request was accepted -> start receiving
	if (accept)
	{
		licqManager = new CFileTransferManager(getLicqDaemon(), strtoul(info->licqID, (char**)NULL, 10));
		licqManager->SetUpdatesEnabled(2);
		installPipe(licqManager->Pipe(), (GIOFunc)this->cb_pipeCallback);
		
		// this is so weird ...
		if (licqManager->ReceiveFiles(charData))
		{
			getLicqDaemon()->icqFileTransferAccept(strtoul(info->licqID, (char**)NULL, 10),
									   					   licqManager->LocalPort(),
												  		   event->basic->licqEvent->Sequence(),
												  		   ((CEventFile*)event->basic->licqEvent)->MessageID(),
												  		   event->basic->licqEvent->IsDirect(),
														   ((CEventFile*)event->basic->licqEvent)->FileDescription(),
														   ((CEventFile*)event->basic->licqEvent)->Filename(),
														   ((CEventFile*)event->basic->licqEvent)->FileSize());
			
			isTransferring = TRUE;
		}	
	} else
	
	// request was refused -> inform remote user
	{		
		getLicqDaemon()->icqFileTransferRefuse(strtoul(info->licqID, (char**)NULL, 10), 
											   charData, 
											   event->basic->licqEvent->Sequence(), 
											   ((CEventFile*)event->basic->licqEvent)->MessageID(),
											   event->basic->licqEvent->IsDirect());
	}
	
	deleteEvent(EO_FIRST | EO_NORMAL);
}




void IMFileTransferManager::sendFiles()
{
}




gboolean IMFileTransferManager::finishEvent(ICQEvent* event)
{
	CExtendedAck  *extendedAck;
	CEventFile	  *eventFile;
	gchar		  *tmp;
	
	
	if (!isServer || !event->ExtendedAck() || 
					  event->Result()!=EVENT_RESULT_SUCCESS || 
					  event->Result()!=EVENT_RESULT_ACKED ||
					  event->SubResult() == ICQ_TCPxACK_RETURN)
		return IMEventManager::finishEvent(event);

	initatedEventID = 0;
	isProcessing = FALSE;

	
	// looks like we have an answer to our request
	extendedAck = event->ExtendedAck();
	
	// the request was accepted -> send files
	if (extendedAck->Accepted())
	{	
		licqManager = new CFileTransferManager(getLicqDaemon(), strtoul(info->licqID, (char**)NULL, 10));
		licqManager->SetUpdatesEnabled(2);
		installPipe(licqManager->Pipe(), (GIOFunc)this->cb_pipeCallback);	
		eventFile = (CEventFile*)event->UserEvent();		
		
		licqManager->SendFiles(eventFile->FileList(), extendedAck->Port());
		isTransferring = TRUE;
		
		// inform connected function that the request was accepted and is up and running
		startCallback(EVENT_REQUEST_ACCEPTED, 0, NULL);
		
	} else
	{
		// inform function that the request was refused
		tmp = convertToSystemCharset((gchar*)extendedAck->Response(), info->charset);
	
		startCallback(EVENT_REQUEST_REFUSED, 0, tmp);

		g_free(tmp);		
	}
	
	return TRUE;
	
}



void IMFileTransferManager::retrySendingEvent(ICQEvent* event, gushort level)
{
	CUserEvent	*ev;
	CEventFile	*fileEvent;
	
	ev = event->UserEvent();
	fileEvent = (CEventFile*)ev;
	ConstFileList files(fileEvent->FileList());
	
	if (ev->SubCommand() != USER_EVENT_FILE)
	{
		IMEventManager::finishEvent(event);
		return;
	}
	
	initatedEventID = getLicqDaemon()->icqFileTransfer(strtoul(info->licqID, (char**)NULL, 10), 
													   fileEvent->Filename(), 
													   fileEvent->FileDescription(), 
													   files, 
													   level, 
													   info->sendViaServer);
	isProcessing = TRUE;
}



void IMFileTransferManager::cancelInitatedEvent()
{
	IMEventManager::cancelInitatedEvent();
	
	if (!isTransferring || !licqManager)
		return;
	
	delete licqManager;
	
	licqManager = NULL;
	isTransferring = FALSE;
}




gpointer IMFileTransferManager::mergeEvent(CUserEvent* event)
{
	fileEventInfo	*newEvent;
	
	newEvent = (fileEventInfo*)g_malloc0(sizeof(fileEventInfo));
	
	newEvent->text = convertToSystemCharset((gchar*)((CEventFile*)event)->FileDescription(),info->charset);
	newEvent->fileName = g_strdup_printf(((CEventFile*)event)->Filename());
	newEvent->fileSize = ((CEventFile*)event)->FileSize();
	
	return newEvent;
}





gboolean IMFileTransferManager::cb_pipeCallback(GIOChannel* source, GIOCondition condition, IMFileTransferManager* self)
{
	gchar buffer[32];
	CFileTransferEvent* transferEvent;
	fileEventInfo* sendInfo;
		
	// if this event is not intended for this instance just return
	if (!(self->licqManager->Direction() == D_RECEIVER && !self->isServer) &&
		!(self->licqManager->Direction() != D_RECEIVER && self->isServer) )
	return TRUE;
	

	read(self->licqManager->Pipe(), buffer, 32);
	
	
	while( (transferEvent = self->licqManager->PopFileTransferEvent()) )
	{
		// update file transfer info
		sendInfo = (fileEventInfo*)g_malloc(sizeof(fileEventInfo));
		sendInfo->elapsed = time(0) - self->licqManager->StartTime();

		// FIXME: convert to UTF-8
		sendInfo->fileName		= (gchar*)self->licqManager->FileName();
		sendInfo->fileSize 		= self->licqManager->FileSize();
		sendInfo->filePosition 	= self->licqManager->FilePos();
		sendInfo->batchPosition = self->licqManager->BytesTransfered();
		sendInfo->batchCount	= self->licqManager->BatchFiles();
		sendInfo->batchSize		= self->licqManager->BatchSize();
		sendInfo->batchFile		= self->licqManager->CurrentFile();
		
		if (sendInfo->elapsed>0)
		{
			sendInfo->averageSpeed	= (float)sendInfo->batchPosition / (float)sendInfo->elapsed;
			sendInfo->remaining		= (time_t)((float)(sendInfo->batchSize - sendInfo->batchPosition) / sendInfo->averageSpeed);
		}
		else
		{
			sendInfo->averageSpeed = 0.0f;
			sendInfo->remaining = 0;
		}
		
		if (sendInfo->fileSize>0)
			sendInfo->fileProgress = (float)sendInfo->filePosition / (float)sendInfo->fileSize; 
		else
			sendInfo->fileProgress = 0.0f;
		
		if (sendInfo->batchSize>0)
			sendInfo->batchProgress = (float)sendInfo->batchPosition / (float)sendInfo->batchSize;
		else
			sendInfo->batchProgress = 0.0f;
		
		if (transferEvent->Command() == FEVENT_RES_CONFIRM)
		{
			// FIXME: convert filename to some charset (local name)
			self->licqManager->StartReceivingFile((char*)self->licqManager->FileName());
		}
		
		if (transferEvent->Command() == FEVENT_RERR_BIND ||
			transferEvent->Command() == FEVENT_RERR_CONNECT ||
			transferEvent->Command() == FEVENT_RERR_RES ||
			transferEvent->Command() == FEVENT_RERR_DISCONNECT ||
			transferEvent->Command() == FEVENT_RERR_FILE ||
			transferEvent->Command() == FEVENT_RERR_HANDSHAKE)
		{
			self->isTransferring = FALSE;
		}
		
		self->startCallback(transferEvent->Command(), 0, sendInfo);
				
		g_free(sendInfo);
		delete transferEvent;
		
		// the callback function may have deleted the manager (eg transfer was stopped)
		if (!self->licqManager)
			return FALSE;		
		
	}	

	return TRUE;	
}


// ==================== IMChatManager ==========================================

IMChatManager::IMChatManager(IMUserInfo *_info) :
IMAdmissionManager::IMAdmissionManager(_info)
{
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_CHAT));	// listens to chat events	
	licqManager = NULL;
}




IMChatManager::~IMChatManager()
{
	if (licqManager)
	{
		licqManager->CloseChat();
		delete licqManager;	
	}
	
	// the mallet-technique: because the licq daemon will send some disconnection
	// events we just cut off from the channel (otherwise these events would be
	// sent to something deleted). IMAdmissionManager::~IMAdmissionManager will do
	// this job.
}



void IMChatManager::sendRequest(gchar* descr, gpointer data, gushort mPort, gushort level)
{
	isServer = TRUE;
	
	if (!mPort)
		initatedEventID = getLicqDaemon()->icqChatRequest(strtoul(info->licqID, (char**)NULL, 10),
														  (gchar*)descr,
														  level,
														  info->sendViaServer);
	else
		initatedEventID = getLicqDaemon()->icqMultiPartyChatRequest(strtoul(info->licqID, (char**)NULL, 10),
																	(gchar*)descr,
																	"Der und der",
																	mPort,
																	level,
																	info->sendViaServer);
	isProcessing = TRUE;
}



void IMChatManager::answerRemoteRequest(gboolean accept, gchar* charData, chatFontInfo* font, gushort mPort)
{
	generalEventInfo *event;
	CEventChat		 *eventChat;
	gushort			 targetPort;
		
	isServer = FALSE;
	
	event = (generalEventInfo*)getEvent(EO_FIRST | EO_NORMAL);	
	eventChat = (CEventChat*)event->basic->licqEvent;
	
	// request was accepted
	if (accept)
	{
		if (!font)
			licqManager = new CChatManager(getLicqDaemon(), strtoul(info->licqID, (char**)NULL, 10));
		else
			licqManager = new CChatManager(getLicqDaemon(), strtoul(info->licqID, (char**)NULL, 10),
										   font->family, 
										   ENCODING_DEFAULT, 
										   STYLE_DONTCARE | STYLE_VARIABLExPITCH, 
										   font->size,
										   font->bold,
										   font->italic,
										   font->underline,
										   font->strikeout,
										   font->foreground.red>>8,
										   font->foreground.green>>8,
										   font->foreground.blue>>8,
										   font->background.red>>8,
										   font->background.green>>8,
										   font->background.blue>>8);
		
		installPipe(licqManager->Pipe(), (GIOFunc)this->cb_pipeCallback);		
		
		// this is so weird ...
		if (eventChat->Port() )
		{
			// connect to a multi party chat
			licqManager->StartAsClient(eventChat->Port());
			getLicqDaemon()->icqChatRequestAccept(strtoul(info->licqID, (char**)NULL, 10),
											    	0, 
												  	eventChat->Clients(), 
												  	eventChat->Sequence(),
												  	eventChat->MessageID(),
												  	eventChat->IsDirect());
		} else
		{
			// connect to a single party chat
			if (licqManager->StartAsServer())
			{
				if (!mPort)
					targetPort = mPort;
				else
					targetPort = licqManager->LocalPort();
				
				getLicqDaemon()->icqChatRequestAccept(strtoul(info->licqID, (char**)NULL, 10),
													  licqManager->LocalPort(), 
													  eventChat->Clients(), 
													  eventChat->Sequence(),
													  eventChat->MessageID(),
													  eventChat->IsDirect());				
			}
		}
	} else
	
	// request was refused -> inform remote user
	{		
		getLicqDaemon()->icqChatRequestRefuse(strtoul(info->licqID, (char**)NULL, 10), 
											   charData, 
											   event->basic->licqEvent->Sequence(), 
											   eventChat->MessageID(),
											   eventChat->IsDirect());
	}
	
	deleteEvent(EO_FIRST | EO_NORMAL);
	
}



	
gboolean IMChatManager::finishEvent(ICQEvent* event)
{
	CExtendedAck *extendedAck;
	CEventChat	 *c;
	gchar		 *tmp;
	
	
	if (!isServer || 
		!event->ExtendedAck() || 
		event->Result() != EVENT_RESULT_ACKED ||
		event->Result() != EVENT_RESULT_SUCCESS ||
		event->SubResult() == ICQ_TCPxACK_RETURN)
		return IMEventManager::finishEvent(event);

	initatedEventID = 0;
	isProcessing = FALSE;
	
	
	// looks like we have an answer to our request
	extendedAck = event->ExtendedAck();
	
	// the request was accepted -> start session
	if (extendedAck->Accepted())
	{	
		licqManager = new CChatManager(getLicqDaemon(), strtoul(info->licqID, (char**)NULL, 10));
		installPipe(licqManager->Pipe(), (GIOFunc)this->cb_pipeCallback);
		
		c = (CEventChat*)event->UserEvent();
		
		if (c->Port() == 0)
			licqManager->StartAsClient(extendedAck->Port());
		
		// inform connected function that the request was accepted and is up and running
		startCallback(EVENT_REQUEST_ACCEPTED, 0, NULL);
		
	} else
	{
		// inform function that the request was refused
		tmp = convertToSystemCharset((gchar*)extendedAck->Response(), info->charset);
	
		startCallback(EVENT_REQUEST_REFUSED, 0, tmp);

		g_free(tmp);		
	}
	
	return TRUE;
}



void IMChatManager::retrySendingEvent(ICQEvent* event, gushort level)
{
	CEventChat	*chatEvent;
	
	if (event->UserEvent()->SubCommand() != USER_EVENT_CHAT)
	{
		IMEventManager::finishEvent(event);
		return;
	}
	
	chatEvent = (CEventChat*)event->UserEvent();
	
	if (!chatEvent->Clients())
		initatedEventID = getLicqDaemon()->icqChatRequest(strtoul(info->licqID, (char**)NULL, 10),
														  chatEvent->Reason(),
														  level,
														  info->sendViaServer);
	else		
		initatedEventID = getLicqDaemon()->icqMultiPartyChatRequest(strtoul(info->licqID, (char**)NULL, 10),
																	chatEvent->Reason(),
																	chatEvent->Clients(),
																	chatEvent->Port(),
																	level,
																	info->sendViaServer);
	isProcessing = TRUE;
}



void IMChatManager::cancelInitatedEvent()
{
	IMEventManager::cancelInitatedEvent();
	
	if (!licqManager)
		return;
	
	delete licqManager;
	
	licqManager = NULL;
}




void IMChatManager::sendInformation(chatSendType type, gchar data)
{	
	if (!licqManager)
		return;
	
	
	switch(type)
	{
		case CST_BEEP:
			break;
		
		case CST_LAUGH:
			break;
		
		case CST_CHAR:
			licqManager->SendCharacter(data);
			break;
		
		case CST_NEWLINE:
			licqManager->SendNewline();
			break;
		
		case CST_BACKSPACE:
			licqManager->SendBackspace();
			break;
	}
}



void IMChatManager::setColor(gboolean background, gushort r, gushort g, gushort b)
{
	if (!licqManager)
		return;
	
	if (background)
		licqManager->ChangeColorBg(r, g, b);
	else
		licqManager->ChangeColorFg(r, g, b);
		
}



void IMChatManager::setFontSize(gushort size)
{
	if (!licqManager)
		return;
	licqManager->ChangeFontSize(size);
}



void IMChatManager::setFontFamily(gchar* family, guchar encoding, guchar style)
{
	if (!licqManager)
		return;
	licqManager->ChangeFontFamily(family, encoding, style);
}



void IMChatManager::setFontFace(gboolean bold, gboolean italic, gboolean underline, gboolean strikeout)
{
	if (!licqManager)
		return;
	licqManager->ChangeFontFace(bold, italic, underline, strikeout);
}



void IMChatManager::sendKickAnswer(gulong uin, gboolean kick)
{
	if (!licqManager)
		return;
	
	if (kick)
		licqManager->SendVoteYes(uin);
	else
		licqManager->SendVoteNo(uin);
}



void IMChatManager::kickUser(gulong uin)
{
	if (!licqManager)
		return;
	
	licqManager->SendKick(uin);
}


gushort IMChatManager::getChatPort()
{
	if (!licqManager)
		return 0;
	
	return licqManager->LocalPort();
}


	
gpointer IMChatManager::mergeEvent(CUserEvent* event)
{
	chatEventInfo	*newEvent;
		
	newEvent = (chatEventInfo*)g_malloc0(sizeof(chatEventInfo));
	
	newEvent->text = convertToSystemCharset("IMPLEMENT: add real text here", info->charset);
	newEvent->port = ((CEventChat*)event)->Port();
	// newEvent->description = convertToSystemCharset((gchar*)event->Text(),info->charset);
		
	return newEvent;
}


	
	
gboolean IMChatManager::cb_pipeCallback(GIOChannel* source, GIOCondition condition, IMChatManager* self)
{
	gchar buffer[32];
	CChatEvent		*chatEvent;
	CChatUser		*chatUser;
	chatEventInfo	*sendInfo;
	

	read(self->licqManager->Pipe(), buffer, 32);	
	
	while( (chatEvent = self->licqManager->PopChatEvent()) )
	{
		chatUser = chatEvent->Client();
		
		sendInfo = (chatEventInfo*)g_malloc0(sizeof(chatEventInfo));
		sendInfo->userID = chatUser->Id();
		sendInfo->userName = convertToSystemCharset((gchar*)chatUser->Name(), self->info->charset);
		sendInfo->encoding = chatUser->FontEncoding();
		sendInfo->character = g_strdup(chatEvent->Data());
		
		// do font stuff
		sendInfo->font.family = chatUser->FontFamily();
		sendInfo->font.bold = chatUser->FontBold();
		sendInfo->font.italic = chatUser->FontItalic();
		sendInfo->font.underline = chatUser->FontUnderline();
		sendInfo->font.strikeout = chatUser->FontStrikeOut();
		sendInfo->font.style = chatUser->FontStyle();
		sendInfo->font.size = chatUser->FontSize();
		
		// colors
		sendInfo->font.foreground.red = chatUser->ColorFg()[0]<<8;
		sendInfo->font.foreground.green = chatUser->ColorFg()[1]<<8;
		sendInfo->font.foreground.blue = chatUser->ColorFg()[2]<<8;

		sendInfo->font.background.red = chatUser->ColorBg()[0]<<8;
		sendInfo->font.background.green = chatUser->ColorBg()[1]<<8;
		sendInfo->font.background.blue = chatUser->ColorBg()[2]<<8;

		// special kick event
		if (chatEvent->Command() == CEVENT_KICK)
			sendInfo->toKick = chatUser->ToKick();
		
		self->startCallback(UC_CHATEVENT, chatEvent->Command(), sendInfo);
				
		g_free(sendInfo->userName);
		g_free(sendInfo);
		
		delete chatEvent;
		
		// the callback function may have deleted the manager (eg chat was stopped)
		if (!self->licqManager)
			return FALSE;
	}	
	
	return TRUE;
}


// ===================== IMGPGManager ==========================================
IMGPGManager::IMGPGManager(IMUserInfo *_info):
IMEventManager::IMEventManager(_info)
{
	
}


IMGPGManager::~IMGPGManager()
{
	
}
	

gchar* IMGPGManager::getGPGKey()
{
	gchar* toReturn;
	
	if (!parentUser)
		return NULL;
	
	ICQUser* u = parentUser->getLicqUser();
	
	toReturn = u->GPGKey();
	
	parentUser->dropLicqUser(u);
	
	return toReturn;
}


void IMGPGManager::setGPGKey(gchar* newGPGKey)
{
	if (!parentUser)
		return;
	
	ICQUser* u = parentUser->getLicqUser();
	
	u->SetGPGKey(newGPGKey);
	
	parentUser->dropLicqUser(u);	
}


gboolean IMGPGManager::getGPGActive()
{
	gboolean toReturn;
	
	if (!parentUser)
		return FALSE;
	
	ICQUser* u = parentUser->getLicqUser();
	
	toReturn = u->UseGPG();
	
	parentUser->dropLicqUser(u);
	
	return toReturn;	
}


void IMGPGManager::setGPGActive(gboolean active)
{
	if (!parentUser)
		return;
	
	ICQUser* u = parentUser->getLicqUser();
	
	u->SetUseGPG(active);
	
	parentUser->dropLicqUser(u);	
}


// ===================== IMMiscManager =========================================
IMMiscManager::IMMiscManager(IMUserInfo	*_info):
IMEventManager::IMEventManager(_info)
{
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_ADDED_TO_LIST));
}



IMMiscManager::~IMMiscManager()
{
	
}
	

gpointer IMMiscManager::mergeEvent(CUserEvent* event)
{
	generalEventInfo	*newEvent;
		
	newEvent = (generalEventInfo*)g_malloc0(sizeof(generalEventInfo));
	newEvent->text = convertToSystemCharset((gchar*)event->Text(), info->charset);
	
	return newEvent;
}



// ==================== IMSecurityManager ======================================

IMSecurityManager::IMSecurityManager(IMUserInfo *_info):
IMEventManager::IMEventManager(_info)
{
	listensTo = g_list_append(NULL, GINT_TO_POINTER((gint)USER_EVENT_SECURE_CLOSE));			
	listensTo = g_list_append(listensTo, GINT_TO_POINTER((gint)USER_EVENT_SECURE_OPEN));			
	listensTo = g_list_append(listensTo, GINT_TO_POINTER((gint)USER_EVENT_SECURE_OLD));				
}


IMSecurityManager::~IMSecurityManager()
{
	
}




guchar IMSecurityManager::secureChannelSupport()
{
	ICQUser *u;
	guchar ret;
	
	u = parentUser->getLicqUser();
	
	ret = u->SecureChannelSupport();
	
	parentUser->dropLicqUser(u);
	
	return ret;
}



void IMSecurityManager::connectSecureChannel()
{
	//FIXME: will probably change soon
	gchar	*p;
	gulong	uin;
	
	
	uin = strtoul(info->licqID, &p, 10);
	
	if (( p==0 || (p && !*p)) && info->pluginID == LICQ_PPID)
	{
		initatedEventID = getLicqDaemon()->icqOpenSecureChannel(uin);
		isProcessing = TRUE;
	}
	else
		// no secure connection could be enabled -> the result failed
		startCallback(UC_EVENT_FINISHED, EVENT_RESULT_FAILED, NULL);
}



void IMSecurityManager::disconnectSecureChannel()
{
	//FIXME: will probably change soon
	gchar	*p;
	gulong	uin;
	
	
	uin = strtoul(info->licqID, &p, 10);
	
	if (( p==0 || (p && !*p)) && info->pluginID == LICQ_PPID)
	{
		initatedEventID = getLicqDaemon()->icqCloseSecureChannel(uin);
		isProcessing = TRUE;
	}
	else
		// no secure connection could be enabled -> the result failed
		startCallback(UC_EVENT_FINISHED, EVENT_RESULT_FAILED, NULL);
}



gboolean IMSecurityManager::secureChannelActivated()
{
	ICQUser *u;
	gboolean ret;
	
	u = parentUser->getLicqUser();
	
	ret = u->Secure();
	
	parentUser->dropLicqUser(u);
	
	return ret;	
}
	
	

gboolean IMSecurityManager::hasSecureChannelOnThisClient()
{
	return getLicqDaemon()->CryptoEnabled();
}






// ==================== IMUserDaemon ===========================================

IMUserDaemon::IMUserDaemon(gchar* ID, guint _pluginID, IMUserDaemon* _owner)
{
	licqUser = NULL;	
	
	// initialize user information
	info = (IMUserInfo*)g_malloc0(sizeof(IMUserInfo));
	info->licqID   = g_strdup(ID);
	info->pluginID = _pluginID;
	owner = _owner;
	groups = NULL;
	managers = NULL;	
	editUserInformation(TRUE, USER_UPDATE_ALL);

	basicQueue = NULL;
	
	dowry = NULL;
	
	callBack = NULL;
	callBackData = NULL;
	
	conversationID = 0;
}


IMUserDaemon::IMUserDaemon(ICQUser* _licqUser, IMUserDaemon* _owner)
{
	licqUser = _licqUser;
	info = (IMUserInfo*)g_malloc0(sizeof(IMUserInfo));
	info->licqID   = g_strdup(licqUser->IdString());
	info->pluginID = licqUser->PPID();
	owner = _owner;
	groups = NULL;
	managers = NULL;	
	editUserInformation(TRUE, USER_UPDATE_ALL);

	basicQueue = NULL;
	
	dowry = NULL;
	
	callBack = NULL;
	callBackData = NULL;
	
	groups = NULL;
	
	conversationID = 0;
}



IMUserDaemon::~IMUserDaemon()
{
	clearAllEvents(FALSE);
	
	editUserInformation(FALSE, USER_UPDATE_ALL);
	g_free(info);	
	info = NULL;
}



guint IMUserDaemon::getQueueCount()
{
	return g_list_length(basicQueue);
}


ICQUser* IMUserDaemon::getLicqUser()
{
	if (licqUser)
		return licqUser;
	else
		return gUserManager.FetchUser(info->licqID, info->pluginID, LOCK_R);
}



void IMUserDaemon::dropLicqUser(ICQUser* todrop)
{
	if (!licqUser)
		gUserManager.DropUser(todrop);
}




void IMUserDaemon::setStatus(guint status)
{
	ICQUser *u;
	
	u = getLicqUser();

	u->SetStatusToUser(status);
	
	dropLicqUser(u);
}



void IMUserDaemon::setCharset(gchar* charset, gboolean saveInfo)
{
	ICQUser	*u;
	
	u = getLicqUser();
	u->SetUserEncoding(charset);
	if (saveInfo)
	{
		u->SetEnableSave(true);
		u->SaveLicqInfo();
	}
	dropLicqUser(u);
	
	g_free(info->charset);
	info->charset = g_strdup(charset);
}




gboolean IMUserDaemon::getAcceptModes(acceptMode mode)
{
	ICQUser	*u;
	gboolean toReturn;
	
	u = getLicqUser();
	
	switch(mode)
	{
		case AM_ACCEPTINAWAY: toReturn = u->AcceptInAway(); break;
		case AM_ACCEPTINNA: toReturn = u->AcceptInNA(); break;
		case AM_ACCEPTINOCC: toReturn = u->AcceptInOccupied(); break;
		case AM_ACCEPTINDND: toReturn = u->AcceptInDND(); break;
		case AM_AUTOACCEPTFILES: toReturn = u->AutoFileAccept(); break;
		case AM_AUTOACCEPTCHATS: toReturn = u->AutoChatAccept(); break;
		case AM_AUTOSECURE: toReturn = u->AutoSecure(); break;
		case AM_SENDREALIP: toReturn = u->SendRealIp(); break;
		case AM_USEGPG: toReturn = FALSE; break;
	}
	
	dropLicqUser(u);
	
	return toReturn;
}


		
void IMUserDaemon::setAcceptMode(acceptMode mode, gboolean setting)
{
	ICQUser	*u;
	
	u = getLicqUser();

	switch(mode)
	{
		case AM_ACCEPTINAWAY: u->SetAcceptInAway(setting); break;
		case AM_ACCEPTINNA: u->SetAcceptInNA(setting); break;
		case AM_ACCEPTINOCC: u->SetAcceptInOccupied(setting); break;
		case AM_ACCEPTINDND: u->SetAcceptInDND(setting); break;
		case AM_AUTOACCEPTFILES: u->SetAutoFileAccept(setting); break;
		case AM_AUTOACCEPTCHATS: u->SetAutoChatAccept(setting); break;
		case AM_AUTOSECURE: u->SetAutoSecure(setting); break;
		case AM_SENDREALIP: u->SetSendRealIp(setting); break;	
		case AM_USEGPG: break;			
	}
	
	dropLicqUser(u);	
}



		
gulong IMUserDaemon::getLicqGroups(GroupType g)
{
	gulong toReturn;
	ICQUser *u;
	
	u = getLicqUser();
	toReturn = u->GetGroups(g);
	dropLicqUser(u);
	
	return toReturn;	
}
	


void IMUserDaemon::setLicqGroups(GroupType g, gulong newGroups)
{
	ICQUser		*u;
	gulong		oldGroupInfo;
	gboolean	isIgnored;
	
	u = getLicqUser();
	oldGroupInfo = u->GetGroups(g);
	u->SetGroups(g, newGroups);
	dropLicqUser(u);
		
	editUserInformation(TRUE, USER_UPDATE_BASIC);
	if (oldGroupInfo != newGroups)
	{
		isIgnored = newGroups & (1<<(GROUP_IGNORE_LIST-1));
		
		if (owner)
			owner->startCallback(UC_UPDATEMODES, (gint) isIgnored, this);
	}
	
	// The system groups have changed
	if (g == GROUPS_SYSTEM)
	{
		if (oldGroupInfo != newGroups)
		{
			isIgnored = newGroups & (1<<(GROUP_IGNORE_LIST-1));
		
			if (owner)
				owner->startCallback(UC_UPDATEMODES, (gint) isIgnored, this);
		
			// we are ignored now? Request to delete us
			if (isIgnored && owner)
			{
				((IMOwnerDaemon*)owner)->users = g_list_remove(((IMOwnerDaemon*)owner)->users, this);				
				owner->startCallback(OC_REMOVEUSER, 0, this);
			} else
			
			// we are not ignored anymore but were before?
			if (owner && !isIgnored && (oldGroupInfo & (1<<(GROUP_IGNORE_LIST-1))))
			{
				((IMOwnerDaemon*)owner)->ignoredUsers = g_list_remove(((IMOwnerDaemon*)owner)->ignoredUsers, this);
				((IMOwnerDaemon*)owner)->users = g_list_append(((IMOwnerDaemon*)owner)->users, this);
				owner->startCallback(OC_NEWUSER, 0, this);
			}
		}
	}
}




IMEventManager* IMUserDaemon::addManager(IMEventManager *newManager)
{	
	if (newManager->canDropEvents)
		managers = g_list_append(managers, newManager);	
	else
		// add the manager to the start so it will be called before managers
		// that are able to drop events
		managers = g_list_prepend(managers, newManager);
	
	newManager->parentUser = this;
		
	return newManager;
}



void IMUserDaemon::initializeManagerForUser(IMEventManager* man)
{
	GList			 *search,
					 *lsearch,
					 *ulist;
	gpointer		 newEvent;
	basicEventInfo	 *basicEvent;
	guint			 pendingCount;
	
	
	man->setSendViaServer(info->sendViaServer);
	
	ulist = NULL;
	pendingCount = 0;
	
	search = basicQueue;
	
	while(search)
	{
		basicEvent = (basicEventInfo*)search->data;
		
		lsearch = man->listensTo;
		while(lsearch)
		{
			if (basicEvent->type == GPOINTER_TO_INT(lsearch->data))
			{ 
				basicEvent->refCount++;
				ulist = g_list_append(ulist, basicEvent);
				pendingCount++;
				break;				
			}
			
			lsearch = lsearch->next;
		}
		
		search = search->next;		
	}
	
	
	// tell the connected window that the manager is installed and how many pending events we have
	man->startCallback(EC_INSTALLED, pendingCount, NULL);
	
	
	
	// now send all events to the manager
	search = ulist;
	while(search)
	{
		newEvent = man->addEvent((basicEventInfo*)search->data);
		man->startCallback(UC_EVENT_RECEIVED, 0, newEvent);
	
		search = search->next;
	}
	
	g_list_free(ulist);
}




void IMUserDaemon::removeManager(IMEventManager *toRemove)
{
	managers = g_list_remove(managers, toRemove);
	// inform general manager
	startCallback(EC_UNINSTALLED, 0, toRemove);
}



void IMUserDaemon::removeManagerByCallback(gpointer removeCallback)
{
	GList* search;
	IMEventManager* man;
	
	for (search = managers; search; search = search->next)
	{
		man = (IMEventManager*)search->data;
		if (man->containsCallback(removeCallback))
		{
			delete man;
			break;
		}
	}
}



void IMUserDaemon::informAllManagers(gint command, gint lastResult, gpointer data)
{
	GList* search;
	
	search = managers;
	while(search)
	{
		((IMEventManager*)search->data)->startCallback(command, lastResult, data);
		search = search->next;
	}
}



void IMUserDaemon::dispatchPendingEvents()
{
	ICQUser* 			u;		
	CUserEvent* 		event;
	gint				i;


	
	u = getLicqUser();
	if (!u) return;
	
	
	// check all events that we have received	
	for (i=0;i<u->NewMessages();i++)
	{		
		event = u->EventPeek(i);
		
		addEventToQueue(event, FALSE);
	}		
	
	dropLicqUser(u);		
}




void IMUserDaemon::addEventToQueue(CUserEvent* event, gboolean isOwnerEvent)
{	
	GList			 *search,
					 *lsearch;
	gpointer		 newEvent;
	IMEventManager	 *toCall;
	basicEventInfo *newEvInfo;
	
	// look whether this event is already in our event types list
	search = basicQueue;
	while(search)
	{
		if (((basicEventInfo*)search->data)->eventID == event->Id())
			break;
		search = search->next;
	}
	
	
	// the event isn't in the list -> add it and call the appropriate manager
	if (!search)
	{	
		// set the new event time
		info->stats.lastReceived = time(NULL);
		
		// create the new general part
		newEvInfo = createBasicEvent(event);
		newEvInfo->isOwnerEvent = isOwnerEvent; 
		basicQueue = g_list_append(basicQueue, newEvInfo);
		
		
		// inform the owner that there is a new event
		if (owner)
			owner->startCallback(OC_EVENT_ADDED, isOwnerEvent, newEvInfo);
		else
			startCallback(OC_EVENT_ADDED, isOwnerEvent, newEvInfo);

		// and also tell the general manager
		startCallback(UC_EVENT_RECEIVED, event->SubCommand(), newEvInfo);
		
		newEvent = NULL;
		search = managers;

		while(search)
		{	
			toCall = (IMEventManager*)search->data;
			
			lsearch = toCall->listensTo;

			while(lsearch)
			{
				if (GPOINTER_TO_INT(lsearch->data) == newEvInfo->type)
				{
					newEvInfo->refCount++;
					newEvent = toCall->addEvent(newEvInfo);			
					toCall->startCallback(UC_EVENT_RECEIVED, 0, newEvent);
				
					break;
				}
			
				lsearch = lsearch->next;
			}			
		
			search = search->next;
		}	
	}
		
}


void IMUserDaemon::addEventTag(gulong tag)
{
	GList			*search;
	IMEventManager	*man;
	// add a tag to the last event.
	// FIXME: this is not secure and could cause confusion :)

	search = managers;
	while(search)
	{
		man = (IMEventManager*)search->data;
		
		// is there an event that is processing but doesn't have a tag id?
		if (man->isProcessing && !man->initatedEventID)
		{
			man->initatedEventID = tag;
			break;
		}
		
		search = search->next;
	}
	
}


void IMUserDaemon::setConversationID(gulong newID)
{
	conversationID = newID;
}



gulong IMUserDaemon::getConversationID()
{
	return conversationID;
}



gulong IMUserDaemon::findConversationID()
{
	CConversation	*conv;
	ICQUser			*u;
	
	conv = NULL;
	
	u = getLicqUser();
	
	if (u->SocketDesc(ICQ_CHNxNONE) != 1)
	{
		conv = getLicqDaemon()->FindConversation(u->SocketDesc(ICQ_CHNxNONE));
		if (conv)
			conversationID = conv->CID();
      }
	  
	dropLicqUser(u);
	
	return conversationID;	
}



void IMUserDaemon::addConversationUser(IMUserDaemon* cuser, gulong convID)
{
	GList			*search;
	
	search = managers;
	while(search)
	{
		((IMEventManager*)search->data)->conversationUserAdded(cuser);
		search = search->next;
	}
}
		


void IMUserDaemon::removeConversationUser(IMUserDaemon* cuser, gulong convID)
{
	GList			*search;
	
	search = managers;
	while(search)
	{
		((IMEventManager*)search->data)->conversationUserRemoved(cuser);
		search = search->next;
	}	
}



basicEventInfo* IMUserDaemon::createBasicEvent(CUserEvent* event)
{
	basicEventInfo	*newEvInfo;
	
	newEvInfo = (basicEventInfo*)g_malloc0(sizeof(basicEventInfo));
	newEvInfo->licqEvent = event;
	newEvInfo->userID = info->licqID;	
	newEvInfo->refCount = 0;

	if (event)
	{		
		newEvInfo->eventID = event->Id();
		newEvInfo->type = event->SubCommand();
		newEvInfo->time = event->Time();
	}
	
	return newEvInfo;
}





void IMUserDaemon::dispatchInformation(CICQSignal* signal)
{
	GList	*search;
	gint	lastResult;
	basicEventInfo	*basicEvent;
		
	if (signal)
	{
		// setup intern user information
		switch(signal->SubSignal())
		{
			// an event was received or deleted
			case USER_UPDATE_EVENTS:
				// an event was created				
				if (signal->Argument()>0)
					dispatchPendingEvents();
				else
				// the user checked your auto response	
				if (signal->Argument()==0)
					startCallback(UC_AUTORESPONSE_CHECKED, 0, NULL);
				// an event was deleted
				else
				{
					// find the info for that event
					for (search = basicQueue; search; search = search->next)
					{
						basicEvent = (basicEventInfo*)search->data;
						if (basicEvent->eventID == -signal->Argument())
							break;						
					}
					
					if (search)
					{
						basicQueue = g_list_remove(basicQueue, basicEvent);
						
						if (owner)
							owner->startCallback(OC_EVENT_DELETED, basicEvent->isOwnerEvent, basicEvent);
						else
							startCallback(OC_EVENT_DELETED, basicEvent->isOwnerEvent, basicEvent);
						
						startCallback(UC_EVENT_FINISHED, 0, NULL);
						
						// call all managers that they should erase this event
						for (search = managers; search; search = search->next)
							((IMEventManager*)search->data)->eraseEventByBasicInfo(basicEvent);
						
						g_free(basicEvent);
					}	
				}
					
				break;
		
			case USER_UPDATE_SECURITY:
				// security status has changed
				break;
		
			default:
				editUserInformation(TRUE, signal->SubSignal(), signal->Argument());
				break;
		}
		
		lastResult = signal->SubSignal();
	} else
	{
		lastResult = USER_UPDATE_ALL;
		
		editUserInformation(TRUE, USER_UPDATE_ALL);
	}
	
	// tell all managers that something has happend
	search = managers;
	while(search)
	{
		// update send-server information for all managers
		((IMEventManager*)search->data)->setSendViaServer(info->sendViaServer);
		
		// start callback
		((IMEventManager*)search->data)->startCallback(UC_UPDATEUSER, lastResult, NULL);
		
		search = search->next;
	}

	
	// also tell the general manager
	startCallback(UC_UPDATEUSER, lastResult, this);
}





void IMUserDaemon::finishEvent(ICQEvent* event)
{
	GList			*search1,
					*managers2;
	IMEventManager	*toCall;
	
	if (!managers)
		return;
		
	// check which event is going to be finished and call it
	managers2 = g_list_copy(managers);
	search1 = managers2;
	
	
	// the event matches when it's eventID matches a started event
	while(search1)
	{
		toCall = (IMEventManager*)search1->data;
		
		if (toCall->initatedEventID != 0 && event->Equals(toCall->initatedEventID))
		{
			toCall->finishEvent(event);
			break;
		}
		
		search1 = search1->next;
	}
		
	g_list_free(managers2);
}



void IMUserDaemon::clearEvent(basicEventInfo *toClear, gboolean canBeDropped)
{
	ICQUser* u;
	ICQOwner* o;
	
	
	if (toClear->refCount > 0)
		toClear->refCount--;
	
	if (!canBeDropped || toClear->refCount>0)
		return;		
	
	if (toClear->isOwnerEvent)
	{
		o = gUserManager.FetchOwner(info->pluginID, LOCK_R);
		o->EventClearId(toClear->eventID);
		gUserManager.DropOwner(info->pluginID);		
	} else
	{
		u = getLicqUser();
		u->EventClearId(toClear->eventID);
		dropLicqUser(u);
	}
	
	

}


void IMUserDaemon::clearAllEvents(gboolean deleteEvents)
{
	GList			*search,
					*basicsCopy;
	
	/*
	// delete all manager events
	if (deleteEvents)
	for (search = managers; search; search = search->next)
	{
		man = (IMEventManager*)search->data;
		man->deleteEvent(EO_ALL | EO_NORMAL | EO_SYSTEM | EO_SUNKEN);			
	}
		*/
	// delete all events that don't have a manager yet
	basicsCopy = g_list_copy(basicQueue);
	for (search = basicsCopy; search; search = search->next)
	{
		if (deleteEvents)
			clearEvent((basicEventInfo*)search->data, TRUE);
//		else
			// FIXME: free event info		
	}	
	g_list_free(basicsCopy);
}



basicEventInfo* IMUserDaemon::getEvent(gchar returnFlag)
{
	GList	*search;
	basicEventInfo *toReturn;
		
	if (!basicQueue)
		return NULL;	
	
	if (returnFlag & EO_LAST)
		search = g_list_last(basicQueue);
	else
		search = basicQueue;
		
	while(search)
	{
		toReturn = (basicEventInfo*)search->data;
		
		if ((toReturn->isOwnerEvent && (returnFlag & EO_SYSTEM)) ||
			(!toReturn->isOwnerEvent && (returnFlag & EO_NORMAL)))
			break;		
		
		if (returnFlag & EO_LAST)
			search = search->prev;
		else
			search = search->next;
	}
		
	// something was found
	if (search)
		return toReturn;
	else
		return NULL;
	
}

#define IMU_FREESTRING(str)	\
		if (str)			\
		{					\
			g_free(str);	\
			str = NULL;		\
		}


void IMUserDaemon::editUserInformation(gboolean update, gushort toUpdate, gint statusChangeArgument)
{	
	GString		*str;
	char  		buf[32];
	ICQUser		*u;
	gboolean	oldViaServer;
	
	
	if (!info)
		return;
	
	// ** update information **
	if (update)
	{
		// first free the info that should be updated
		editUserInformation(FALSE, toUpdate, statusChangeArgument);
		
		u = getLicqUser();
		
		// get character set information
		if (u->UserEncoding() && *u->UserEncoding())
			info->charset = g_strdup(u->UserEncoding());
		else
			u->SupportsUTF8()? info->charset = g_strdup("UTF-8"): info->charset = g_strdup("");
				
		
		// update the recent fields
		switch(toUpdate)
		{
			case USER_UPDATE_ALL:
				
			case USER_UPDATE_GENERAL:
			case USER_UPDATE_BASIC:
			case USER_UPDATE_EXT:
				info->alias = convertToSystemCharset(u->GetAlias(), info->charset);
				info->keepAlias = u->KeepAliasOnUpdate();	
				info->firstname = convertToSystemCharset(u->GetFirstName(), info->charset);
				info->lastname = convertToSystemCharset(u->GetLastName(), info->charset);
				info->email1 = convertToSystemCharset(u->GetEmailPrimary(), info->charset);
				info->email2 = convertToSystemCharset(u->GetEmailSecondary(), info->charset);
				info->email3 = convertToSystemCharset(u->GetEmailOld(), info->charset);

				info->home.city = convertToSystemCharset(u->GetCity(), info->charset);
	  			info->home.state = convertToSystemCharset(u->GetState(), info->charset);
	  			info->home.address = convertToSystemCharset(u->GetAddress(), info->charset);
	  			info->home.zip = convertToSystemCharset(u->GetZipCode(), info->charset);
	  			info->home.mobile = convertToSystemCharset(u->GetCellularNumber(), info->charset);
	  			info->home.phone = convertToSystemCharset(u->GetPhoneNumber(), info->charset);
	  			info->home.fax = convertToSystemCharset(u->GetFaxNumber(), info->charset);
				GetCountryByCode(u->GetCountryCode())? info->home.country = GetCountryByCode(u->GetCountryCode())->nIndex: info->home.country = 0;
				info->formattedID = convertToSystemCharset(u->IdString(), info->charset);
	  			info->specialAR = convertToSystemCharset(u->CustomAutoResponse(), info->charset);
				
				info->localtime = u->LocalTimeOffset();
				info->gmtoffset = u->LocalTimeGMTOffset();
				info->isOnList = (licqUser == NULL);
				info->onVisibleList = u->VisibleList();
				info->onIgnoreList = u->IgnoreList();
				info->statusToUser 	= u->StatusToUser();
				
				// update group information
				editUserGroups(TRUE, u);
				
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
			
			case USER_UPDATE_HP:
			case USER_UPDATE_MORE:
				info->website = convertToSystemCharset(u->GetHomepage(), info->charset);
				info->age = (gushort)u->GetAge();
				info->gender = (gchar)u->GetGender(); 
				info->birthday.day 	= (gint)u->GetBirthDay();
				info->birthday.month = (gint)u->GetBirthMonth();
				info->birthday.year = (gint)u->GetBirthYear();
				info->birthday.hasBirthday = u->Birthday() == 0;
				if (u->GetLanguage1() >= 0) 
					info->language1 = GetLanguageByCode(u->GetLanguage1())->nIndex;
				else
					info->language1 = LANGUAGE_UNSPECIFIED;
				
				if (u->GetLanguage2() >= 0) 
					info->language2 = GetLanguageByCode(u->GetLanguage2())->nIndex;
				else
					info->language2 = LANGUAGE_UNSPECIFIED;
				
				if (u->GetLanguage3() >= 0) 
					info->language3 = GetLanguageByCode(u->GetLanguage3())->nIndex;
				else
					info->language3 = LANGUAGE_UNSPECIFIED;
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_MORE2:
				editUserInfoCategories(u->GetInterests(), GetInterestByCode, &info->interests, TRUE);
				editUserInfoCategories(u->GetBackgrounds(), GetBackgroundByCode, &info->background, TRUE);
				editUserInfoCategories(u->GetOrganizations(), GetOrganizationByCode, &info->organizations, TRUE);	
				
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_ABOUT:
				info->about = convertToSystemCharset(u->GetAbout(), info->charset);
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_WORK:
				info->company.city = convertToSystemCharset(u->GetCompanyCity(), info->charset);
				info->company.state = convertToSystemCharset(u->GetCompanyState(), info->charset);
				info->company.address = convertToSystemCharset(u->GetCompanyAddress(), info->charset);
				info->company.zip = convertToSystemCharset(u->GetCompanyZip(), info->charset);
				info->company.phone = convertToSystemCharset(u->GetCompanyPhoneNumber(), info->charset);
				info->company.fax = convertToSystemCharset(u->GetCompanyFaxNumber(), info->charset);
				info->company.department = convertToSystemCharset(u->GetCompanyDepartment(), info->charset);
				info->company.position = convertToSystemCharset(u->GetCompanyPosition(), info->charset);
				info->company.name = convertToSystemCharset(u->GetCompanyName(), info->charset);
				info->company.website = convertToSystemCharset(u->GetCompanyHomepage(), info->charset);
				GetCountryByCode(u->GetCompanyCountry())? info->company.country = GetCountryByCode(u->GetCompanyCountry())->nIndex: info->company.country = 0;
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_PHONES:
				// ...
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_PICTURE:
				// ...
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_TYPING:
				info->typing = u->GetTyping();
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_STATUS:
				if (info->status == BUDDY_STATUS_OFFLINE && u->Status()  != BUDDY_STATUS_OFFLINE)
					info->wentJustOnline = TRUE;
				info->status = u->Status();
				info->isInvisible = u->StatusInvisible();
				if (!info->sendViaServerByUser)
				{
					oldViaServer = info->sendViaServer;
					info->sendViaServer = u->SendServer() || info->status == BUDDY_STATUS_OFFLINE || (u->Port() == 0 && u->SocketDesc(ICQ_CHNxNONE) == -1);
			
					if (oldViaServer != info->sendViaServer)
						informAllManagers(UC_SENDSERVER, (gint)info->sendViaServer, NULL);
				}
				// special fields
				str = g_string_new(u->IpStr(buf));
				if (u->Ip() != u->IntIp() && u->IntIp())
					g_string_append_printf(str, " / %s", u->IntIpStr(buf));
				if (u->Port() != 0)
					g_string_append_printf(str, " : %s", u->PortStr(buf));
				info->formattedIP = g_string_free(str, FALSE);

				if ( info -> status != BUDDY_STATUS_OFFLINE && u->ClientInfo() && *u->ClientInfo() ) {
					str = g_string_new( u->ClientInfo() );
					info->clientName = g_string_free(str, FALSE);
				}
				if ( !info->clientName ) {
					str = g_string_new( "*unknown*" );
					info->clientName = g_string_free(str, FALSE);
				}
				
				info->stats.lastOnline = u->LastOnline();
				info->stats.lastSent = u->LastSentEvent();
				info->stats.lastReceived = u->LastReceivedEvent();
				info->stats.lastAR = u->LastCheckedAutoResponse();
				info->stats.onlineSince = u->OnlineSince();
				info->stats.idleSince = u->IdleSince();

				
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_SECURITY:
				// ...
				break;
		}
		
		dropLicqUser(u);
	} else
	
	// ** free information **
	{

		// FIXME: not that good but neccessary ...
		info->wentJustOnline = FALSE;
		
		IMU_FREESTRING(info->charset);		
		
		// update the recent fields
		switch(toUpdate)
		{
			case USER_UPDATE_ALL:
				
			case USER_UPDATE_GENERAL:
			case USER_UPDATE_BASIC:
			case USER_UPDATE_EXT:
				IMU_FREESTRING(info->alias);
				IMU_FREESTRING(info->firstname);
				IMU_FREESTRING(info->lastname);
				IMU_FREESTRING(info->email1);
				IMU_FREESTRING(info->email2);
				IMU_FREESTRING(info->email3);
			
				IMU_FREESTRING(info->home.city);
				IMU_FREESTRING(info->home.state);
				IMU_FREESTRING(info->home.address);
				IMU_FREESTRING(info->home.zip);
				IMU_FREESTRING(info->home.mobile);
				IMU_FREESTRING(info->home.phone);
				IMU_FREESTRING(info->home.fax);
				IMU_FREESTRING(info->formattedID);
				IMU_FREESTRING(info->specialAR);
				editUserGroups(FALSE, NULL);
				
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
			
			case USER_UPDATE_HP:
			case USER_UPDATE_MORE:
				IMU_FREESTRING(info->website);
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_MORE2:
				editUserInfoCategories(NULL, GetInterestByCode, &info->interests, FALSE);
				editUserInfoCategories(NULL, GetBackgroundByCode, &info->background, FALSE);
				editUserInfoCategories(NULL, GetOrganizationByCode, &info->organizations, FALSE);	
				
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_ABOUT:
				IMU_FREESTRING(info->about);
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_WORK:
				IMU_FREESTRING(info->company.city);
				IMU_FREESTRING(info->company.state);
				IMU_FREESTRING(info->company.address);
				IMU_FREESTRING(info->company.zip);
				IMU_FREESTRING(info->company.phone);
				IMU_FREESTRING(info->company.fax);
				IMU_FREESTRING(info->company.department);
				IMU_FREESTRING(info->company.position);
				IMU_FREESTRING(info->company.name);
				IMU_FREESTRING(info->company.website);
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_PHONES:
				// ...
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
				
			case USER_UPDATE_PICTURE:
				// ...
				if (toUpdate != USER_UPDATE_ALL)
					break;
												
				
			case USER_UPDATE_SECURITY:
				// ...
				if (toUpdate != USER_UPDATE_ALL)
					break;
				
			case USER_UPDATE_STATUS:
				IMU_FREESTRING(info->formattedIP);
				break;
		}
		
	}
	
/*	
	typedef struct infoField
	{
		gchar	**toSet;
		gchar	 *value;
	};

	// if data should be updated free old data first
	if (update)
		editUserInformation(FALSE, toUpdate);
	
	
	ICQUser* u = getLicqUser();
	int i, i1, group;
	gchar *tmp;
	char  buf[32];
	GList* search;
	gushort defaultGroup,
			newUserGroup;
	groupInfo	*currentGroup;
	
	
	// general char* infos
	infoField general[15] =
	{ {&info->firstname, u ? u->GetFirstName():NULL},
	  {&info->lastname, u ? u->GetLastName():NULL},
	  {&info->alias, u ? u->GetAlias():NULL},
	  {&info->email1, u ? u->GetEmailPrimary():NULL},
	  {&info->email2, u ? u->GetEmailSecondary():NULL},
	  {&info->email3, u ? u->GetEmailOld():NULL},
	  {&info->home.city, u ? u->GetCity():NULL},
	  {&info->home.state, u ? u->GetState():NULL},
	  {&info->home.address, u ? u->GetAddress():NULL},
	  {&info->home.zip, u ? u->GetZipCode():NULL},
	  {&info->home.mobile, u ? u->GetCellularNumber():NULL},
	  {&info->home.phone, u ? u->GetPhoneNumber():NULL},
	  {&info->home.fax, u ? u->GetFaxNumber():NULL},
	  {&info->formattedID, u ? u->IdString():NULL},
	  {&info->specialAR, u ? u->CustomAutoResponse():NULL},
    };
		
	// work char* infos
	infoField work[10] =
	{
		{&info->company.city, u ? u->GetCompanyCity():NULL},
		{&info->company.state, u ? u->GetCompanyState():NULL},
		{&info->company.address, u ? u->GetCompanyAddress():NULL},
		{&info->company.zip,u ? u->GetCompanyZip():NULL },
		{&info->company.phone, u ? u->GetCompanyPhoneNumber():NULL},
		{&info->company.fax, u ? u->GetCompanyFaxNumber():NULL},
		{&info->company.department, u ? u->GetCompanyDepartment():NULL},
		{&info->company.position, u ? u->GetCompanyPosition():NULL},
		{&info->company.name, u ? u->GetCompanyName():NULL},
		{&info->company.website, u ? u->GetCompanyHomepage():NULL}
	};
	
	// more char* infos
	infoField more[1]=
	{
		{&info->website, u ? u->GetHomepage():NULL}
	};
	
	printf("u1\n");
	
	if (update)
	{	
		info->isOnList  = (licqUser == NULL);
		info->onVisibleList = u->VisibleList();
		info->wentJustOnline = FALSE;
		info->statusToUser 	= u->StatusToUser();
		
		if (!u->UserEncoding() || !*u->UserEncoding())
		{		
			// does the user support UTF-8?
			if (u->SupportsUTF8())
				info->charset = g_strdup("UTF-8");
			else
			// if he has an owner take his standard charset
				info->charset 	= g_strdup(""); 	
		}
		else
			info->charset 	= g_strdup(u->UserEncoding()); 

		
		info->statusToUser 	= u->StatusToUser();
		if (!info->sendViaServerByUser)
			info->sendViaServer = u->SendServer() || (info->status == BUDDY_STATUS_OFFLINE && u->SocketDesc(ICQ_CHNxNONE) == -1);
		
		info->stats.lastOnline = u->LastOnline();
		info->stats.lastSent = u->LastSentEvent();
		info->stats.lastReceived = u->LastReceivedEvent();
		info->stats.lastAR = u->LastCheckedAutoResponse();
		info->stats.onlineSince = u->OnlineSince();
		info->stats.idleSince = u->IdleSince();
		
		// update group information
		if (owner)
		{
			i1=0;	
			// one user can be in different groups at once
			// example: group 0 = new users, 1 = friends, 2 = general
			// 			group ID = 7 -> binary 111 -> user is in all three groups
			// ... this is buggy crap in my eyes ..
			group = u->GetGroups(GROUPS_USER);
			defaultGroup = gUserManager.DefaultGroup();
			newUserGroup = gUserManager.NewUserGroup();			

			search = IO_getGroupManager()->groups;
			while(search)
			{
				currentGroup = (groupInfo*)search->data;

				if (currentGroup->id == 0xFFFF)
				{
					search = search->next;
					continue;
				}						
				if ( (1 << i1) & group)
					groups = g_list_append(groups, currentGroup);	
		
				i1++;
				search = search->next;
			}
		} 
				
		if (!g_list_length(groups))
			groups = g_list_append(groups, IO_getGroupManager()->getNoGroup());

	} else
	{
		if (info->charset)
		{
			g_free(info->charset);
			info->charset = NULL;
		}
		
		if (owner && groups)
		{
			g_list_free(groups);
			groups = NULL;
		}
	}
	
	if (toUpdate == USER_UPDATE_BASIC ||
		toUpdate == USER_UPDATE_GENERAL ||
		toUpdate == USER_UPDATE_EXT ||
		toUpdate == USER_UPDATE_ALL)
	{
		for (i=0;i<15;i++)
		{

			if (update)
				*general[i].toSet = convertToSystemCharset(general[i].value, info->charset);
			else
				if (*general[i].toSet) 
				{
					g_free(*general[i].toSet);
					*general[i].toSet = NULL;
				}
		}

		printf("u2.5\n");		
		
		// time
		// security
		if (update)
		{	
			info->formattedIP = g_strdup(u->IpStr(buf));
			
			if (u->Ip() != u->IntIp() && u->IntIp())
			{
				tmp = g_strdup_printf("%s / %s", info->formattedIP, u->IntIpStr(buf));
				g_free(info->formattedIP);
				info->formattedIP = tmp;
			}		
			
			if (u->Port()!=0)
			{
				tmp = g_strdup_printf("%s : %s", info->formattedIP, u->PortStr(buf));
				g_free(info->formattedIP);
				info->formattedIP = tmp;
			} 		
				
			
			info->keepAlias = u->KeepAliasOnUpdate();			
			
			info->localtime = u->LocalTimeOffset();
			info->gmtoffset = u->LocalTimeGMTOffset();
			
			// the alias must be protected against pango stuff		
			info->formattedAlias = g_markup_escape_text(info->alias, strlen(info->alias));
			
			// FIXME: this is somehow dirty
			ICQOwner* myOwner = gUserManager.FetchOwner(info->pluginID, LOCK_R);
			info->ownerAlias = convertToSystemCharset(myOwner->GetAlias(), myOwner->UserEncoding());
			gUserManager.DropOwner(info->pluginID);
			if (GetCountryByCode(u->GetCountryCode()))
				info->home.country = GetCountryByCode(u->GetCountryCode())->nIndex;
			else
				info->home.country = 0;
		} else
		{
			if (info->formattedAlias)
			{
				printf("bla1\n");
				g_free(info->formattedAlias);
				info->formattedAlias = NULL;
			}
			if (info->formattedIP)
			{
				printf("bla2\n");
				g_free(info->formattedIP);
				info->formattedIP = NULL;
			}
			if (info->ownerAlias)
			{
				printf("bla3\n");
				g_free(info->ownerAlias);
				info->ownerAlias = NULL;
			}
		}
	
	}
	printf("u3\n");	
	
	if (toUpdate == USER_UPDATE_MORE ||
		toUpdate == USER_UPDATE_HP ||
		toUpdate == USER_UPDATE_ALL)
	{
		for (i=0;i<1;i++)
		{
			if (update)
				*more[i].toSet = convertToSystemCharset(more[i].value, info->charset);
			else
				if (*more[i].toSet) 
				{
					g_free(*more[i].toSet);
					*more[i].toSet = NULL;
				}
		}
					
		if (update)
		{
			info->age = (gushort)u->GetAge();
			info->gender = (gchar)u->GetGender(); 
			info->birthday.day = (gint)u->GetBirthDay();
			info->birthday.month = (gint)u->GetBirthMonth();
			info->birthday.year = (gint)u->GetBirthYear();
			info->birthday.hasBirthday = u->Birthday() == 0;
			info->language1 = GetLanguageByCode(u->GetLanguage1())->nIndex;
			info->language2 = GetLanguageByCode(u->GetLanguage2())->nIndex;
			info->language3 = GetLanguageByCode(u->GetLanguage3())->nIndex;		
		}			
	}
	
	printf("u4\n");	
	if (toUpdate == USER_UPDATE_MORE2 ||
		toUpdate == USER_UPDATE_ALL)
	{
		editUserInfoCategories(u, update? u->GetInterests(): NULL, GetInterestByCode, &info->interests, update);
		editUserInfoCategories(u, update? u->GetBackgrounds():NULL, GetBackgroundByCode, &info->background, update);
		editUserInfoCategories(u, update? u->GetOrganizations(): NULL, GetOrganizationByCode, &info->organizations, update);	
	}
	printf("u5\n");	
	
	if (toUpdate == USER_UPDATE_WORK ||
		toUpdate == USER_UPDATE_ALL)
	{
		for (i=0;i<10;i++)
		{
			if (update)
				*work[i].toSet = convertToSystemCharset(work[i].value, info->charset);
			else
				if (*work[i].toSet) 
				{
					g_free(*work[i].toSet);
					*work[i].toSet = NULL;
				}
		}
				
		if (update)
		{			
			if (GetCountryByCode(u->GetCompanyCountry()))
				info->company.country = GetCountryByCode(u->GetCompanyCountry())->nIndex;
			else
				info->company.country = 0;
		}

	}
	
	printf("u6\n");	
	if (toUpdate == USER_UPDATE_ABOUT ||
		toUpdate == USER_UPDATE_ALL)
	{		
		if (update)
			info->about = convertToSystemCharset(u->GetAbout(), info->charset);
		
		else
			if (info->about) 
			{
				g_free(info->about);
				info->about = NULL;
			}
	}
	
	printf("u7\n");	
	if (toUpdate == USER_UPDATE_PHONES ||
		toUpdate == USER_UPDATE_ALL)
	{
		
	} 
	
	
	if (toUpdate == USER_UPDATE_PICTURE ||
		toUpdate == USER_UPDATE_ALL)
	{
		
	}
	
	
	if (toUpdate == USER_UPDATE_TYPING ||
		toUpdate == USER_UPDATE_ALL)
	{
		if (update)
			info->typing = u->GetTyping();
	}
	
	
	if (toUpdate == USER_UPDATE_STATUS ||
		toUpdate == USER_UPDATE_ALL)
	{	
		if (update)
		{
			if (info->status == BUDDY_STATUS_OFFLINE && u->Status() != BUDDY_STATUS_OFFLINE)
				info->wentJustOnline = TRUE;
		
			info->status = u->Status();
			info->isInvisible = u->StatusInvisible();
		}
	}
	printf("u8\n");	
	if (toUpdate == USER_UPDATE_SECURITY ||
		toUpdate == USER_UPDATE_ALL)
	{
		
	}
	
	if (update)
		info->lastUpdated = time(NULL);
	
	dropLicqUser(u); */
}



void IMUserDaemon::editUserGroups(gboolean update, ICQUser* u)
{
	gint 		i1,
				group;
	GList		*search;
	gushort 	defaultGroup,
				newUserGroup;
	groupInfo	*currentGroup;
	
	if (!owner)
		return;
	
	// update group information
	if (update)
	{
		i1 = 0;	
		// one user can be in different groups at once
		// example: group 0 = new users, 1 = friends, 2 = general
		// 			group ID = 7 -> binary 111 -> user is in all three groups
		// ... this is buggy crap in my eyes ..
		group = u->GetGroups(GROUPS_USER);
		defaultGroup = gUserManager.DefaultGroup();
		newUserGroup = gUserManager.NewUserGroup();			

		search = IO_getGroupManager()->groups;
		
		for (search = IO_getGroupManager()->groups; search; search = search->next)
		{
			currentGroup = (groupInfo*)search->data;
			if (currentGroup->id == 0xFFFF)
				continue;
				
			if ( (1 << i1) & group)
				groups = g_list_append(groups, currentGroup);	
		
			i1++;				
		}
				
		if (!g_list_length(groups))
			groups = g_list_append(groups, IO_getGroupManager()->getNoGroup());
	} else
	{		
		if (groups)
		{
			g_list_free(groups);
			groups = NULL;
		}
	}
}



void IMUserDaemon::editUserInfoCategories(ICQUserCategory* cat, const struct SCategory *(*catfunc)(gushort), GList** catList, gboolean update)
{
	const gchar* 	descr;
	gushort		 	id;
	SCategory	 	*userCat;
	IMUserListInfo	*myUserCat;
	gchar			*pos1,
					*pos2,
					*tmp,
					*tmp2;
	GList			*search;
	guint			i;
	
	if (update)
	{
		
		for (i=0; cat->Get(i, &id, &descr); i++)
		{
			userCat = (SCategory*)catfunc(id);
				
			myUserCat = (IMUserListInfo*)g_malloc0(sizeof(IMUserListInfo));
			myUserCat->headline = localeToSystemCharset(userCat->szName);
			myUserCat->headlineNumber = id;
				
			// split the interest list by comma
			pos1 = (gchar*)descr;
			while((pos2 = strchr(pos1, ',')))
			{
				tmp = (gchar*)g_malloc0(pos2-pos1+1);
				memcpy(tmp, pos1, pos2-pos1);
				tmp2 = convertToSystemCharset(tmp, info->charset);
				myUserCat->entries = g_list_append(myUserCat->entries, tmp2);
				g_free(tmp);
				
				pos1 = g_strchug(pos2+1);
			}
			
			tmp = (gchar*)g_malloc0(strlen(pos1)+1);
			memcpy(tmp, pos1, strlen(pos1));
			myUserCat->entries = g_list_append(myUserCat->entries, tmp);
			
			*catList = g_list_append(*catList, myUserCat);			
		}
	
	} else
	{
		if (!*catList)
			return;
		search = *catList;
		while(search)
		{
			myUserCat = (IMUserListInfo*)search->data;
			g_free(myUserCat->headline);
			g_list_foreach(myUserCat->entries, (GFunc)g_free, NULL);
			g_list_free(myUserCat->entries);
			g_free(myUserCat);
			search = search->next;
		}
		g_list_free(*catList);
		*catList = NULL;
	}	
}


void IMUserDaemon::clearUserInfoCategories(GList *cat)
{
	GList	*search;
	IMUserListInfo	*listInfo;
	
	search = cat;
	while(search)
	{
		listInfo = (IMUserListInfo*)search->data;
		g_list_foreach(listInfo->entries, (GFunc)g_free, NULL);
		g_list_free(listInfo->entries);
		
		g_free(listInfo->headline);
		g_free(listInfo);
		
		search = search->next;
	}
	
	g_list_free(cat);
}


// -----------------------------------------------------------------------------

IMGroupManager::IMGroupManager()
{
	groups = NULL;
}


IMGroupManager::~IMGroupManager()
{
	
}


GList* IMGroupManager::loadGroups()
{
	GroupList	*groupsList;
	GroupIDList	*groupsIDList;
	guint	  	i;
	gushort	  	defaultGroup;
	groupInfo	*newGroup;
	
	if (groups)
		clearGroups();
	
	defaultGroup = gUserManager.DefaultGroup();
	
	// append pseudo group for groupless users
	newGroup = (groupInfo*)g_malloc0(sizeof(groupInfo));
	newGroup->name = g_strdup(tr("Not in Group"));
	newGroup->id = 0xFFFF;
	newGroup->icqndID = g_strdup_printf("icqnd-group-%d", newGroup->id);
	newGroup->num = 0xFFFF;
	groups = g_list_append(groups, newGroup);
	
	groupsList = gUserManager.LockGroupList(LOCK_R);
	groupsIDList = gUserManager.LockGroupIDList(LOCK_R);
	
	for (i=0;i<groupsList->size();i++)
	{
		newGroup = (groupInfo*)g_malloc0(sizeof(groupInfo));

		newGroup->name = localeToSystemCharset((*groupsList)[i]);
		newGroup->id = (*groupsIDList)[i];
		newGroup->icqndID = g_strdup_printf("icqnd-group-%d", newGroup->id);
		newGroup->num = i;
		
		groups = g_list_append(groups, newGroup);
	}

	gUserManager.UnlockGroupIDList();		
	gUserManager.UnlockGroupList();
	
	return groups;
}


void IMGroupManager::checkForNewGroups()
{	
	GroupList	*groupsList;
	GroupIDList	*groupsIDList;
	guint	  	i;
	groupInfo	*newGroup;
	GList		*search,
				*search2,
				*groupsCopy,
				*newGroupList;
	
	groupsList = gUserManager.LockGroupList(LOCK_R);
	groupsIDList = gUserManager.LockGroupIDList(LOCK_R);
	
	newGroupList = NULL;
	
	for (i=0;i<groupsIDList->size();i++)
	{
		search = groups;
		while(search)
		{
			if ( ((groupInfo*)search->data)->id == (*groupsIDList)[i])
			{
				newGroupList = g_list_append(newGroupList, search->data);
				break;
			}
			search = search->next;
		}
			
		if (!search)
		{
			newGroup = (groupInfo*)g_malloc0(sizeof(groupInfo));
			newGroup->name = localeToSystemCharset((*groupsList)[i]);
			newGroup->id = (*groupsIDList)[i];
			newGroup->icqndID = g_strdup_printf("icqnd-group-%d", newGroup->id);
			newGroup->num = i;
			groups = g_list_insert(groups, newGroup, i);
				
			newGroupList = g_list_append(newGroupList, newGroup);
		}
	}
	
	newGroupList = g_list_append(newGroupList, getNoGroup());

	gUserManager.UnlockGroupIDList();		
	gUserManager.UnlockGroupList();
		
		
	// delete old groups
	groupsCopy = g_list_copy(groups);
	search = groupsCopy;
	search2 = newGroupList;
	
	while(search)
	{
		search2 = newGroupList;
		while(search2)
		{
			if (search->data == search2->data)
				break;
			search2 = search2->next;
		}
		
		if (!search2)
		{
			newGroup = (groupInfo*)search->data;
			groups = g_list_remove(groups, newGroup);
			g_free(newGroup->name);
			g_free(newGroup->icqndID);
			g_free(newGroup);
		}
		
		search = search->next;
	}
		
	g_list_free(groupsCopy);
}



void IMGroupManager::clearGroups()
{
	GList		*search;
	groupInfo	*currentGroup;
	
	if (!groups)
		return;
	
	search = groups;
	while(search)
	{
		currentGroup = (groupInfo*)search->data;
		g_free(currentGroup->name);
		g_free(currentGroup->icqndID);
		g_free(currentGroup);
		search = search->next;
	}
	
	g_list_free(groups);
	groups = NULL;
}



void IMGroupManager::renameGroup(gushort id, gchar* newName)
{
	GList		*search;
	groupInfo 	*currentGroup;
	
	search = groups;
	while(search)
	{
		currentGroup = (groupInfo*)search->data;
		if (currentGroup->id == id)
		{
			g_free(currentGroup->name);
			currentGroup->name = g_strdup(newName);
			gUserManager.RenameGroup(gUserManager.GetGroupFromID(id), newName);
			break;
		}
		search = search->next;
	}
}


groupInfo* IMGroupManager::addGroup(gchar* name, gint where)
{
	groupInfo	*newGroup;
	GroupList	*groupsList;
	GroupIDList	*groupsIDList;
	gchar		*savedName;
	guint		id;
	GroupList::iterator iter;
	GroupIDList::iterator idIter;
	gint 		i;
	
	// FIXME: do some charset conversion here
	if (gUserManager.AddGroup(name))
	{
		newGroup = (groupInfo*)g_malloc0(sizeof(groupInfo));
		newGroup->name = g_strdup(name);
		newGroup->id = gUserManager.GetIDFromGroup(name);
		newGroup->icqndID = g_strdup_printf("icqnd-group-%d", newGroup->id);
		groups = g_list_append(groups, newGroup);
		
		if (where != -1 && where<g_list_length(groups))
		{
			groupsList = gUserManager.LockGroupList(LOCK_R);
			groupsIDList = gUserManager.LockGroupIDList(LOCK_R);
			
			iter = groupsList->begin();
			idIter = groupsIDList->begin();
			for (i=0; i<where; i++)
			{
				iter++;
				idIter++;
			}
			
			savedName = *iter;
			id = *idIter;
			
			groupsList->insert(iter, savedName);
			groupsIDList->insert(idIter, id);
			
			groupsList->pop_back();
			groupsIDList->pop_back();
			
			gUserManager.UnlockGroupIDList();
			gUserManager.UnlockGroupList();
		}
		
		return newGroup;
	}
	
	return NULL;
}


void IMGroupManager::removeGroup(gushort id)
{
	GList		*search;
	groupInfo	*info;
	
	search = groups;
	while(search)
	{
		info = (groupInfo*)search->data;
		if (info->id == id)
		{
			g_free(info->name);
			g_free(info->icqndID);
			groups = g_list_remove(groups, info);
			g_free(info);
			break;
		}
		search = search->next;
	}
	
	gUserManager.RemoveGroup(gUserManager.GetGroupFromID(id)); 
}



groupInfo* IMGroupManager::getNoGroup()
{
	GList		*search;
	groupInfo 	*info;
	
	search = groups;
	while(search)
	{
		info = (groupInfo*)search->data;
		if (info->id == 0xFFFF)
			return info;
		
		search = search->next;
	}
	
	return NULL;
}




gushort IMGroupManager::getDefaultGroup()
{
	return gUserManager.DefaultGroup();
}



void IMGroupManager::setDefaultGroup(gushort newGroup)
{
	gUserManager.SetDefaultGroup(newGroup);
}


gushort IMGroupManager::getNewUsersGroup()
{
	return gUserManager.NewUserGroup();
}


void IMGroupManager::setNewUsersGroup(gushort newGroup)
{
	gUserManager.SetNewUserGroup(newGroup);
}



groupInfo* IMGroupManager::getGroupByID(gushort id)
{
	GList	*search;
	
	search = groups;
	while(search)
	{
		if (((groupInfo*)search->data)->id == id)
			return (groupInfo*)search->data;
		search = search->next;
	}
	
	return NULL;
}



groupInfo* IMGroupManager::getGroupByIcqndID(gchar* icqndID)
{
	GList	*search;
	
	search = groups;
	while(search)
	{
		if (!strcmp(((groupInfo*)search->data)->icqndID, icqndID))
			return (groupInfo*)search->data;
		search = search->next;
	}
	
	return NULL;	
}



groupInfo* IMGroupManager::getGroupByNum(gushort num)
{
	return (groupInfo*)g_list_nth_data(groups, num);
}



groupInfo* IMGroupManager::getGroupByName(gchar* name)
{
	GList	*search;
	
	search = groups;
	while(search)
	{
		if (!strcmp(((groupInfo*)search->data)->name, name))
			return (groupInfo*)search->data;
		search = search->next;
	}
	
	return NULL;	
}


void IMGroupManager::swapGroups(gushort n1, gushort n2)
{
	GList	*el1,
			*el2;
	gpointer tmpGroup;
	
	el1 = g_list_nth(groups, n1);
	el2 = g_list_nth(groups, n2);
	
	tmpGroup = el1->data;
	el1->data = el2->data;
	el2->data = tmpGroup;
	
	gUserManager.SwapGroups(n1, n2);
}


void IMGroupManager::resortGroups(GList* newGroups)
{
	GList		*oldGroups,
				*groups2,
				*search;
	gushort 	newPos[128] = {0, };
	gushort 	count,
				i1,
				i2;
	groupInfo	*group1,
				*group2;
	GroupList	*groupsList;
	GroupIDList	*groupsIDList;
	gulong		oldUserGroups,
				newUserGroups;
	

	count = g_list_length(newGroups);

	// create a delta array. eg If the first group of the old group array moved
	// 	to the 5th position, newPos[0] would 4
	oldGroups = groups;
	
	i1 = 0;
	while(oldGroups)
	{
		group1 = (groupInfo*)oldGroups->data;
		
		search = newGroups;
		i2 = 0;
		while(search)
		{
			group2 = (groupInfo*)search->data;
			
			if (group1->id == group2->id)
			{
				newPos[i2] = i1;
				break;
			}
			i2++;
			search = search->next;
		}
		
		i1++;
		oldGroups = oldGroups->next;
	}
	
	
	// reorder the group list
	groupsList = gUserManager.LockGroupList(LOCK_R);
	groupsIDList = gUserManager.LockGroupIDList(LOCK_R);
	
	search = newGroups->next;
	i1 = 0;
	
	while(search)
	{
		group1 = (groupInfo*)search->data;
				
		g_free((*groupsList)[i1]);
		(*groupsList)[i1] = g_strdup(group1->name);
		
		(*groupsIDList)[i1] = group1->id;
		
		i1++;
		search = search->next;
	}
	gUserManager.SaveGroups();
	
	
	gUserManager.UnlockGroupList();
	gUserManager.UnlockGroupIDList();
	
	// now reorder our own list
	groups2 = g_list_copy(groups);
	g_list_free(groups);
	groups = NULL;
	
	groups = g_list_append(groups, groups2->data);
	
	for (i1 = 1; i1<count; i1++)
		if (newPos[i1]!= 0)
		groups = g_list_append(groups, g_list_nth_data(groups2, newPos[i1]));
	
	g_list_free(groups2);
	
	search = groups;
	while(search)
	{
		group1 = (groupInfo*)search->data;
		search = search->next;
	}
	
	// now renumber the groups
	i1 = 0;
	for (search = groups; search; search = search->next)
		((groupInfo*)search->data)->num = i1++;

	getNoGroup()->num = 0xFFFF;

	
	
	// reorder all users
	FOR_EACH_USER_START(LOCK_R)
	{
		oldUserGroups = pUser->GetGroups(GROUPS_USER);
		newUserGroups = 0;
		
		for (i1=1; i1<count; i1++)
			if (oldUserGroups & (1<<(newPos[i1]-1)))
				newUserGroups = newUserGroups | (1 << (i1-1));
						
		pUser->SetGroups(GROUPS_USER, newUserGroups);
	}
	FOR_EACH_USER_END
}

// ======================= Misc ===============================================



CICQDaemon *activeDaemon;

void setLicqDaemon(CICQDaemon *daemon)
{
	activeDaemon = daemon;
}


CICQDaemon* getLicqDaemon()
{
	return activeDaemon;
}


void IO_setGroupManager(IMGroupManager* man)
{
	gMan = man;
}


IMGroupManager* IO_getGroupManager()
{
	return gMan;
}


IMSignalSource* IO_getGeneralSource()
{
	if (!generalSource)
		generalSource = new IMSignalSource();
	
	return generalSource;
}
