/***************************************************************************
 *            mainwindow.cpp
 *
 *  Tue Jan 25 22:22:32 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "mainwindow.h"

mainWindow::mainWindow(mwMenuClickFunction mcf, mwDropFunction dcf, GCallback quitfunc, gpointer cbdata)
{
	GtkWidget	*vbox;
	gulong		width, 
				height, 
				posX, 
				posY;
	gboolean	alwaysOnTop,
				showInPager,
				showOnAll;

	// drag&drop targets
	static const GtkTargetEntry targets[] = 
	{
	//	{"GTK_TREE_MODEL_ROW", 0, DROP_CONTACT},
		{"application/x-im-contact", 0, DROP_GCONTACT},	
		{"text/uri-list", 0, DROP_FILES},
		{"STRING", 0, DROP_STRING}
	};	
	

	// init values
	menuCallBack 	 = mcf;
	menuCallBackData = cbdata;
	
	droppingCallback = dcf;
	
	mainMenu		 = NULL;
	notifyText		 = NULL;
	statusBox 		 = NULL;
	eventPixbuf 	 = NULL;
	title 			 = NULL;
	statusButtons 	 = NULL;
	isMoving 		 = FALSE;	
	menusLocked		 = FALSE;
	lastEventState	 = FALSE;
	
	statusName		 = g_string_new("");
	notifyName		 = g_string_new("");
	
	showMenu = FALSE;
	showNotify = TRUE;
	showStatus = FALSE;
	showTitle = TRUE;
	
	dragOpenGroupSource = 0;
	dragOpenGroup 		= NULL;
	
	
	settings_getSettings()->getProperties("contactlist",
						  				  "showTooltips", &showTooltips,
						  				  NULL);
										  
	settings_getSettings()->getProperties("mainwindow",
						  				  "width", &width,
						  				  "height", &height,
						  				  "positionX", &posX,
						  				  "positionY", &posY,
							  			  "showMainMenu", &showMenu,
							  			  "showNotifyBox", &showNotify,
							  			  "showStatusButtons", &showStatus,
							  			  "showWindowBorders", &showTitle,	
										  "alwaysOnTop", &alwaysOnTop,		
							  			  "showStarInTitle", &titleShowStar,
							  			  "showOwnersInTitle", &titleShowOwners,
										  "showWindowOnAll", &showOnAll,										  
										  "inPager", &showInPager,										  
										  "useDoubleClick", &useDoubleClick,
						  				  NULL);						  
						  
	// create contact list
	contacts = new contactList();

	g_signal_connect(contacts->widget, "button_press_event", G_CALLBACK(this->cb_contactListClicked), this);
	g_signal_connect(contacts->widget, "button_release_event", G_CALLBACK(this->cb_contactListReleased), this);
	g_signal_connect(contacts->widget, "row-activated", G_CALLBACK(this->cb_contactListActivated), this);

	// set up drag&drop
	gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(contacts->widget), targets, 3, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
	//gtk_drag_dest_set(contacts->widget, GTK_DEST_DEFAULT_ALL, targets, 3, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
	gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(contacts->widget), GDK_BUTTON1_MASK, targets, 1, GDK_ACTION_COPY);

	g_signal_connect_swapped(contacts->widget, "drag-data-received", G_CALLBACK(this->cb_contactListElementDropped), this);
	g_signal_connect(contacts->widget, "drag-motion", G_CALLBACK(this->cb_contactListDragMotion), this);
	//g_signal_connect_swapped(contacts->widget, "drag-leave", G_CALLBACK(this->cb_contactListDragLeave), this);
	g_signal_connect_swapped(contacts->widget, "drag-data-get", G_CALLBACK(this->cb_contactListDragDataGet), this);
	
	// create the tooltips
	tooltips = new userTooltips();

	if (showTooltips)
		tooltips->attachWidget(contacts->widget, (UTWidgetFunc)this->tooltipFunction, this);
	
	
	// create all menus
	createMenus();
	
	
	// create the main menu
	mainMenuContainer = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	if (showMenu)
		createMainMenu();
	
	
	// create the system notify box
	notifyBoxContainer = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	if (showNotify)
		createNotifyBox();
	
	
	// ... and the status bar
	statusBoxContainer = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	if (showStatus)
		createStatusBox();
	
	
	
	// make the mainvbox containing all elements
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), mainMenuContainer, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), contacts->frame, TRUE , TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), notifyBoxContainer, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), statusBoxContainer, FALSE, TRUE, 0);
	
	
	// ... and add everything into the main window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	g_signal_connect_swapped(G_OBJECT(window), "delete-event", G_CALLBACK(quitfunc), cbdata);
	g_signal_connect_swapped(G_OBJECT(window), "delete-event", G_CALLBACK(this->cb_destroyWindow), this);
	g_signal_connect_swapped(G_OBJECT(window), "visibility-notify-event", G_CALLBACK(this->cb_visibilityChange), this);
	g_signal_connect_swapped(G_OBJECT(window), "window-state-event", G_CALLBACK(this->cb_windowStateChanged), this);
	
	g_signal_connect_after(G_OBJECT(window), "map", G_CALLBACK(this->cb_mainWindowRealized), this);
	
	gtk_widget_add_events(window, GDK_VISIBILITY_NOTIFY_MASK | GDK_STRUCTURE_MASK);
	
	if (!showTitle)
		gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
	
	gtk_window_set_default_size(GTK_WINDOW(window), width, height);
	gtk_window_move(GTK_WINDOW(window), posX, posY);
	
	gtk_window_set_keep_above(GTK_WINDOW(window), alwaysOnTop);
	gtk_window_set_skip_pager_hint(GTK_WINDOW(window), !showInPager);	
	
	if (showOnAll)
		gtk_window_stick(GTK_WINDOW(window));
	
	// give the main window a certain role to make it distinguishable from the wm
	gtk_window_set_role(GTK_WINDOW(window), "Licq Main");
	
	gtk_widget_show_all(window);

	
	// add accelerator stuff
	
	accelerators = gtk_accel_group_new();
	// CTRL + H minimizes the window
	gtk_accel_group_connect(accelerators, 'h', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(gtk_window_iconify), window, NULL));
	// CTRL + X quits the program
	gtk_accel_group_connect(accelerators, 'x', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_exitAccelPressed), this, NULL));
	// CTRL + O switches "show offline"
	gtk_accel_group_connect(accelerators, 'o', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_showOfflineAccelPressed), this, NULL));
	// CTRL + G switches "show groups"
	gtk_accel_group_connect(accelerators, 'g', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_showGroupsAccelPressed), this, NULL));
	// CTRL + V opens the message dialog
	gtk_accel_group_connect(accelerators, 'v', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_messageAccelPressed), this, NULL));
	// CTRL + S opens the message dialog - same for historical reasons
	gtk_accel_group_connect(accelerators, 's', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_messageAccelPressed), this, NULL));
	// CTRL + U opens the URL dialog
	gtk_accel_group_connect(accelerators, 'u', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_URLAccelPressed), this, NULL));
	// CTRL + C opens the chat dialog
	gtk_accel_group_connect(accelerators, 'c', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_chatAccelPressed), this, NULL));
	// CTRL + F opens the file send dialog
	gtk_accel_group_connect(accelerators, 'f', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_ftAccelPressed), this, NULL));
	// CTRL + A opens auto response window
	gtk_accel_group_connect(accelerators, 'a', GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_aResponseAccelPressed), this, NULL));
	// CTRL + DEL removes a user from the cl
	//gtk_accel_group_connect(accelerators, GDK_Del, GDK_CONTROL_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_deleteUserFromListAccelPressed), this, NULL));

	gtk_window_add_accel_group(GTK_WINDOW(window), accelerators);	
	
	
	
	// create the tray icon stuff
	createTrayIcon();
	
			
	updateStatus(BUDDY_STATUS_OFFLINE, FALSE);
	
	// connect to the settings callback
	settings_getSettings()->installListener("contactlist", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);	
	settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);	
	settings_getSettings()->installListener("mainwindow", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
	
}



mainWindow::~mainWindow()
{

}




void mainWindow::createMainMenu()
{	
	mainMenu = gtk_menu_bar_new();

	// remove the sub menu from the other menus -> we need them here now	
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu_system1));
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu_system2));
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu_users1));
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu_users2));
	
	
	menu_system = gtk_menu_item_new_with_mnemonic("_System");
	gtk_menu_shell_append(GTK_MENU_SHELL(mainMenu), menu_system);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_system), GTK_WIDGET(systemMenu));

	
	menu_users = gtk_menu_item_new_with_mnemonic("_Users");
	gtk_menu_shell_append(GTK_MENU_SHELL(mainMenu), menu_users);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_users), GTK_WIDGET(usersMenu));
	
	
	gtk_widget_set_sensitive(menu_users, !menusLocked);

	gtk_container_add(GTK_CONTAINER(mainMenuContainer), mainMenu);	
	gtk_widget_show_all(mainMenuContainer);
}


void mainWindow::createNotifyBox()
{
	GtkWidget	*viewPort,
				*hbox,
				*frame;
	
	notifyText = gtk_label_new(notifyName->str);
	gtk_label_set_use_markup(GTK_LABEL(notifyText), TRUE);
	gtk_misc_set_alignment(GTK_MISC(notifyText), 0.0f, 0.5f);
	gtk_widget_set_size_request(notifyText, 100, -1);
	
	// add a viewport so the text can be truncated
	viewPort = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewPort), GTK_SHADOW_NONE);
	gtk_container_add(GTK_CONTAINER(viewPort), notifyText);
	gtk_widget_add_events(viewPort, GDK_BUTTON1_MOTION_MASK);
	g_signal_connect(viewPort, "button_press_event", G_CALLBACK(this->cb_notifyBoxClicked), this);

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), viewPort, TRUE, TRUE, 0);
	
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), hbox);


	gtk_container_add(GTK_CONTAINER(notifyBoxContainer), frame);
	gtk_widget_show_all(notifyBoxContainer);
}


void mainWindow::createStatusBox()
{
	GtkWidget	*hbox;
	GList		*search;
	GtkTooltips	*tips;
	
	statusButtonText = gtk_label_new(statusName->str);
	statusButtonIcon = gtk_image_new();
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), statusButtonIcon, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), statusButtonText, FALSE, TRUE, 0);
	statusButton = gtk_button_new();
	gtk_button_set_relief(GTK_BUTTON(statusButton), GTK_RELIEF_NONE);
	gtk_container_add(GTK_CONTAINER(statusButton), hbox);
	g_signal_connect(G_OBJECT(statusButton), "clicked", G_CALLBACK(this->cb_menuClicked), statusMenu);
	g_signal_connect(G_OBJECT(statusButton), "button-press-event", G_CALLBACK(this->cb_statusButtonRightClicked), this);

	statusBox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(statusBox), statusButton, TRUE, TRUE, 0);

	gtk_widget_set_sensitive(statusButton, !menusLocked);	
	
	// add all owner buttons
	search = IO_getOwnerList();
	if (g_list_length(search)>1)
		for( ; search; search = search->next)
			addStatusButton((IMOwnerDaemon*)search->data);	
		
	gtk_container_add(GTK_CONTAINER(statusBoxContainer), statusBox);
	gtk_widget_show_all(statusBoxContainer);
		
	tips = gtk_tooltips_new();
	gtk_tooltips_set_tip(tips, statusButton, tr("Set the status\nLeft button: open menu\nRight button: quick online/offline"), "");
	gtk_tooltips_enable(tips);
}



void mainWindow::createTrayIcon()
{
	gboolean showTray;
	
	settings_getSettings()->getProperties("appearance",
						  				  "showTrayIcon", &showTray,
						  				  NULL);
	trayConnectSource = 0;
	trayBlinkSource = 0;
						  	
	
	// set up tray icon
	if (showTray)
	{
		tray = new trayIcon();
		if (!tray->installTrayIcon(gtk_image_new()))
		{
			delete tray;
			tray = NULL;
		} else
			g_signal_connect_swapped(tray->eventWidget, "button_press_event", G_CALLBACK(this->cb_trayClicked), this);
	}
	else
		tray = NULL;
}


void mainWindow::setTitle(gchar* _title)
{
	if (title)
		g_free(title);
	
	title = g_strdup(_title);
	gtk_window_set_title(GTK_WINDOW(window), title);	
}


void mainWindow::setPendingEventNotify(gboolean state)
{
	GString	*newTitle;
	
	lastEventState = state;
	
	if (!title || !titleShowStar)
		return;
	
	if (state)
		newTitle = g_string_new("*");
	else
		newTitle = g_string_new("");
	
	newTitle = g_string_append(newTitle, title);
	
	gtk_window_set_title(GTK_WINDOW(window), newTitle->str);
	
	g_string_free(newTitle, TRUE);	
}



void mainWindow::updateSystemInfo(gint noe, gchar* evname, gchar* appendix)
{
	gchar	*tmp2;
	
	if (noe == 0)
		g_string_set_size(notifyName, 0);
	else
	{
		if (noe>1)
			tmp2 = appendix;
		else
			tmp2 = "";
		g_string_printf(notifyName, "%d %s%s", noe, evname, tmp2);
	}
	
	if (notifyText)
		gtk_label_set_markup(GTK_LABEL(notifyText), notifyName->str);
}



void mainWindow::updateStatus(guint status, gboolean isInvisible)
{
	if (status == BUDDY_STATUS_CONNECT)
	{
		startTrayAnimation();
		gtk_label_set_label(GTK_LABEL(statusButtonText), uu_getStatusDescription(status));
		return;
	}
	else
	if (trayConnectSource)
		stopTrayAnimation();
	
	
	lastStatusPixbuf = i_getIcons()->getStatusPixbuf("icq", status, isInvisible);
	
	// set the status button
	if (isInvisible)
		g_string_printf(statusName, "(%s)", uu_getStatusDescription(status));
	else
		g_string_printf(statusName, uu_getStatusDescription(status));

	if (statusBox)
	{		
		gtk_image_set_from_pixbuf(GTK_IMAGE(statusButtonIcon), lastStatusPixbuf);
		gtk_label_set_label(GTK_LABEL(statusButtonText), statusName->str);
	}

	// set the tray
	if (tray)
		gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
	
	lastStatus = status;
	lastInvisible = isInvisible;
	
	// set the window icon
	gtk_window_set_icon(GTK_WINDOW(window), lastStatusPixbuf);
}


GtkMenu *mainWindow::addStatusButton(IMOwnerDaemon *owner)
{
	gint i;
	iconManager			*iconMan;
	gchar				*protoName;
	statusButtonType	*newButton;
	GtkMenu				*menu;
	GtkWidget			*hbox,
						*label,
						*img;
	GList				*children;
	
	u_menuItem statusMenuList[]=
	{
		{UMENU_NORMAL	, BUDDY_STATUS_ONLINE	, NULL		, tr_NOOP("Online")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_AWAY		, NULL		, tr_NOOP("Away")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_NA		, NULL		, tr_NOOP("Not Available")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_OCC		, NULL		, tr_NOOP("Busy")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_DND		, NULL		, tr_NOOP("Do Not Disturb")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_FFC		, NULL		, tr_NOOP("Free For Chat")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_OFFLINE	, NULL		, tr_NOOP("Offline")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_INV		, NULL		, tr("Invisible")		, NULL, TRUE, FALSE, 0, &menu_statusInvisible}
	};
	
	iconMan = i_getIcons();	
	
	// update the menu list icons
	if (owner)
	{
		if (owner->protocol->pluginID == LICQ_PPID)
			protoName = g_strdup("icq");		
		else
			protoName = g_ascii_strdown(owner->protocol->licqName, -1);
	} else
		protoName = g_strdup("icq");
		
	// add the right icons
	for (i=0; i<9; i++)
		if (statusMenuList[i].type != UMENU_SEPARATOR)
			statusMenuList[i].stockIcon = g_strdup_printf("icqnd-status-%s-%d", protoName, statusMenuList[i].id);
				
	menu = u_createMenuList(statusMenuList, 9, (GtkSignalFunc)this->cb_menuItemClicked, MENU_STATUS, this, TRUE);	


	// FIXME: this is not good, but how to separate the menus otherwise?
	g_object_set_data(G_OBJECT(menu), "icqnd-status-owner", owner);
		
	if (owner)
	{
		// create a new struc
		newButton = (statusButtonType*)g_malloc0(sizeof(statusButtonType));
	
		newButton->owner 			= owner;
		newButton->connectTimeout 	= 0;
		newButton->status 			= owner->info->status;
		newButton->statusMenu 		= menu;	
		newButton->statusImage		= gtk_image_new();
		gtk_image_set_from_pixbuf(GTK_IMAGE(newButton->statusImage), 
								  iconMan->getStatusPixbuf(protoName, 
								 						   owner->info->status, 
								 						   owner->info->isInvisible));
		
		
		// create the button
		newButton->button = gtk_button_new();
		g_signal_connect(newButton->button, "clicked", G_CALLBACK(this->cb_menuClicked), newButton->statusMenu);
		g_signal_connect(G_OBJECT(newButton->button), "button-press-event", G_CALLBACK(this->cb_statusButtonRightClicked), this);

		gtk_container_add(GTK_CONTAINER(newButton->button), newButton->statusImage);
		g_object_set_data(G_OBJECT(newButton->button), "icqnd-status-button-class", newButton);
		
		
		// create the menu item for the main menu
		// FIXME: change this as soon there is a multi user support for a protocol
		label = gtk_label_new_with_mnemonic(owner->protocol->name);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
			
		img = gtk_image_new_from_stock(statusMenuList[0].stockIcon, GTK_ICON_SIZE_MENU);
		gtk_widget_set_size_request(img, 16, 16);
	
		hbox = gtk_hbox_new(FALSE, 2);
		gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 2);
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
			
		newButton->mainMenuEntry = gtk_menu_item_new();
		gtk_container_add(GTK_CONTAINER(newButton->mainMenuEntry), hbox);
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(newButton->mainMenuEntry), GTK_WIDGET(newButton->statusMenu));
		
		// check whether there has to be inserted a separator
		// (we do this by checking the count of menu items)
		children = gtk_container_get_children(GTK_CONTAINER(statusMenu));
		if (g_list_length(children) == 9)
			gtk_menu_shell_prepend(GTK_MENU_SHELL(statusMenu), gtk_separator_menu_item_new());
		g_list_free(children);
		
		gtk_menu_shell_prepend(GTK_MENU_SHELL(statusMenu), newButton->mainMenuEntry);
		gtk_widget_show_all(GTK_WIDGET(statusMenu));		
		
		
		
		gtk_box_pack_end(GTK_BOX(statusBox), newButton->button, FALSE, FALSE, 0);
		gtk_widget_show_all(statusBox);
			
		statusButtons = g_list_append(statusButtons, newButton);	
	} 
	

	// free our created stock id's
	for (i=0; i<9; i++)
		if (statusMenuList[i].type != UMENU_SEPARATOR)
			g_free(statusMenuList[i].stockIcon);	
	
	g_free(protoName);

	return menu;	
}



void mainWindow::removeStatusButton(IMOwnerDaemon *owner)
{
	GList				*search,
						*children;
	statusButtonType	*but;
	
	search = statusButtons;
	
	while(search)
	{
		but = (statusButtonType*)search->data;
		if (but->owner == owner)
		{
			gtk_container_remove(GTK_CONTAINER(statusBox), but->button);
			gtk_widget_destroy(GTK_WIDGET(but->statusMenu));	
			gtk_widget_destroy(but->mainMenuEntry);
			// check whether the separator has to be removed
			children = gtk_container_get_children(GTK_CONTAINER(statusMenu));
			if (g_list_length(children) == 10)
				gtk_widget_destroy(GTK_WIDGET(children->data));
			g_list_free(children);
			
			statusButtons = g_list_remove(statusButtons, but);
			
			g_free(but);
			return;
		}
		
		search = search->next;
	}
}



void mainWindow::updateStatusButton(IMOwnerDaemon *owner, glong status, gboolean invisible)
{
	GList				*search;
	statusButtonType	*but;
	gchar				*protoName;
	
	if (!statusBox)
		return;
	
	search = statusButtons;
	while(search)
	{
		but = (statusButtonType*)search->data;
		
		if (but->owner == owner)
			break;
		
		search = search->next;
	}
	
	if (!search)
		return;
		
	if (status == BUDDY_STATUS_CONNECT && !but->connectTimeout)
	{
		but->connectTimeout = g_timeout_add(300, (GSourceFunc)this->cb_statusButtonBlinking, but);
		return;
	}
	else
		if (but->connectTimeout)
		{
			g_source_remove(but->connectTimeout);
			but->connectTimeout = 0;
		}
		
		
	if (owner->info->pluginID == LICQ_PPID)
		protoName = "icq";
	else
		protoName = owner->protocol->licqName;
				
	gtk_image_set_from_pixbuf(GTK_IMAGE(but->statusImage), 
							  i_getIcons()->getStatusPixbuf(protoName,
															status,
															invisible));
	but->status = status;

}



void mainWindow::setLockUserMenus(gboolean lock)
{
	if (showMenu)
		gtk_widget_set_sensitive(menu_users, !lock);
	
	gtk_widget_set_sensitive(menu_status, !lock);
	gtk_widget_set_sensitive(menu_reloadContacts, !lock);
	
	if (showStatus)
		gtk_widget_set_sensitive(statusButton, !lock);	
	
	menusLocked = lock;
}



void mainWindow::setWindowTitle(gboolean newEvents)
{
	GString			*title;
	gchar			*tmpTitle;
	GList			*search;
	IMOwnerDaemon	*owner;

	search = IO_getOwnerList();

	title = g_string_new("");
	
	if (titleShowOwners)
	{
		for (search = IO_getOwnerList(); search; search = search->next)
		{
			owner = (IMOwnerDaemon*)search->data;
		
			tmpTitle = uu_getFormattedBuddyName(owner->info, FALSE);
			g_string_append_printf(title, "%s, ", tmpTitle);
			g_free(tmpTitle);		
		}
	
		if (strlen(title->str)>0)
			title = g_string_set_size(title, strlen(title->str)-2);
	
		tmpTitle = g_strdup(title->str);
		g_string_printf(title, "Licq (%s)", tmpTitle);
		g_free(tmpTitle);
	} else
		g_string_printf(title, "Licq");
	
	setTitle(title->str);
	setPendingEventNotify(newEvents);
	
	g_string_free(title, TRUE);	
}



void mainWindow::destroyWindow()
{
	cb_destroyWindow(this);
	// gtk_widget_destroy(window);
}



void mainWindow::trayClicked(GdkEventType type, guchar button)
{
	GdkEventType clickType;

	settings_getSettings() -> getProperties( "mainwindow", "useDoubleClick", &useDoubleClick, NULL );

	clickType = useDoubleClick ? GDK_2BUTTON_PRESS : GDK_BUTTON_PRESS;
	
	if ( type == clickType && button == 1 ) {
		// left button was clicked -> show / hide window
		if ( GTK_WIDGET_VISIBLE( window ) ) {
			// show window when it's iconified or obscured
			if ( ( gdk_window_get_state( window -> window ) & GDK_WINDOW_STATE_ICONIFIED ) || isObscured ) {
				gtk_window_present( GTK_WINDOW( window ) );
			} else {
				gtk_window_iconify( GTK_WINDOW( window ) );
			}
		} else {
			gtk_window_present( GTK_WINDOW( window ) );
		}
	} else if ( type == GDK_BUTTON_PRESS && button != 1 ) {
		switch ( button ) {
			case 3:
				// right button was clicked -> show menu
				gtk_menu_popup(systemMenu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
				break;
			case 2:
				if (trayBlinkSource!=0)
					menuCallBack(NULL, MENU_NOTIFYBOX, 0, menuCallBackData);	
				break;
			default:
				break;
		}
	}
}


void mainWindow::startTrayAnimation()
{
	if (!tray || trayConnectSource)
		return;
	
	trayIter = 0;
	trayConnectSource = g_timeout_add(150, (GSourceFunc)this->cb_trayAnimation, this);
}


void mainWindow::stopTrayAnimation()
{
	if (tray && trayConnectSource != 0)
	{
		g_source_remove(trayConnectSource);
		trayConnectSource = 0;
		// set the original pixbuf
		gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
		
	}
}


gboolean mainWindow::isTrayAnimation()
{
	return trayConnectSource != 0;
}


void mainWindow::setTrayBlinkEvent(basicEventInfo *ev)
{		
	if ((!ev && !trayBlinkSource) || !tray)
		return;
	
	if (!ev)
	{
		if (trayBlinkSource)
		{
			g_source_remove(trayBlinkSource);
			gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
			trayBlinkSource = 0;
		}
	} else
	{
		eventPixbuf = i_getIcons()->getEventPixbuf(ev->type);		
		
		if (!trayBlinkSource)
			trayBlinkSource = g_timeout_add(300, (GSourceFunc)this->cb_trayEventBlinking, this);
		
	}
}



void mainWindow::createMenus()
{
	GtkMenu	*viewMenu,
			*debuggingMenu;
	gboolean	showGroups,
				showEmptyGroups,
				showOfflineUsers;

	
	// create the user menu
	// ====================
	u_menuItem buddyMenuList[]=
	{
		{UMENU_HEADLINE	, MENU_NONE				 , NULL, "", NULL, FALSE, FALSE, 0, &menu_userMenuLabel},
		{UMENU_NORMAL	, MENU_OPENEVENT		 , ICON_MESSAGE		, tr_NOOP("Open Event")			  , NULL, FALSE, FALSE, 0, &menu_openUserEvent},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_MESSAGE			 , ICON_MESSAGE		, tr_NOOP("Send Message")			  , NULL, TRUE, FALSE, 0, &menu_sendMessage},
		{UMENU_NORMAL	, UE_URL				 , ICON_URL			, tr_NOOP("Send URL")				  , NULL, TRUE, FALSE, 0, &menu_sendURL},
		{UMENU_NORMAL	, UE_FILETRANSFERREQUEST , ICON_FILE		, tr_NOOP("Send Files")				  , NULL, TRUE, FALSE, 0, &menu_sendFiles},
		{UMENU_NORMAL	, UE_CHATREQUEST		 , ICON_CHAT		, tr_NOOP("Request A Chat")			  , NULL, TRUE, FALSE, 0, &menu_sendChat},
		{UMENU_NORMAL	, UE_CONTACTSREQUEST	 , ICON_CONTACTS	, tr_NOOP("Send Contacts")			  , NULL, TRUE, FALSE, 0, &menu_sendContacts},
		{UMENU_NORMAL	, UE_AUTHORIZATIONREQUEST, ICON_AUTH		, tr_NOOP("Ask For Authorization")	  , NULL, TRUE, FALSE, 0, &menu_sendAuthReq},
		{UMENU_NORMAL	, UE_AUTHORIZATIONGRANT	 , ICON_AUTH		, tr_NOOP("Send Authorization")		  , NULL, TRUE, FALSE, 0, &menu_sendAuth},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_ENCRYPTION			 , NULL				, tr_NOOP("Encrypt Connection with OpenSSL")		  , NULL, TRUE, FALSE, 0, &menu_sendEncryption},
#ifdef HAVE_LIBGPGME
		{UMENU_NORMAL	, UE_ENCRYPTGPG			 , NULL				, tr_NOOP("Encrypt Connection with GPG")			  , NULL, TRUE, FALSE, 0, NULL},
#endif		
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTORESPONSEGET	 , NULL				, tr_NOOP("Check Auto Response")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTORESPONSESET	 , NULL				, tr_NOOP("Set Special Auto Response"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_EDITUSERMODES		 , NULL				, tr_NOOP("Edit Groups / Modes")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_USERINFO			 , ICON_INFO		, tr_NOOP("Users Information")		  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_HISTORY			 , ICON_HISTORY		, tr_NOOP("History")				  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE				 , NULL				, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_REMOVEUSER		 , GTK_STOCK_CANCEL	, tr_NOOP("Remove User")			  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, &menu_syssep},
		{UMENU_NORMAL	, MENU_NONE				 , NULL, tr_NOOP("System"), NULL, TRUE, FALSE, 0, &menu_system2},
		{UMENU_NORMAL	, MENU_NONE				 , NULL, tr_NOOP("Users"), NULL, TRUE, FALSE, 0, &menu_users2}	

	};
	
#ifdef HAVE_LIBGPGME
	userMenu = u_createMenuList(buddyMenuList, 25, (GtkSignalFunc)this->cb_menuItemClicked, MENU_USER, this, TRUE);
#else
	userMenu = u_createMenuList(buddyMenuList, 24, (GtkSignalFunc)this->cb_menuItemClicked, MENU_USER, this, TRUE);
#endif
	

	// create the main menu
	// ====================	
	
	// Menus of the second level. These menus will be sub menus of the first level menus
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  "showOfflineUsers", &showOfflineUsers,
										  NULL);

	u_menuItem viewMenuList[3]=
	{
		{UMENU_CHECK	, MENU_SHOWGROUPS		, NULL, tr_NOOP("Show Groups")			, NULL, TRUE		, showGroups	  , 0, &menu_showGroups},
		{UMENU_CHECK	, MENU_SHOWEMPTYGROUPS	, NULL, tr_NOOP("Show Empty Groups")	, NULL, showGroups	, showEmptyGroups , 0, &menu_showEmptyGroups},
		{UMENU_CHECK	, MENU_SHOWOFFLINEUSERS	, NULL, tr_NOOP("Show Offline Users")	, NULL, TRUE		, showOfflineUsers, 0, &menu_showOfflineUsers},
	};		
		
	
	u_menuItem debuggingMenuList[1]=
	{
		{UMENU_NORMAL, UE_LOGMESSAGES	, GTK_STOCK_NETWORK, tr_NOOP("Network Window"), NULL, TRUE, FALSE, 0, NULL}
	};
	
	
	// create the menus
	viewMenu = u_createMenuList(viewMenuList, 3, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, FALSE);
	debuggingMenu = u_createMenuList(debuggingMenuList, 1, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	statusMenu = addStatusButton(NULL);
	
	// Menus of the first level
	u_menuItem usersMenuList[13]=
	{
		{UMENU_NORMAL	, MENU_OPENALL			 , ICON_MESSAGE, tr_NOOP("Open All Messages")	 , NULL, TRUE, FALSE, 0, &menu_openAllEvents},
		{UMENU_SEPARATOR, MENU_NONE				 , NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_WHITEPAGES			 , GTK_STOCK_FIND, tr_NOOP("Find Users")			 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_ADDUSER			 , GTK_STOCK_ADD, tr_NOOP("Add User")			 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONGRANT	 , ICON_AUTH, tr_NOOP("Authorize User")		 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONREQUEST, ICON_AUTH, tr_NOOP("Request Authorization"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_SETRANDOMCHAT		 , ICON_CHAT, tr_NOOP("Set Random Chat Group"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_STARTRANDOMCHAT	 , ICON_CHAT, tr_NOOP("Start Random Chat")	 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_EDITGROUPS			 , GTK_STOCK_INDEX, tr_NOOP("Edit Groups ...")		 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_USERLISTS			 , NULL		, tr_NOOP("Edit Users ..."),	NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_UPDATEALL		 , NULL, tr_NOOP("Update All Users")	 , NULL, TRUE, FALSE, 0, NULL}
	};
	
	
	u_menuItem systemMenuList[]=
	{
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Status")		, statusMenu	, TRUE, FALSE, 0, &menu_status},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("View")			, viewMenu		, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Debugging")	, debuggingMenu	, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_MANAGEOWNERS, NULL, tr_NOOP("Owner Manager..."), NULL, TRUE, FALSE, 0, NULL},
#ifdef HAVE_LIBGPGME
		{UMENU_NORMAL	, UE_GPGMANAGER, NULL, tr_NOOP("Manage GPG Keys..."), NULL, TRUE, FALSE, 0, NULL},
#endif		
		{UMENU_NORMAL	, UE_OPTIONS	, GTK_STOCK_PREFERENCES, tr_NOOP("Options...")	, NULL			, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_STATISTICS, NULL, tr_NOOP("Statistics")	, NULL			, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_TIPS	, NULL, tr_NOOP("Tips")			, NULL			, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_ABOUT, NULL, tr_NOOP("About")		, NULL			, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_QUIT		, GTK_STOCK_QUIT, tr_NOOP("Exit")			, NULL			, TRUE, FALSE, 0, NULL}
	};

	
	usersMenu = u_createMenuList(usersMenuList, 13, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);

#ifdef HAVE_LIBGPGME
	systemMenu = u_createMenuList(systemMenuList, 13, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
#else
	systemMenu = u_createMenuList(systemMenuList, 12, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
#endif
	g_object_ref(G_OBJECT(usersMenu));
	g_object_ref(G_OBJECT(systemMenu));
	
	// create the system-user menu
	u_menuItem systemUsersList[2] =
	{
		{UMENU_NORMAL	, MENU_NONE, NULL, tr_NOOP("System")	, NULL, TRUE, FALSE, 0, &menu_system1},
		{UMENU_NORMAL	, MENU_NONE, NULL, tr_NOOP("Users")		, NULL, TRUE, FALSE, 0, &menu_users1}
	};
	
	systemUsersMenu = u_createMenuList(systemUsersList, 2, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
}



void mainWindow::showAboutWindow()
{
	gchar* authors[] = 
	{"Joachim Staib (Lead Programmer)",
	 "",
	 "Patch Writers",
	 NULL};
	 
	gchar* documenters[] = 
	{"Joachim Staib (who hasn't written any docs yet)", NULL};
	 
	gchar	*license,
			*translators;
	 
	license = "IcQ'nD is under the GPL V2. Visit http://www.gnu.org/copyleft/gpl.html to read more.\n\n"
		      "Some icons are taken from the Tango Icon Gallery created by the Tango Desktop Project. These icons are under the \"Creative Commons Attribution Share-Alike license\". Read more at http://creativecommons.org/licenses/by-sa/2.5/";
	
	translators = "Noone has translated the program yet ...";
	
	gtk_show_about_dialog(GTK_WINDOW(window),
						  "name", "IcQ'nD",
						  "comments", "A Gtk2 frontend for Licq",
						  "copyright", "Joachim Staib <camael@gmx.net>",
					      "version", "0.2",
						  "website", "http://icqnd.sourceforge.net",
	 					  "authors", authors,
	 					  "wrap-license", TRUE,
	 					  "license", license,
						  "documenters", documenters,
						  "translator-credits", translators,
						  NULL);
}


/*
GtkWidget* mainWindow::createMainMenu()
{
	GtkMenu	*systemMenu,
			*userMenu,
			*helpMenu,
			*_statusMenu;
	gboolean showGroups,
			 showEmptyGroups,
			 showOfflineUsers; 
	GtkWidget	*menuBar;
	
	if (mainMenu)
		return mainMenu;
	
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  "showOfflineUsers", &showOfflineUsers,
										  NULL);	
		
	
	// order of a menu item
	// menu type (see utilities.h), radio group id, title, icon, id, enabled?, selected?, target (or NULL for none), submenu
	

	// menus of the 2nd level
	u_menuItem systemMenuList[5]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("User information")			, ICON_INFO		, UE_USERINFO		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Security / Password")  		, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Network window") 	  		, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Debugging")		  			, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL}
	};

	u_menuItem userMenuList[12]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Find user") 				, GTK_STOCK_FIND, USER_FIND			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Add user")	  			, GTK_STOCK_ADD	, USER_ADD			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorize user")			, NULL		  	, USER_AUTHORIZE	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Request authorization")	, NULL		  	, USER_GETAUTH		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL}	,
		{UMENU_NORMAL, 0, tr_NOOP("Open all messages")		, ICON_MESSAGE	, USER_OPENALL		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Random chat")  			, ICON_CHAT	  	, RANDOM_USERCHAT	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Random chat group") 		, ICON_CHAT	  	, RANDOM_CHATGROUP	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Update all users")		, NULL		  	, USER_UPDATEALL	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Register user")   		, NULL		  	, USER_REGISTER		, TRUE, TRUE, NULL, NULL}
	};

	
	
	u_menuItem groupMenuList[4]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Edit groups")	  		, NULL		, GROUP_EDIT		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_CHECK, 0, tr_NOOP("Show Groups")				, NULL			, GROUP_SHOW, TRUE, showGroups, NULL, NULL},
		{UMENU_CHECK, 0, tr_NOOP("Show empty groups")		, NULL			, GROUP_SHOWEMPTY, showGroups, showEmptyGroups, &showEmptyGroupsMenu, NULL}				
	};

	u_menuItem helpMenuList[3]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Tips")	  				, NULL		  	, HELP_TIPS			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("About")		  			, NULL		  	, HELP_ABOUT		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Statistics")	  			, NULL		  	, HELP_STATS		, TRUE, TRUE, NULL, NULL}
	};

	// create menus
	systemMenu  = u_createMenuList(systemMenuList, 5, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this);
	userMenu    = u_createMenuList(userMenuList,  12, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	groupMenu   = u_createMenuList(groupMenuList,  4, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	helpMenu    = u_createMenuList(helpMenuList,   3, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	_statusMenu = createStatusMenu();
	
	
	// create main menu
	u_menuItem mainMenuList[14]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("System")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, systemMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Users") 				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, userMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Groups")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, groupMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Status")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, _statusMenu},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Mini mode") 			, NULL		  	, MAIN_MINIVIEW		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Reload contact list"), NULL		  	, MAIN_ACTUALIZE	, TRUE, TRUE, NULL, NULL},
		{UMENU_CHECK,  0, tr_NOOP("Show offline users")	, NULL		  	, MAIN_SHOWOFFLINE	, TRUE, showOfflineUsers, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Options ...")		, NULL		  	, MAIN_OPTIONS		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Plugins ...") 		, NULL		  	, MAIN_PLUGINS		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Save settings") 		, NULL		  	, MAIN_SAVESETTINGS	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Help") 				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, helpMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Exit")	  			, NULL		  	, MAIN_EXIT			, TRUE, TRUE, NULL, NULL}
	};
	
	mainMenu = u_createMenuList(mainMenuList, 14, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	
	return mainMenu;
	
} 




GtkMenu* mainWindow::createStatusMenu()
{
	if (statusMenu)
		return statusMenu;
	
	u_menuItem statusMenuList[]=
	{
		{UMENU_NORMAL, BUDDY_STATUS_ONLINE	, ICON_ONLINE		, tr_NOOP("Online")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_AWAY	, ICON_AWAY			, tr_NOOP("Away")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_NA		, ICON_NA			, tr_NOOP("Not Available")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_OCC		, ICON_OCCUPIED		, tr_NOOP("Busy")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_DND		, ICON_DND			, tr_NOOP("Do Not Disturb")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_FFC		, ICON_CHAT			, tr_NOOP("Free For Chat")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_OFFLINE	, ICON_OFFLINE		, tr_NOOP("Offline")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, UE_UNDEFINED, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_INV		, ICON_INVISIBLE	, tr_NOOP("Invisible")		, NULL, TRUE, FALSE, 0, NULL}
	};
	

	statusMenu = u_createMenuList(statusMenuList, 9, (GtkSignalFunc)this->cb_menuItemClicked, MENU_STATUS, this);	

	return statusMenu;
}



GtkMenu* mainWindow::createUserMenu()
{
	GtkMenu *sendMenu;
	if (userMenu)
		return userMenu;
	
	u_menuItem userMenuSendList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Message")				, ICON_MESSAGE	, UE_MESSAGE			 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("URL")					, ICON_URL		, UE_URL				 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Chat request")			, ICON_CHAT		, UE_CHATREQUEST		 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Files")					, ICON_FILE		, UE_FILETRANSFERREQUEST , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Contacts")				, ICON_INFO		, UE_CONTACTSREQUEST	 , TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE	, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorization request")	, ICON_INFO		, UE_AUTHORIZATIONREQUEST, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorization")			, ICON_INFO		, UE_AUTHORIZATIONGRANT	 , TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Encrypt connection")		, ICON_INFO		, UE_ENCRYPTION			 , TRUE, TRUE, NULL, NULL}
	};
	
	sendMenu = u_createMenuList(userMenuSendList, 10, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SEND, this);
	
	
	
	u_menuItem userMenuList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Show messages")			, NULL			, POPUP_SHOWEVENTS, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Send event")				, NULL			, POPUP_SEND	, TRUE, TRUE, NULL, sendMenu},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE	, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Check auto response")	, NULL			, POPUP_ARESPONSEGET, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Special auto response")	, NULL			, POPUP_ARESPONSESET, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("User information")		, ICON_INFO		, POPUP_INFO	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("History")				, ICON_HISTORY	, POPUP_HISTORY	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Remove user")			, ICON_BLANK	, POPUP_DELETE	, TRUE, TRUE, NULL, NULL}
	};
	
	
	// userMenu = u_createMenuList(userMenuList, 8, (GtkSignalFunc)this->cb_menuItemClicked, MENU_POPUP, this);

	return userMenu;
}




GtkMenu* mainWindow::createGroupMenu()
{
	gboolean showGroups,
			 showEmptyGroups;
	
	if (groupMenu)
		return groupMenu;
	
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  NULL);
	
	
	u_menuItem groupMenuList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Rename group")		, ICON_MESSAGE	, 0	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Edit group")			, ICON_URL		, 0	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Remove group")		, ICON_FILE		, 0	, TRUE, TRUE, NULL, NULL}
	};
	
		
	groupMenu = u_createMenuList(groupMenuList, 4, (GtkSignalFunc)this->cb_menuItemClicked, MENU_POPUP, this);

	return groupMenu;	
	
}
*/






// =============================================================================

gboolean mainWindow::cb_contactListClicked(GtkTreeView* cList, GdkEventButton* event, mainWindow* mw)
{
	contactListEntry *focus;
	gulong			 capats;
	
	if (event->button == 2 && !mw->isMoving)
	{
		mw->isMoving = TRUE;
		gdk_pointer_grab(mw->contacts->widget->window,
						 FALSE,
						 (GdkEventMask)(GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK),
						 GDK_WINDOW(GDK_NONE),
						 NULL,
						 GDK_CURRENT_TIME);
		
		gtk_window_get_position(GTK_WINDOW(mw->window), &mw->moveXDelta, &mw->moveYDelta);
		
		mw->moveXDelta-=(gint)event->x_root;
		mw->moveYDelta-=(gint)event->y_root;
	}		
	
	if (event->button != 3)
		return FALSE;
	
	focus = mw->contacts->getEntryAtCursor((gint)event->x, (gint)event->y);
	
	if (focus && focus->type == CE_USER)
	{
		// find out what menus to enable / disable
		capats = ((IMOwnerDaemon*)((IMUserDaemon*)focus->data)->owner)->protocol->functions;

		(capats & PP_SEND_MSG) != 0? gtk_widget_show(mw->menu_sendMessage): gtk_widget_hide(mw->menu_sendMessage);
		(capats & PP_SEND_URL) != 0? gtk_widget_show(mw->menu_sendURL): gtk_widget_hide(mw->menu_sendURL);
		(capats & PP_SEND_FILE) != 0? gtk_widget_show(mw->menu_sendFiles): gtk_widget_hide(mw->menu_sendFiles);
		(capats & PP_SEND_CHAT) != 0? gtk_widget_show(mw->menu_sendChat): gtk_widget_hide(mw->menu_sendChat);
		(capats & PP_SEND_CONTACT) != 0? gtk_widget_show(mw->menu_sendContacts): gtk_widget_hide(mw->menu_sendContacts);
		(capats & PP_SEND_AUTH) != 0? gtk_widget_show(mw->menu_sendAuth): gtk_widget_hide(mw->menu_sendAuth);
		(capats & PP_SEND_AUTHxREQ) != 0? gtk_widget_show(mw->menu_sendAuthReq): gtk_widget_hide(mw->menu_sendAuthReq);
		(capats & PP_SEND_SECURE) != 0? gtk_widget_show(mw->menu_sendEncryption): gtk_widget_hide(mw->menu_sendEncryption);
		
		gtk_label_set_label(GTK_LABEL(mw->menu_userMenuLabel), ((IMUserDaemon*)focus->data)->info->alias);
		
		// do we have to show the system and users menu?
		if (!mw->showMenu)
		{
			gtk_menu_item_remove_submenu(GTK_MENU_ITEM(mw->menu_system1));
			gtk_menu_item_remove_submenu(GTK_MENU_ITEM(mw->menu_users1));
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(mw->menu_system2), GTK_WIDGET(mw->systemMenu));
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(mw->menu_users2), GTK_WIDGET(mw->usersMenu));
			
			gtk_widget_show(mw->menu_system2);
			gtk_widget_show(mw->menu_users2);
			gtk_widget_show(mw->menu_syssep);
		} else
		{
			gtk_widget_hide(mw->menu_system2);
			gtk_widget_hide(mw->menu_users2);
			gtk_widget_hide(mw->menu_syssep);
		}
		
		
		gtk_menu_popup(mw->userMenu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
	} else
	
	// show our systems and user menu again if neccessary
	if (!mw->showMenu)
	{
		
		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(mw->menu_system2));
		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(mw->menu_users2));
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(mw->menu_system1), GTK_WIDGET(mw->systemMenu));
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(mw->menu_users1), GTK_WIDGET(mw->usersMenu));
		
		gtk_menu_popup(mw->systemUsersMenu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
	}
	return FALSE;
}



gboolean mainWindow::cb_contactListReleased(GtkTreeView* cList, GdkEventButton* event, mainWindow* self)
{	
	if (self->isMoving)
	{
		self->isMoving = FALSE;
		gdk_pointer_ungrab(GDK_CURRENT_TIME);
	}
	
	return FALSE;
}



void mainWindow::cb_contactListActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, mainWindow* mw)
{
	contactListEntry *focus;
	
	focus = mw->contacts->getFocusedEntry();
	
	if (!focus)
		return;
	
	// a group was clicked -> collapse or expand it
	if (focus->type == CE_USER)
		mw->menuCallBack(NULL, MENU_DOUBLECLICK, 0, mw->menuCallBackData);		
}



void mainWindow::cb_contactListElementDropped(mainWindow* self, 
										   	  GdkDragContext   *drag_context,
                                              gint              x,
                                              gint              y,
                                              GtkSelectionData *data,
                                              guint             info,
                                              guint             time)
{
	contactListEntry* entry;
	
	entry = self->contacts->getEntryAtDropPosition(x, y);
	
	if (self->droppingCallback)
		self->droppingCallback(entry, info, data, self->menuCallBackData);
}



gboolean mainWindow::cb_contactListDragMotion(GtkWidget		 *widget, 
                                              GdkDragContext *drag_context,
                                              gint            x,
                                              gint            y,
                                              guint           time,
 											  mainWindow* self)
{
	GtkTreePath		 *path;
	contactListGroup *entry;
	
	gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(self->contacts->widget), x, y, &path, NULL);
	if (path)
	{
		gtk_tree_view_set_drag_dest_row(GTK_TREE_VIEW(self->contacts->widget), path, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
		gtk_tree_path_free(path);
		entry = (contactListGroup*)self->contacts->getEntryAtDropPosition(x, y);
		if (entry && entry->type == CE_GROUP && !entry->isGroupOpen() && (!self->dragOpenGroup || self->dragOpenGroup == entry))
		{
			if (self->dragOpenGroupSource)
				g_source_remove(self->dragOpenGroupSource);
		
			self->dragOpenGroup = entry;			
			self->dragOpenGroupSource = g_timeout_add(500, (GSourceFunc)self->cb_contactListDragOpenGroup, self);
		}
	}
	
	return FALSE;
}


void mainWindow::cb_contactListDragLeave(mainWindow* self, GdkDragContext *drag_context, guint time)
{	
	if (self->dragOpenGroupSource)
	{
		g_source_remove(self->dragOpenGroupSource);
		self->dragOpenGroupSource = 0;
		self->dragOpenGroup = NULL;
	}
}


gboolean mainWindow::cb_contactListDragOpenGroup(mainWindow* self)
{
	if (self->dragOpenGroup)
	{
		self->dragOpenGroup->openGroup(TRUE);
		self->dragOpenGroup = NULL;
		self->dragOpenGroupSource = 0;
	}
	return FALSE;
}



void mainWindow::cb_contactListDragDataGet(mainWindow		*self,
                               GdkDragContext   *drag_context,
                               GtkSelectionData *data,
                               guint             info,
                               guint             time)
{
	contactListEntry	*entry;
	gchar				*str;
	
	// a request for gaim contact info
	if (info == DROP_GCONTACT)
	{
		entry = self->contacts->getContextEntry(drag_context);
		if (entry->type == CE_USER)
		{
			str = uu_createGaimDnDMimeType((IMUserDaemon*)entry->data, entry->getParent());
			gtk_selection_data_set(data, 
								   gdk_atom_intern("application/x-im-contact", FALSE),
								   8,
								   (guchar*)str,
								   strlen(str)+1);
			g_free(str);
		}
	}
}





void mainWindow::cb_menuClicked(GtkWidget* w, GtkWidget* toshow)
{
	gtk_menu_popup(GTK_MENU(toshow), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());

}


void mainWindow::cb_menuItemClicked(GtkWidget* item, mainWindow* mw)
{
	guint menu, 
		  selection;
	
	menu = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "ICQmenuMENU"));
	selection = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "ICQmenuID"));
			
	if (selection != MENU_NONE && mw->menuCallBack)
		mw->menuCallBack(item, menu, selection, mw->menuCallBackData);
}



gboolean mainWindow::cb_destroyWindow(mainWindow* mw)
{
	gint width, height, x, y;
	
	gtk_window_get_position(GTK_WINDOW(mw->window), &x, &y);
	gtk_window_get_size(GTK_WINDOW(mw->window), &width, &height);
	
	settings_getSettings()->setProperties(FALSE, "mainwindow",
										  "width", width,
										  "height", height,
										  "positionX", x,
										  "positionY", y,
										  NULL);
	
	// delete the tray
	if (mw->tray)
	{
		mw->tray->removeTrayIcon();
		delete mw->tray;
	}
	
	// delete tooltips
	delete mw->tooltips;
	
	
	gtk_main_quit();
	
	gtk_widget_destroy(mw->window);
	
	return FALSE;
}



gboolean mainWindow::cb_visibilityChange(mainWindow* mw, GdkEventVisibility *event)
{
	mw->isObscured = (event->state == GDK_VISIBILITY_FULLY_OBSCURED);
		
	return FALSE;
}


gboolean mainWindow::cb_windowStateChanged(mainWindow* self, GdkEventWindowState* event)
{
	gboolean setting;
	
	// did we maximize the window?
	if (event->new_window_state == 0)
	{
		settings_getSettings()->getProperties("mainwindow", "inTaskbarWhenMaximized", &setting, NULL);
		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(self->window), !setting);
		
	} else
	if (event->new_window_state & 2)
	{
		if (self->tray)
		{
			settings_getSettings()->getProperties("mainwindow", "inTaskbarWhenMinimized", &setting, NULL);
			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(self->window), !setting);		
		} else
			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(self->window), FALSE);
	}
	
	return FALSE;
}




gboolean mainWindow::cb_trayClicked(mainWindow* mw, GdkEventButton* event)
{
	mw->trayClicked(event->type, event->button);
	
	return FALSE;
}


gboolean mainWindow::cb_trayAnimation(mainWindow* mw)
{
	GdkPixbuf	*icon;
	
	mw->trayIter++;
	mw->trayIter = mw->trayIter % 2;
	
	if (mw->trayIter)
		icon = i_getIcons()->getStatusPixbuf("icq", BUDDY_STATUS_ONLINE, FALSE);
	else
		icon = i_getIcons()->getStatusPixbuf("icq", BUDDY_STATUS_OFFLINE, FALSE);
	
	gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), icon);
	
	return TRUE;
}



gboolean mainWindow::cb_trayEventBlinking(mainWindow* mw)
{
	// don't do anything if the connect animation is visible
	if (mw->trayConnectSource)
		return TRUE;
	
	mw->trayShowEventIcon = !mw->trayShowEventIcon;
	
	if (mw->trayShowEventIcon)
		gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), mw->eventPixbuf);
	else
		gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), mw->lastStatusPixbuf);
	
	return TRUE;
}


gboolean mainWindow::cb_statusButtonBlinking(statusButtonType* but)
{
	gchar	*protoName;
	
	but->connectImg = !but->connectImg;
	
	protoName = ((IMOwnerDaemon*)but->owner)->protocol->licqName;
	if (!strcmp(protoName, "Licq"))
		protoName = "icq";
	
	if (but->connectImg)
		gtk_image_set_from_pixbuf(GTK_IMAGE(but->statusImage), i_getIcons()->getStatusPixbuf(protoName, BUDDY_STATUS_ONLINE, FALSE));
	else
		gtk_image_set_from_pixbuf(GTK_IMAGE(but->statusImage), i_getIcons()->getStatusPixbuf(protoName, BUDDY_STATUS_OFFLINE, FALSE));		

	return TRUE;	
}



gboolean mainWindow::cb_statusButtonRightClicked(GtkWidget* but, GdkEventButton* event, mainWindow* self)
{
	statusButtonType	*statusButton;
	GtkWidget			*w1, *w2;	

	statusButton = (statusButtonType*)g_object_get_data(G_OBJECT(but), "icqnd-status-button-class");
	
	
	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
	{
		
		// FIXME: this is weird (CLEAR UP THIS MESS!!)!
		// we do this because the menuCallback function from IMOwner will 
		// look for the parent of the given widget for the owner in the key "icqnd-status-owner"
		w1 = gtk_event_box_new();
		w2 = gtk_event_box_new();
		if (statusButton)
			g_object_set_data(G_OBJECT(w1), "icqnd-status-owner", statusButton->owner);
		gtk_container_add(GTK_CONTAINER(w1), w2);		
		
		if ((statusButton && ((IMUserDaemon*)statusButton->owner)->info->status == BUDDY_STATUS_OFFLINE) || (!statusButton && self->lastStatus == BUDDY_STATUS_OFFLINE))
			self->menuCallBack(w2, MENU_STATUS, BUDDY_STATUS_ONLINE, self->menuCallBackData);
		else
			self->menuCallBack(w2, MENU_STATUS, BUDDY_STATUS_OFFLINE, self->menuCallBackData);
			
		gtk_widget_destroy(w1);
	}
	
	return FALSE;
}



gboolean mainWindow::cb_notifyBoxClicked(GtkWidget* box, GdkEventButton* event, mainWindow* mw)
{
	if (event->type == GDK_2BUTTON_PRESS && mw->menuCallBack)
		mw->menuCallBack(NULL, MENU_NOTIFYBOX, 0, mw->menuCallBackData);
	return FALSE;
}


void mainWindow::cb_mainWindowRealized(GtkWidget* w, mainWindow* self)
{
	gboolean	dockWindow;
	gulong		dockMode;
	
	settings_getSettings()->getProperties("mainwindow",
										  "dockWindow", &dockWindow,
										  "dockMode", &dockMode,
										  NULL);
	

	if (dockWindow)
		u_dockWindow(self->window, STRUT_LEFT);
}




void mainWindow::cb_settingsChangedCallback(gchar* sname, GList *values, mainWindow* self)
{
	settingsProperty *prop;
	gboolean		 updateSortRules;
	GList			 *children,
					 *search,
					 *searchCopy;
	IMUserDaemon	 *user;
	statusButtonType *but;
	contactListUser	 *contactUser;
	iconManager		 *iconMan;
	gint			 windowX,
					 windowY;
	
	
	if (!strcmp(sname, "appearance"))
	{
		while(values)
		{
			prop = (settingsProperty*)values->data;
			
			// was the status theme changed? update all icons / status button images ...
			if (!strcmp(prop->name, "StatusTheme"))
			{
				iconMan = i_getIcons();
		
				// update the contact list icons			
				children = self->contacts->getAllChildrenOfType(CE_USER);
			
				search = children;
				while(search)
				{
					contactUser = (contactListUser*)search->data;
					user = (IMUserDaemon*)contactUser->data;
				
					contactUser->setStatus(user->info->status, iconMan->getUserStatusPixbuf(user));
				
					search = search->next;
				}
				g_list_free(children);
			
				// update the status buttons
				search = self->statusButtons;
				while(search)
				{
					but = (statusButtonType*)search->data;
					self->updateStatusButton((IMOwnerDaemon*)but->owner, ((IMOwnerDaemon*)but->owner)->info->status, ((IMOwnerDaemon*)but->owner)->info->isInvisible);
				
					search = search->next;
				}
			
				self->updateStatus(self->lastStatus, self->lastInvisible);	

				break;				
			}
			
			values = values->next;
		}
		
	} else
	if (!strcmp(sname, "contactlist"))
	{
		while(values)
		{
			prop = (settingsProperty*)values->data;
		
			if (!strcmp(prop->name, "showGroups"))
				gtk_widget_set_sensitive(self->menu_showEmptyGroups, g_value_get_boolean(prop->val));
			else		
			if (!strcmp(prop->name, "showTooltips"))
			{
				if (g_value_get_boolean(prop->val))
					self->tooltips->attachWidget(self->contacts->widget, (UTWidgetFunc)self->tooltipFunction, self);
				else
					self->tooltips->detachWidget(self->contacts->widget);
				
			} 
			
			values = values->next;
		}
	} else
	if (!strcmp(sname, "mainwindow"))
	{
		for (; values; values = values->next)
		{
			prop = (settingsProperty*)values->data;
			
			if (!strcmp(prop->name, "showMainMenu"))
			{
				if ((self->showMenu = g_value_get_boolean(prop->val)))
					self->createMainMenu();
				else
				{
					gtk_menu_item_remove_submenu(GTK_MENU_ITEM(self->menu_system));
					gtk_menu_item_remove_submenu(GTK_MENU_ITEM(self->menu_users));
					gtk_container_foreach(GTK_CONTAINER(self->mainMenuContainer), (GtkCallback)gtk_widget_destroy, NULL);
					self->mainMenu = NULL;
				}
			} else
			
			if (!strcmp(prop->name, "showNotifyBox"))
			{
				if ((self->showNotify = g_value_get_boolean(prop->val)))
					self->createNotifyBox();
				else
				{
					gtk_container_foreach(GTK_CONTAINER(self->notifyBoxContainer), (GtkCallback)gtk_widget_destroy, NULL);
					self->notifyText = NULL;
				}
			} else
			
			if (!strcmp(prop->name, "showStatusButtons"))
			{
				if ((self->showStatus = g_value_get_boolean(prop->val)))
				{
					self->createStatusBox();
					self->updateStatus(self->lastStatus, self->lastInvisible);
				}
				else
				{
					gtk_container_foreach(GTK_CONTAINER(self->statusBoxContainer), (GtkCallback)gtk_widget_destroy, NULL);
					self->statusBox = NULL;
					// remove all status buttons
					searchCopy = g_list_copy(self->statusButtons);
					for (search = searchCopy; search; search = search->next)
						self->removeStatusButton((IMOwnerDaemon*)((statusButtonType*)search->data)->owner);
					
					g_list_free(searchCopy);
				}
			} else
			
			if (!strcmp(prop->name, "showWindowBorders"))
			{
				gtk_window_get_position(GTK_WINDOW(self->window), &windowX, &windowY);
				self->showTitle = g_value_get_boolean(prop->val);
				gtk_window_set_decorated(GTK_WINDOW(self->window), g_value_get_boolean(prop->val));
				gtk_window_move(GTK_WINDOW(self->window), windowX, windowY);
			} else
			
			if (!strcmp(prop->name, "alwaysOnTop"))
				gtk_window_set_keep_above(GTK_WINDOW(self->window), g_value_get_boolean(prop->val));
			else
				
			if (!strcmp(prop->name, "showStarInTitle"))
			{
				self->titleShowStar = g_value_get_boolean(prop->val);
				self->setWindowTitle(self->lastEventState);
			} else
			
			if (!strcmp(prop->name, "showOwnersInTitle"))
			{
				self->titleShowOwners = g_value_get_boolean(prop->val);
				self->setWindowTitle(self->lastEventState);
			} else
			
			if (!strcmp(prop->name, "inTaskbarWhenMaximized") 
				&& self->window->window && !(gdk_window_get_state(self->window->window) & 2))
			{
				gtk_window_set_skip_taskbar_hint(GTK_WINDOW(self->window), !g_value_get_boolean(prop->val));
			} else
			
			if (!strcmp(prop->name, "inTaskbarWhenMinimized") 
				&& self->window->window && (gdk_window_get_state(self->window->window) & 2))
			{
				if (self->tray)
					gtk_window_set_skip_taskbar_hint(GTK_WINDOW(self->window), !g_value_get_boolean(prop->val));
			} else
			
			if (!strcmp(prop->name, "inPager"))
				gtk_window_set_skip_pager_hint(GTK_WINDOW(self->window), !g_value_get_boolean(prop->val));	
			else
			if (!strcmp(prop->name, "showWindowOnAll"))
			{
				if (g_value_get_boolean(prop->val))
					gtk_window_stick(GTK_WINDOW(self->window));
				else
					gtk_window_unstick(GTK_WINDOW(self->window));
			}
		}
	}
}




IMUserDaemon* mainWindow::tooltipFunction(GtkWidget* w, GdkEventMotion* event, mainWindow* self)
{
	contactListEntry	*entry;
	
	entry = self->contacts->getEntryAtCursor((gint)event->x, (gint)event->y);
	
	if (entry && entry->type == CE_USER)
		return (IMUserDaemon*)entry->data;
	
	return NULL;
}



void mainWindow::cb_showOfflineAccelPressed(mainWindow* self)
{
	gboolean setting;
	
	if (!self->menu_showOfflineUsers)
		return;
	
	setting = !gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(self->menu_showOfflineUsers));
	
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(self->menu_showOfflineUsers), setting);
	
	if (self->menuCallBack)
		self->menuCallBack(self->menu_showOfflineUsers, MENU_SYSTEM, MENU_SHOWOFFLINEUSERS, self->menuCallBackData);
}


void mainWindow::cb_showGroupsAccelPressed(mainWindow* self)
{
	gboolean setting;
	
	if (!self->menu_showGroups)
		return;
	
	setting = !gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(self->menu_showGroups));
	
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(self->menu_showGroups), setting);
	
	if (self->menuCallBack)
		self->menuCallBack(self->menu_showGroups, MENU_SYSTEM, MENU_SHOWGROUPS, self->menuCallBackData);
}


void mainWindow::cb_exitAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_SYSTEM, MENU_QUIT, self->menuCallBackData);
}



void mainWindow::cb_messageAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_USER, UE_MESSAGE, self->menuCallBackData);
}



void mainWindow::cb_URLAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_USER, UE_URL, self->menuCallBackData);	
}



void mainWindow::cb_chatAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_USER, UE_CHATREQUEST, self->menuCallBackData);
}



void mainWindow::cb_ftAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_USER, UE_FILETRANSFERREQUEST, self->menuCallBackData);
}



void mainWindow::cb_aResponseAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_USER, UE_AUTORESPONSEGET, self->menuCallBackData);
}



void mainWindow::cb_deleteUserFromListAccelPressed(mainWindow* self)
{
	if (self->menuCallBack)
		self->menuCallBack(NULL, MENU_USER, MENU_REMOVEUSER, self->menuCallBackData);
}
